@@ -10,11 +10,13 @@
 #undef USE_PERL
 #undef USE_PLUGIN
 #undef USE_GDK_PIXBUF
+#undef USE_XLIB
 #undef USE_PANEL
 #undef USE_MYGLIB
 #undef USE_TRANS
 #undef USE_HEBREW
 #undef USE_OPENSSL
+#undef USE_LINGER
 #undef SOCKS
 #undef PREFIX
 #undef USING_LINUX
