@@ -24,6 +24,7 @@
 #endif
 #include <string.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include "xchat.h"
 #include "cfgfiles.h"
@@ -44,13 +45,11 @@ list_addentry (GSList ** list, char *cmd, char *name)
 	struct popup *pop = malloc (sizeof (struct popup));
 	if (!pop)
 		return;
-	safe_strcpy (pop->name, name, 81);
+	safe_strcpy (pop->name, name, sizeof (pop->name));
 	if (!cmd)
 		pop->cmd[0] = 0;
 	else
-	{
-		safe_strcpy (pop->cmd, cmd, 255);
-	}
+		safe_strcpy (pop->cmd, cmd, sizeof (pop->cmd));
 	*list = g_slist_append (*list, pop);
 }
 
@@ -77,11 +76,8 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 		}
 		return;
 	}
-	if (fstat (fh, &st) != 0)
-	{
-		perror ("fstat");
-		abort ();
-	}
+
+	fstat (fh, &st);
 	ibuf = malloc (st.st_size);
 	read (fh, ibuf, st.st_size);
 	close (fh);
@@ -95,11 +91,11 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 		{
 			if (!strncasecmp (buf, "NAME ", 5))
 			{
-				safe_strcpy (name, buf + 5, 81);
+				safe_strcpy (name, buf + 5, sizeof (name));
 			}
 			if (!strncasecmp (buf, "CMD ", 4))
 			{
-				safe_strcpy (cmd, buf + 4, 255);
+				safe_strcpy (cmd, buf + 4, sizeof (cmd));
 				if (*name)
 				{
 					list_addentry (list, cmd, name);
@@ -275,6 +271,9 @@ static struct prefs vars[] = {
 	{"autodccchat", PREFS_OFFINT (autodccchat), TYPE_BOOL},
 	{"autodccsend", PREFS_OFFINT (autodccsend), TYPE_BOOL},
 	{"autodialog", PREFS_OFFINT (autodialog), TYPE_BOOL},
+	{"autoopendccchatwindow", PREFS_OFFINT (autoopendccchatwindow), TYPE_BOOL},
+	{"autoopendccrecvwindow", PREFS_OFFINT (autoopendccrecvwindow), TYPE_BOOL},
+	{"autoopendccsendwindow", PREFS_OFFINT (autoopendccsendwindow), TYPE_BOOL},
 	{"autoreconnect", PREFS_OFFINT (autoreconnect), TYPE_BOOL},
 	{"autoreconnectonfail", PREFS_OFFINT (autoreconnectonfail), TYPE_BOOL},
 	{"autorejoin", PREFS_OFFINT (autorejoin), TYPE_BOOL},
@@ -287,34 +286,35 @@ static struct prefs vars[] = {
 	{"ban_type", PREFS_OFFINT (bantype), TYPE_INT},
 	{"beep_msg", PREFS_OFFINT (beepmsg), TYPE_BOOL},
 	{"bluestring", PREFS_OFFSET (bluestring), TYPE_STR},
-	{"bt_color", PREFS_OFFINT (bt_color), TYPE_INT},
 
-	{"colorednicks", PREFS_OFFINT (colorednicks), TYPE_BOOL},
+	{"chanmodebuttons", PREFS_OFFINT (chanmodebuttons), TYPE_BOOL},
 	{"channelbox", PREFS_OFFINT (channelbox), TYPE_INT},
+	{"cmdchar", PREFS_OFFSET (cmdchar), TYPE_STR},
+	{"colorednicks", PREFS_OFFINT (colorednicks), TYPE_BOOL},
 	{"ctcp_number_limit", PREFS_OFFINT (ctcp_number_limit), TYPE_INT},
 	{"ctcp_time_limit", PREFS_OFFINT (ctcp_time_limit), TYPE_INT},
 
 	{"dcc_blocksize", PREFS_OFFINT (dcc_blocksize), TYPE_INT},
 	{"dcc_permissions", PREFS_OFFINT (dccpermissions), TYPE_INT},
-	{"dcc_timeout", PREFS_OFFINT (dcctimeout), TYPE_INT},
 	{"dcc_stall_timeout", PREFS_OFFINT (dccstalltimeout), TYPE_INT},
+	{"dcc_timeout", PREFS_OFFINT (dcctimeout), TYPE_INT},
 	{"dccdir", PREFS_OFFSET (dccdir), TYPE_STR},
 	{"dccwithnick", PREFS_OFFINT (dccwithnick), TYPE_BOOL},
 	{"dialog_indent_nicks", PREFS_OFFINT (dialog_indent_nicks), TYPE_BOOL},
 	{"dialog_indent_pixels", PREFS_OFFINT (dialog_indent_pixels), TYPE_INT},
 	{"dialog_show_separator", PREFS_OFFINT (dialog_show_separator), TYPE_BOOL},
 	{"dialog_tint", PREFS_OFFINT (dialog_tint), TYPE_BOOL},
-	{"dialog_tint_red", PREFS_OFFINT (dialog_tint_red), TYPE_INT},
-	{"dialog_tint_green", PREFS_OFFINT (dialog_tint_green), TYPE_INT},
 	{"dialog_tint_blue", PREFS_OFFINT (dialog_tint_blue), TYPE_INT},
+	{"dialog_tint_green", PREFS_OFFINT (dialog_tint_green), TYPE_INT},
+	{"dialog_tint_red", PREFS_OFFINT (dialog_tint_red), TYPE_INT},
 	{"dialog_transparent", PREFS_OFFINT (dialog_transparent), TYPE_BOOL},
 	{"dialog_wordwrap", PREFS_OFFINT (dialog_wordwrap), TYPE_BOOL},
 	{"dnsprogram", PREFS_OFFSET (dnsprogram), TYPE_STR},
-	{"double_buffer", PREFS_OFFINT (double_buffer), TYPE_BOOL},
 	{"doubleclickuser", PREFS_OFFSET (doubleclickuser), TYPE_STR},
 
 	{"fastdccsend", PREFS_OFFINT (fastdccsend), TYPE_BOOL},
 	{"filterbeep", PREFS_OFFINT (filterbeep), TYPE_BOOL},
+	{"first_dcc_send_port", PREFS_OFFINT (first_dcc_send_port), TYPE_INT},
 	{"font_dialog_normal", PREFS_OFFSET (dialog_font_normal), TYPE_STR},
 	{"font_normal", PREFS_OFFSET (font_normal), TYPE_STR},
 	{"fudgeservernotice", PREFS_OFFINT (fudgeservernotice), TYPE_BOOL},
@@ -326,6 +326,7 @@ static struct prefs vars[] = {
 	{"hidemenu", PREFS_OFFINT (hidemenu), TYPE_BOOL},
 	{"hideuserlist", PREFS_OFFINT (hideuserlist), TYPE_BOOL},
 	{"hilight_notify", PREFS_OFFINT (hilitenotify), TYPE_BOOL},
+	{"hilightnick", PREFS_OFFINT (hilightnick), TYPE_BOOL},
 	{"host_in_userlist", PREFS_OFFINT(showhostname_in_userlist), TYPE_BOOL},
 	{"hostname", PREFS_OFFSET (hostname), TYPE_STR},
 
@@ -336,6 +337,7 @@ static struct prefs vars[] = {
 	{"ip_from_server", PREFS_OFFINT (ip_from_server), TYPE_BOOL},
 
 	{"lagometer", PREFS_OFFINT (lagometer), TYPE_BOOL},
+	{"last_dcc_send_port", PREFS_OFFINT (last_dcc_send_port), TYPE_INT},
 	{"limitedtabhighlight", PREFS_OFFINT (limitedtabhighlight), TYPE_BOOL},
 	{"logging", PREFS_OFFINT (logging), TYPE_BOOL},
 	{"logmask", PREFS_OFFSET (logmask), TYPE_STR},
@@ -357,22 +359,18 @@ static struct prefs vars[] = {
 	{"nickname1", PREFS_OFFSET (nick1), TYPE_STR},
 	{"nickname2", PREFS_OFFSET (nick2), TYPE_STR},
 	{"nickname3", PREFS_OFFSET (nick3), TYPE_STR},
-	{"noautoopendccchatwindow", PREFS_OFFINT (noautoopendccchatwindow), TYPE_BOOL},
-	{"noautoopendccrecvwindow", PREFS_OFFINT (noautoopendccrecvwindow), TYPE_BOOL},
-	{"noautoopendccsendwindow", PREFS_OFFINT (noautoopendccsendwindow), TYPE_BOOL},
-	{"nochanmodebuttons", PREFS_OFFINT (nochanmodebuttons), TYPE_BOOL},
-	{"nopaned", PREFS_OFFINT (nopaned), TYPE_BOOL},
 	{"notify_timeout", PREFS_OFFINT (notify_timeout), TYPE_INT},
-	{"nouserhost", PREFS_OFFINT (nouserhost), TYPE_BOOL},
-	{"nouserlistbuttons", PREFS_OFFINT (nouserlistbuttons), TYPE_BOOL},
 	{"nu_color", PREFS_OFFINT (nu_color), TYPE_INT},
 
 	{"old_nickcompletion", PREFS_OFFINT (old_nickcompletion), TYPE_BOOL},
 
+	{"paned_userlist", PREFS_OFFINT (paned_userlist), TYPE_BOOL},
 #ifdef USE_PANEL
 	{"panel_vbox", PREFS_OFFINT (panel_vbox), TYPE_BOOL},
 	{"panelize_hide", PREFS_OFFINT (panelize_hide), TYPE_BOOL},
 #endif
+	{"partreason", PREFS_OFFSET (partreason), TYPE_STR},
+	{"percascii", PREFS_OFFINT (perc_ascii), TYPE_BOOL},
 #ifdef USE_PERL
 	{"perlwarnings", PREFS_OFFINT (perlwarnings), TYPE_BOOL},
 #endif
@@ -384,9 +382,6 @@ static struct prefs vars[] = {
 
 	{"quitreason", PREFS_OFFSET (quitreason), TYPE_STR},
 
-	{"first_dcc_send_port", PREFS_OFFINT (first_dcc_send_port), TYPE_INT},
-	{"last_dcc_send_port", PREFS_OFFINT (last_dcc_send_port), TYPE_INT},
-
 	{"raw_modes", PREFS_OFFINT (raw_modes), TYPE_BOOL},
 	{"realname", PREFS_OFFSET (realname), TYPE_STR},
 	{"reconnect_delay", PREFS_OFFINT (recon_delay), TYPE_INT},
@@ -405,18 +400,19 @@ static struct prefs vars[] = {
 	{"soundcmd", PREFS_OFFSET (soundcmd), TYPE_STR},
 	{"sounddir", PREFS_OFFSET (sounddir), TYPE_STR},
 	{"stripcolor", PREFS_OFFINT (stripcolor), TYPE_BOOL},
-	{"style_namelistgad", PREFS_OFFINT (style_namelistgad), TYPE_BOOL},
 	{"style_inputbox", PREFS_OFFINT (style_inputbox), TYPE_BOOL},
+	{"style_namelistgad", PREFS_OFFINT (style_namelistgad), TYPE_BOOL},
 
 	{"tabchannels", PREFS_OFFINT (tabchannels), TYPE_BOOL},
 	{"tabs_position", PREFS_OFFINT (tabs_position), TYPE_INT},
 	{"thin_separator", PREFS_OFFINT (thin_separator), TYPE_BOOL},
 	{"throttle", PREFS_OFFINT (throttle), TYPE_BOOL},
 	{"timestamp", PREFS_OFFINT (timestamp), TYPE_BOOL},
+	{"timestamp_logs", PREFS_OFFINT (timestamp_logs), TYPE_BOOL},
 	{"tint", PREFS_OFFINT (tint), TYPE_BOOL},
-	{"tint_red", PREFS_OFFINT (tint_red), TYPE_INT},
-	{"tint_green", PREFS_OFFINT (tint_green), TYPE_INT},
 	{"tint_blue", PREFS_OFFINT (tint_blue), TYPE_INT},
+	{"tint_green", PREFS_OFFINT (tint_green), TYPE_INT},
+	{"tint_red", PREFS_OFFINT (tint_red), TYPE_INT},
 	{"topicbar", PREFS_OFFINT (topicbar), TYPE_BOOL},
 	{"transparent", PREFS_OFFINT (transparent), TYPE_BOOL},
 #ifdef USE_TRANS
@@ -429,7 +425,9 @@ static struct prefs vars[] = {
 #ifdef USE_TRANS
 	{"use_trans", PREFS_OFFINT (use_trans), TYPE_BOOL},
 #endif
+	{"userhost", PREFS_OFFINT (userhost), TYPE_BOOL},
 	{"userlist_sort", PREFS_OFFINT (userlist_sort), TYPE_INT},
+	{"userlistbuttons", PREFS_OFFINT (userlistbuttons), TYPE_BOOL},
 	{"username", PREFS_OFFSET (username), TYPE_STR},
 
 	{"wallops", PREFS_OFFINT (wallops), TYPE_BOOL},
@@ -446,18 +444,17 @@ load_config (void)
 	struct stat st;
 	char *cfg, *username;
 	int res, val, i, fh;
-/*   char version[32];*/
 
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 	/* Just for ppl upgrading --AGL */
 	strcpy (prefs.logmask, "%s,%c.xchatlog");
 	strcpy (prefs.nick_suffix, ":");
+	strcpy (prefs.cmdchar, "/");
 	prefs.auto_indent = 1;
-	prefs.max_auto_indent = 112;
+	prefs.max_auto_indent = 256;
 	prefs.mail_check = 1;
 	prefs.show_separator = 1;
 	prefs.dialog_show_separator = 1;
-	prefs.double_buffer = 1;
 	prefs.dcc_blocksize = 1024;
 	prefs.throttle = 1;
 	 /*FIXME*/ prefs.msg_time_limit = 30;
@@ -466,8 +463,14 @@ load_config (void)
 	prefs.ctcp_number_limit = 5;
 	prefs.topicbar = 1;
 	prefs.lagometer = 1;
-	prefs.pingtimeout = 90;
 	prefs.nickgad = 1;
+	prefs.autoopendccrecvwindow = 1;
+	prefs.autoopendccsendwindow = 1;
+	prefs.autoopendccchatwindow = 1;
+	prefs.chanmodebuttons = 1;
+	prefs.userhost = 1;
+	prefs.userlistbuttons = 1;
+	prefs.hilightnick = 1;
 
 	check_prefs_dir ();
 	username = g_get_user_name ();
@@ -501,18 +504,6 @@ load_config (void)
 		}
 		while (vars[i].type != 0);
 
-/*      cfg_get_str (cfg, "version", version);
-      if (strcmp (version, VERSION) != 0)
-      {
-         fe_message (
-"Welcome to X-Chat 1.4!\n\n"
-"Alot of changes have been made since 1.2. You may want to delete\n"
-"your ~/.xchat directory and start with the new defaults (saving\n"
-"your serverlist.conf might be a good idea). Make sure you do it\n"
-"while X-Chat isn't running.\n"
-                  , TRUE);
-      }*/
-
 		free (cfg);
 
 	} else
@@ -533,7 +524,6 @@ load_config (void)
 							"home dir, you should change this at some stage.", TRUE);
 #endif
 		/* put in default values, anything left out is automatically zero */
-		prefs.noautoopendccchatwindow = 1;
 		prefs.autoresume = 1;
 		prefs.show_away_once = 1;
 		prefs.show_away_message = 1;
@@ -543,11 +533,14 @@ load_config (void)
 		prefs.dialog_indent_nicks = 1;
 		prefs.thin_separator = 1;
 		prefs.fastdccsend = 1;
+		prefs.wordwrap = 1;
+		prefs.dialog_wordwrap = 1;
 		prefs.autosave = 1;
 		prefs.autodialog = 1;
 		prefs.autoreconnect = 1;
-		prefs.recon_delay = 3;
+		prefs.recon_delay = 10;
 		prefs.tabchannels = 1;
+		prefs.newtabstofront = 1;
 		prefs.use_server_tab = 1;
 		prefs.privmsgtab = 1;
 		prefs.nickcompletion = 1;
@@ -555,21 +548,19 @@ load_config (void)
 		prefs.show_invite_in_front_session = 1;
 		prefs.show_notify_in_front_session = 1;
 		prefs.whois_on_notifyonline = 1;
-		prefs.inputgad_superfocus = 1;
 		prefs.nu_color = 4;
-		prefs.bt_color = 8;
 		prefs.dccpermissions = 0600;
 		prefs.max_lines = 300;
-		prefs.mainwindow_width = 601;
+		prefs.mainwindow_width = 630;
 		prefs.mainwindow_height = 422;
-		prefs.dcctimeout = 240;
-		prefs.dccstalltimeout = 180;
+		prefs.dcctimeout = 180;
+		prefs.dccstalltimeout = 60;
 		prefs.notify_timeout = 15;
 		prefs.tint_red =
 			prefs.tint_green =
 			prefs.tint_blue =
 			prefs.dialog_tint_red =
-			prefs.dialog_tint_green = prefs.dialog_tint_blue = 208;
+			prefs.dialog_tint_green = prefs.dialog_tint_blue = 195;
 		strcpy (prefs.nick1, username);
 		strcpy (prefs.nick2, username);
 		strcat (prefs.nick2, "_");
@@ -580,7 +571,8 @@ load_config (void)
 		strcpy (prefs.dccdir, g_get_home_dir ());
 		strcpy (prefs.doubleclickuser, "/QUOTE WHOIS %s");
 		strcpy (prefs.awayreason, "I'm busy");
-		strcpy (prefs.quitreason, "I'm too lame to make a quit message");
+		strcpy (prefs.quitreason, "Client Exiting");
+		strcpy (prefs.partreason, prefs.quitreason);
 		strcpy (prefs.font_normal,
 				  "-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-*-*");
 		strcpy (prefs.dialog_font_normal, prefs.font_normal);
@@ -588,14 +580,10 @@ load_config (void)
 		strcpy (prefs.soundcmd, "esdplay");
 		strcpy (prefs.dnsprogram, "host");
 	}
-	if (prefs.mainwindow_height < 106)
-		prefs.mainwindow_height = 106;
-	if (prefs.mainwindow_width < 116)
-		prefs.mainwindow_width = 116;
-	if (prefs.indent_pixels < 1)
-		prefs.indent_pixels = 80;
-	if (prefs.dialog_indent_pixels < 1)
-		prefs.dialog_indent_pixels = 80;
+	if (prefs.mainwindow_height < 138)
+		prefs.mainwindow_height = 138;
+	if (prefs.mainwindow_width < 106)
+		prefs.mainwindow_width = 106;
 }
 
 int
@@ -631,6 +619,40 @@ save_config (void)
 	return 1;
 }
 
+static void
+set_showval (session *sess, struct prefs *var, char *tbuf)
+{
+	int len, dots, j;
+	static char *offon[] = { "OFF", "ON" };
+
+	len = strlen (var->name);
+	memcpy (tbuf, var->name, len);
+	dots = 29 - len;
+	if (dots < 0)
+		dots = 0;
+	tbuf[len++] = '\003';
+	tbuf[len++] = '2';
+	for (j=0;j<dots;j++)
+		tbuf[j + len] = '.';
+	len += j;
+	switch (var->type)
+	{
+	case TYPE_STR:
+		sprintf (tbuf + len, "\0033:\017 %s\n",
+					(char *) &prefs + var->offset);
+		break;
+	case TYPE_INT:
+		sprintf (tbuf + len, "\0033:\017 %d\n",
+					*((int *) &prefs + var->offset));
+		break;
+	case TYPE_BOOL:
+		sprintf (tbuf + len, "\0033:\017 %s\n", offon[
+					*((int *) &prefs + var->offset)]);
+		break;
+	}
+	PrintText (sess, tbuf);
+}
+
 static void
 set_list (session * sess, char *tbuf)
 {
@@ -639,19 +661,7 @@ set_list (session * sess, char *tbuf)
 	i = 0;
 	do
 	{
-		switch (vars[i].type)
-		{
-		case TYPE_STR:
-			sprintf (tbuf, "%s \0033=\017 %s\n", vars[i].name,
-						(char *) &prefs + vars[i].offset);
-			break;
-		case TYPE_INT:
-		case TYPE_BOOL:
-			sprintf (tbuf, "%s \0033=\017 %d\n", vars[i].name,
-						*((int *) &prefs + vars[i].offset));
-			break;
-		}
-		PrintText (sess, tbuf);
+		set_showval (sess, &vars[i], tbuf);
 		i++;
 	}
 	while (vars[i].type != 0);
@@ -712,12 +722,11 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				{
 					strcpy ((char *) &prefs + vars[i].offset, val);
 					sprintf (tbuf, "%s set to: %s\n", var, val);
+					PrintText (sess, tbuf);
 				} else
 				{
-					sprintf (tbuf, "%s (string) has value: %s\n", vars[i].name,
-								(char *) &prefs + vars[i].offset);
+					set_showval (sess, &vars[i], tbuf);
 				}
-				PrintText (sess, tbuf);
 				break;
 			case TYPE_INT:
 			case TYPE_BOOL:
@@ -729,22 +738,21 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 							*((int *) &prefs + vars[i].offset) = 1;
 						else
 							*((int *) &prefs + vars[i].offset) = 0;
+						if (!strcasecmp (val, "YES") || !strcasecmp (val, "ON"))
+							*((int *) &prefs + vars[i].offset) = 1;
+						if (!strcasecmp (val, "NO") || !strcasecmp (val, "OFF"))
+							*((int *) &prefs + vars[i].offset) = 0;
 					} else
 					{
 						*((int *) &prefs + vars[i].offset) = atoi (val);
 					}
 					sprintf (tbuf, "%s set to: %d\n", var,
 								*((int *) &prefs + vars[i].offset));
+					PrintText (sess, tbuf);
 				} else
 				{
-					if (vars[i].type == TYPE_BOOL)
-						sprintf (tbuf, "%s (boolean) has value: %d\n", vars[i].name,
-									*((int *) &prefs + vars[i].offset));
-					else
-						sprintf (tbuf, "%s (number) has value: %d\n", vars[i].name,
-									*((int *) &prefs + vars[i].offset));
+					set_showval (sess, &vars[i], tbuf);
 				}
-				PrintText (sess, tbuf);
 				break;
 			}
 		}
