@@ -9,6 +9,8 @@ char *get_xdir (void);
 void check_prefs_dir (void);
 void load_config (void);
 int save_config (void);
+void list_free (GSList ** list);
+void list_loadconf (char *file, GSList ** list, char *defaultconf);
 
 #define STRUCT_OFFSET_STR(type,field) \
 ( (unsigned int) (((char *) (&(((type *) NULL)->field)))- ((char *) NULL)) )
