@@ -124,9 +124,7 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 			EMIT_SIGNAL (XP_TE_CTCPSND, sess->server->front_session, word[5],
 							 nick, NULL, NULL, 0);
 			sprintf (outbuf, "%s/%s", prefs.sounddir, word[5]);
-			if (	/*strchr (word[5], '`') == 0 &&*/
-					strchr (word[5], '/') == 0 &&
-					access (outbuf, R_OK) == 0)
+			if (strchr (word[5], '/') == 0 && access (outbuf, R_OK) == 0)
 			{
 				sprintf (outbuf, "%s %s/%s", prefs.soundcmd, prefs.sounddir,
 							word[5]);
