@@ -16,6 +16,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
  * Wayne Conrad, 3 Apr 1999: Color-coded DCC file transfer status windows
+ * Bernhard Valenti <bernhard.valenti@gmx.net> 2000-11-21: Fixed DCC send behind nat
  */
 
 #include <stdio.h>
@@ -33,20 +34,9 @@
 #include "plugin.h"
 #include "fe.h"
 
-/* trans */
-extern void serv2user (unsigned char *s);
-extern void user2serv (unsigned char *s);
-/* ~trans */
+char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
 
-static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
-
-struct dccstat_info
-{
-	char *name;						  /* Display name */
-	int color;						  /* Display color (index into colors[] ) */
-};
-
-static struct dccstat_info dccstat[] = {
+struct dccstat_info dccstat[] = {
 	{"Queued", 1 /*black */ },
 	{"Active", 12 /*cyan */ },
 	{"Failed", 4 /*red */ },
@@ -55,11 +45,9 @@ static struct dccstat_info dccstat[] = {
 	{"Aborted", 4 /*red */ },
 };
 
-
 extern GSList *sess_list;
 extern GSList *serv_list;
 extern GSList *dcc_list;
-
 extern struct xchatprefs prefs;
 
 extern int tcp_send_len (struct server *serv, char *buf, int len);
@@ -72,10 +60,13 @@ extern void channel_action (struct session *sess, char *tbuf, char *chan,
 									 char *from, char *text, int fromme);
 extern struct session *find_session_from_channel (char *chan,
 																  struct server *serv);
+/* trans */
+extern void serv2user (unsigned char *s);
+extern void user2serv (unsigned char *s);
+/* ~trans */
 
 #ifdef USE_PERL
 /* perl.c */
-
 int perl_dcc_chat (struct session *sess, struct server *serv, char *buf);
 #endif
 
@@ -180,7 +171,7 @@ dcc_connect_sok (struct DCC *dcc)
 	return sok;
 }
 
-void
+static void
 update_dcc_window (int type)
 {
 	switch (type)
@@ -418,9 +409,9 @@ dcc_read_chat (struct DCC *dcc, gint sok)
 static void
 dcc_read (struct DCC *dcc, gint sok)
 {
+	char buf[4096];
 	guint32 pos;
 	int n;
-	char buf[4098];
 
 	if (dcc->fp == -1)
 	{
@@ -459,7 +450,7 @@ dcc_read (struct DCC *dcc, gint sok)
 	}
 	while (1)
 	{
-		n = recv (dcc->sok, buf, sizeof (buf) - 2, 0);
+		n = recv (dcc->sok, buf, sizeof (buf), 0);
 		if (n < 1)
 		{
 			if (n < 0)
@@ -473,25 +464,22 @@ dcc_read (struct DCC *dcc, gint sok)
 			return;
 		}
 
-		if (n > 0)
+		write (dcc->fp, buf, n);
+		dcc->pos += n;
+		pos = htonl (dcc->pos);
+		send (dcc->sok, (char *) &pos, 4, 0);
+
+		dcc->lasttime = time (0);
+
+		dcc_calc_cps (dcc);
+
+		if (dcc->pos >= dcc->size)
 		{
-			write (dcc->fp, buf, n);
-			dcc->pos += n;
-			pos = htonl (dcc->pos);
-			send (dcc->sok, (char *) &pos, 4, 0);
-
-			dcc->lasttime = time (0);
-
-			dcc_calc_cps (dcc);
-
-			if (dcc->pos >= dcc->size)
-			{
-				sprintf (buf, "%d", dcc->cps);
-				dcc_close (dcc, STAT_DONE, FALSE);
-				EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
-								 dcc->file, dcc->destfile, dcc->nick, buf, 0);
-				return;
-			}
+			sprintf (buf, "%d", dcc->cps);
+			dcc_close (dcc, STAT_DONE, FALSE);
+			EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
+							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
+			return;
 		}
 	}
 }
@@ -543,8 +531,8 @@ dcc_connect_finished (struct DCC *dcc, gint sok)
 					 dcctypes[(int) dcc->type], dcc->nick, host, "to", 0);
 }
 
-void
-dcc_connect (struct session *sess, struct DCC *dcc)
+static void
+dcc_connect (struct DCC *dcc)
 {
 	if (dcc->dccstat == STAT_CONNECTING)
 		return;
@@ -557,7 +545,10 @@ dcc_connect (struct session *sess, struct DCC *dcc)
 		return;
 	}
 	dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
-	fe_dcc_update_recv (dcc);
+	if (dcc->type == TYPE_RECV)
+		fe_dcc_update_recv (dcc);
+	else
+		fe_dcc_update_chat_win ();
 }
 
 static void
@@ -572,31 +563,34 @@ dcc_send_data (struct DCC *dcc, gint sok)
 	if (prefs.dcc_blocksize > 102400)	/* this is too much! */
 		prefs.dcc_blocksize = 102400;
 
-	buf = malloc (prefs.dcc_blocksize | 32);	/* at least 32 bytes for the sprintf */
+	buf = malloc (prefs.dcc_blocksize);
 	if (!buf)
 		return;
 
-	if (dcc->pos < dcc->size)
-	{
-		lseek (dcc->fp, dcc->pos, SEEK_SET);
-		len = read (dcc->fp, buf, prefs.dcc_blocksize);
-		sent = send (sok, buf, len, 0);
+	lseek (dcc->fp, dcc->pos, SEEK_SET);
+	len = read (dcc->fp, buf, prefs.dcc_blocksize);
+	if (len < 0)
+		goto abortit;
+	sent = send (sok, buf, len, 0);
 
-		if (sent < 0 && errno != EWOULDBLOCK)
-		{
-			EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-							 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
-			dcc_close (dcc, STAT_FAILED, FALSE);
-			free (buf);
-			return;
-		}
-		if (sent > 0)
-		{
-			dcc->pos += sent;
-			dcc->lasttime = time (0);
-		}
+	if (sent < 0 && errno != EWOULDBLOCK)
+	{
+abortit:
+		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
+						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
+		dcc_close (dcc, STAT_FAILED, FALSE);
+		free (buf);
+		return;
+	}
+	if (sent > 0)
+	{
+		dcc->pos += sent;
+		dcc->lasttime = time (0);
 		dcc_calc_cps (dcc);
-	} else
+	}
+
+	/* have we sent it all yet? */
+	if (dcc->pos >= dcc->size)
 	{
 		/* it's all sent now, so remove the WRITE/SEND handler */
 		if (dcc->wiotag != -1)
@@ -606,13 +600,6 @@ dcc_send_data (struct DCC *dcc, gint sok)
 		}
 	}
 
-	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
-	{
-		dcc_close (dcc, STAT_DONE, FALSE);
-		sprintf (buf, "%d", dcc->cps);
-		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
-	}
 	free (buf);
 }
 
@@ -652,15 +639,14 @@ dcc_read_ack (struct DCC *dcc, gint sok)
 		if (dcc->ack < dcc->pos)
 			return;
 		dcc_send_data (dcc, sok);
-	} else
+	}
+
+	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
 	{
-		if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
-		{
-			dcc_close (dcc, STAT_DONE, FALSE);
-			sprintf (buf, "%d", dcc->cps);
-			EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
-							 file_part (dcc->file), dcc->nick, buf, NULL, 0);
-		}
+		dcc_close (dcc, STAT_DONE, FALSE);
+		sprintf (buf, "%d", dcc->cps);
+		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
+						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
 	}
 }
 
@@ -678,6 +664,8 @@ dcc_accept (struct DCC *dcc, gint sokk)
 	close (dcc->sok);
 	if (sok < 0)
 	{
+		dcc->iotag = -1;
+		dcc->sok = -1;
 		dcc_close (dcc, STAT_FAILED, FALSE);
 		return;
 	}
@@ -714,17 +702,16 @@ dcc_accept (struct DCC *dcc, gint sokk)
 }
 
 static int
-dcc_listen_init (struct DCC *dcc)
+dcc_listen_init (struct DCC *dcc, session *sess)
 {
 	size_t len;
 	unsigned long my_addr;
-	unsigned long old_addr;
 	struct sockaddr_in SAddr;
-	int bindretval = -1;
+	int i, bindretval = -1;
 
 	dcc->sok = socket (AF_INET, SOCK_STREAM, 0);
 	if (dcc->sok == -1)
-		return 0;
+		return FALSE;
 
 	memset (&SAddr, 0, sizeof (struct sockaddr_in));
 
@@ -732,73 +719,59 @@ dcc_listen_init (struct DCC *dcc)
 	getsockname (dcc->serv->sok, (struct sockaddr *) &SAddr, &len);
 
 	SAddr.sin_family = AF_INET;
-	SAddr.sin_port = 0;
 
-	if (prefs.dcc_ip == 0)
+	/*if local_ip is specified use that*/
+	if (prefs.local_ip != 0)
+	{
 		my_addr = prefs.local_ip;
-	else
-		my_addr = prefs.dcc_ip;
-
-	if (my_addr != 0)
-	{
-		/* store the current adress if my_addr fails */
-		old_addr = SAddr.sin_addr.s_addr;
-		SAddr.sin_addr.s_addr = my_addr;
+		SAddr.sin_addr.s_addr = prefs.local_ip;
 	}
+	/*otherwise use the default*/
+	else
+		my_addr = SAddr.sin_addr.s_addr;
 
-	if(prefs.first_dcc_send_port > 0)
+	/*if we have a valid portrange try to use that*/
+	if (prefs.first_dcc_send_port > 0)
 	{
-		int i = 0;
-		while((prefs.last_dcc_send_port > ntohs(SAddr.sin_port)) && (bindretval == -1))
+		SAddr.sin_port = 0;
+		i = 0;
+		while ((prefs.last_dcc_send_port > ntohs(SAddr.sin_port)) &&
+				(bindretval == -1))
 		{
-			SAddr.sin_port = htons(prefs.first_dcc_send_port+i++);
+			SAddr.sin_port = htons (prefs.first_dcc_send_port+i++);
 			/*printf("Trying to bind against port: %d\n",ntohs(SAddr.sin_port));*/
-			if ((bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr))) == -1)
-			{
-				if(my_addr != 0)
-				{
-					/* try the old adress with same port... */
-					SAddr.sin_addr.s_addr = old_addr;
-					/*printf("Trying to bind against port: %d\n",ntohs(SAddr.sin_port));*/
-					if((bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr))) == -1)
-					{
-						/* try my_addr again for the next port */
-						SAddr.sin_addr.s_addr = my_addr;
-					}
-				}
-			}
-		}
-	}
-	else
-	{
-		if ((bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr))) == -1)
-		{
-			if(my_addr != 0)
-			{
-				/* try the old adress... */
-				SAddr.sin_addr.s_addr = old_addr;
-				bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
-			}
+			bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 		}
 	}
 
-	if(bindretval == -1)
+	/*if we didnt bind yet, try a random port*/
+	if (bindretval == -1)
 	{
-		/* Bind failed, maybe do something about that? */
-		/*perror("Bind failed: ");*/
+		SAddr.sin_port = 0;
+		bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
+	}
+
+	if (bindretval == -1)
+	{
+		/* failed to bind */
+		PrintText (sess, "Failed to bind to any address or port.\n");
+		return FALSE;
 	}
 
 	len = sizeof (SAddr);
 	getsockname (dcc->sok, (struct sockaddr *) &SAddr, &len);
 
-	memcpy ((char *) &dcc->SAddr, (char *) &SAddr,
-			  sizeof (struct sockaddr_in));
+	memcpy ((char *) &dcc->SAddr, (char *) &SAddr, sizeof (struct sockaddr_in));
 	dcc->port = ntohs (SAddr.sin_port);
-	if (my_addr != 0)
-		dcc->addr = ntohl (my_addr);
-	else
-		dcc->addr = ntohl (SAddr.sin_addr.s_addr);
 
+	/*if we have a dcc_ip, we use that, so the remote client can connect*/
+	/*we would tell the client to connect to our LAN ip otherwise*/
+	if (prefs.dcc_ip == 0)
+		dcc->addr = ntohl (my_addr);
+	/*else we use the one we bound to*/
+	else
+		dcc->addr = ntohl (prefs.dcc_ip);
+	
 	fcntl (dcc->sok, F_SETFL, O_NONBLOCK);
 	listen (dcc->sok, 1);
 	fcntl (dcc->sok, F_SETFL, 0);
@@ -818,6 +791,8 @@ dcc_send_wild (char *file)
 	dcc_send (dccsess, dcctbuf, dccto, file);
 }
 
+/* tbuf is at least 400 bytes */
+
 void
 dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 {
@@ -857,7 +832,7 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 				dcc->fp = open (dcc->file, OFLAGS | O_RDONLY);
 				if (dcc->fp != -1)
 				{
-					if (dcc_listen_init (dcc))
+					if (dcc_listen_init (dcc, sess))
 					{
 						file = dcc->file;
 						while (*file)
@@ -867,16 +842,15 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 							file++;
 						}
 						dcc->nick = strdup (to);
-						if (!prefs.noautoopendccsendwindow)
+						if (prefs.autoopendccsendwindow)
 							fe_dcc_open_send_win ();
 						else
 							fe_dcc_update_send_win ();
-						snprintf (tbuf, 255,
+						snprintf (tbuf, 400,
 									 "PRIVMSG %s :\001DCC SEND %s %lu %d %d\001\r\n",
 									 to, file_part (dcc->file), dcc->addr, dcc->port,
 									 dcc->size);
 						tcp_send (sess->server, tbuf);
-
 						EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
 										 to, NULL, NULL, 0);
 					} else
@@ -888,7 +862,7 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 			}
 		}
 	}
-	snprintf (tbuf, 255, "Cannot access %s\n", dcc->file);
+	snprintf (tbuf, 400, "Cannot access %s\n", dcc->file);
 	PrintText (sess, tbuf);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -961,21 +935,21 @@ dcc_change_nick (struct server *serv, char *oldnick, char *newnick)
 void
 dcc_get (struct DCC *dcc)
 {
-	if (dcc)
+	switch (dcc->dccstat)
 	{
-		switch (dcc->dccstat)
+	case STAT_QUEUED:
+		if (dcc->type != TYPE_CHATSEND)
 		{
-		case STAT_QUEUED:
 			dcc->resumable = 0;
 			dcc->pos = 0;
-			dcc_connect (0, dcc);
-			break;
-		case STAT_DONE:
-		case STAT_FAILED:
-		case STAT_ABORTED:
-			dcc_close (dcc, 0, TRUE);
-			break;
+			dcc_connect (dcc);
 		}
+		break;
+	case STAT_DONE:
+	case STAT_FAILED:
+	case STAT_ABORTED:
+		dcc_close (dcc, 0, TRUE);
+		break;
 	}
 }
 
@@ -994,7 +968,7 @@ dcc_get_nick (struct session *sess, char *nick)
 				dcc->resumable = 0;
 				dcc->pos = 0;
 				dcc->ack = 0;
-				dcc_connect (sess, dcc);
+				dcc_connect (dcc);
 				return;
 			}
 		}
@@ -1021,7 +995,7 @@ new_dcc (void)
 void
 dcc_chat (struct session *sess, char *nick)
 {
-	char outbuf[256];
+	char outbuf[400];
 	struct DCC *dcc;
 
 	dcc = find_dcc (nick, "", TYPE_CHATSEND);
@@ -1045,7 +1019,7 @@ dcc_chat (struct session *sess, char *nick)
 		switch (dcc->dccstat)
 		{
 		case STAT_QUEUED:
-			dcc_connect (0, dcc);
+			dcc_connect (dcc);
 			break;
 		case STAT_FAILED:
 		case STAT_ABORTED:
@@ -1064,19 +1038,21 @@ dcc_chat (struct session *sess, char *nick)
 	dcc->type = TYPE_CHATSEND;
 	time (&dcc->starttime);
 	dcc->nick = strdup (nick);
-	if (dcc_listen_init (dcc))
+	if (dcc_listen_init (dcc, sess))
 	{
-		if (!prefs.noautoopendccchatwindow)
+		if (prefs.autoopendccchatwindow)
 			fe_dcc_open_chat_win ();
 		else
 			fe_dcc_update_chat_win ();
-		snprintf (outbuf, sizeof outbuf,
+		snprintf (outbuf, sizeof (outbuf),
 					 "PRIVMSG %s :\001DCC CHAT chat %lu %d\001\r\n", nick,
 					 dcc->addr, dcc->port);
 		tcp_send (dcc->serv, outbuf);
 		EMIT_SIGNAL (XP_TE_DCCCHATOFFERING, sess, nick, NULL, NULL, NULL, 0);
 	} else
+	{
 		dcc_close (dcc, 0, TRUE);
+	}
 }
 
 static void
@@ -1089,17 +1065,14 @@ dcc_malformed (struct session *sess, char *nick, char *data)
 void
 dcc_resume (struct DCC *dcc)
 {
-	char tbuf[256];
+	char tbuf[400];
 
-	if (dcc)
+	if (dcc->dccstat == STAT_QUEUED && dcc->resumable)
 	{
-		if (dcc->dccstat == STAT_QUEUED && dcc->resumable)
-		{
-			snprintf (tbuf, sizeof tbuf,
-						 "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n", dcc->nick,
-						 dcc->file, dcc->port, dcc->resumable);
-			tcp_send (dcc->serv, tbuf);
-		}
+		snprintf (tbuf, sizeof (tbuf),
+					 "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n", dcc->nick,
+					 dcc->file, dcc->port, dcc->resumable);
+		tcp_send (dcc->serv, tbuf);
 	}
 }
 
@@ -1140,15 +1113,16 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->addr = addr;
 			dcc->port = port;
 			dcc->nick = strdup (nick);
+			dcc->starttime = time (0);
 
 			EMIT_SIGNAL (XP_TE_DCCCHATOFFER, sess->server->front_session, nick,
 							 NULL, NULL, NULL, 0);
-			if (!prefs.noautoopendccchatwindow)
+			if (prefs.autoopendccchatwindow)
 				fe_dcc_open_chat_win ();
 			else
 				fe_dcc_update_chat_win ();
 			if (prefs.autodccchat)
-				dcc_connect (0, dcc);
+				dcc_connect (dcc);
 			return;
 		}
 	}
@@ -1164,7 +1138,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->pos = dcc->resumable;
 			dcc->ack = dcc->resumable;
 			lseek (dcc->fp, dcc->pos, SEEK_SET);
-			snprintf (outbuf, 255,
+			snprintf (outbuf, 400,
 						 "PRIVMSG %s :\001DCC ACCEPT %s %d %d\001\r\n", dcc->nick,
 						 file_part (dcc->file), port, dcc->resumable);
 			tcp_send (dcc->serv, outbuf);
@@ -1181,7 +1155,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 		dcc = find_dcc_from_port (port, TYPE_RECV);
 		if (dcc && dcc->dccstat == STAT_QUEUED)
 		{
-			dcc_connect (0, dcc);
+			dcc_connect (dcc);
 			return;
 		}
 	}
@@ -1244,10 +1218,10 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 				{
 					dcc->resumable = 0;
 					dcc->pos = 0;
-					dcc_connect (sess, dcc);
+					dcc_connect (dcc);
 				}
 			}
-			if (!prefs.noautoopendccrecvwindow)
+			if (prefs.autoopendccrecvwindow)
 				fe_dcc_open_recv_win ();
 			else
 				fe_dcc_update_recv_win ();
