@@ -59,5 +59,6 @@ void fe_change_nick (struct server *serv, char *nick, char *newnick);
 void fe_ignore_update (int level);
 void fe_beep (void);
 int fe_is_confmode (struct session *);
-char *fe_buffer_get (session * sess);
+void fe_lastlog (session *sess, session *lastlog_sess, char *sstr);
 void fe_set_lag (server *serv, int lag);
+void fe_set_away (server *serv);
