@@ -20,6 +20,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <sys/types.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <netdb.h>
@@ -60,8 +61,6 @@ extern void setup_logging (struct session *sess);
 
 extern void dcc_change_nick (struct server *serv, char *oldnick,
 									  char *newnick);
-extern void handle_dcc (struct session *sess, char *outbuf, char *nick,
-								char *word[], char *word_eol[]);
 
 /* userlist.c */
 
@@ -72,7 +71,7 @@ extern void change_nick (struct session *sess, char *oldnick, char *newnick);
 extern int handle_command (char *cmd, struct session *sess, int history,
 									int nocommand);
 extern void process_data_init (unsigned char *buf, char *cmd, char *word[],
-										 char *word_eol[]);
+										 char *word_eol[], int handle_quotes);
 
 /* notify.c */
 
@@ -169,48 +168,6 @@ private_msg (struct server *serv, char *tbuf, char *from, char *ip,
 	EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
 }
 
-void
-channel_action (struct session *sess, char *tbuf, char *chan, char *from,
-					 char *text, int fromme)
-{
-	struct session *def = sess;
-
-	sess->highlight_tab = TRUE;
-
-	if (!sess)
-		fprintf (stderr, "*** XCHAT WARNING: sess = 0x0 in channel_action()\n");
-
-	if (EMIT_SIGNAL (XP_CHANACTION, sess, chan, from, text, NULL, fromme) == 1)
-		return;
-
-	if (fromme)
-	{
-		if (prefs.colorednicks)
-			sprintf (tbuf, "\0032%s\003 ", from);
-		else
-			strcpy (tbuf, from);
-	} else
-	{
-		if (prefs.colorednicks)
-			sprintf (tbuf, "\003%d%s\003 ", color_of (from), from);
-		else
-			strcpy (tbuf, from);
-	}
-
-	if (is_channel (chan) || fromme)
-		sess = find_session_from_channel (chan, sess->server);
-	else								  /* it's a private action! */
-		sess = find_session_from_channel (from, sess->server);
-
-	if (!sess && !is_channel (chan) && prefs.autodialog)
-		sess = new_session (def->server, from);
-
-	if (sess)
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, NULL, NULL, 0);
-	else
-		EMIT_SIGNAL (XP_TE_CHANACTION, def, tbuf, text, NULL, NULL, 0);
-}
-
 static int
 SearchNick (char *text, char *nicks)
 {
@@ -241,14 +198,84 @@ SearchNick (char *text, char *nicks)
 	return 0;
 }
 
+static int
+is_hilight (char *from, char *text, char *chan, session *sess, server *serv)
+{
+	if ((prefs.hilightnick && SearchNick (text, serv->nick)) ||
+			SearchNick (text, prefs.bluestring))
+	{
+		if (EMIT_SIGNAL (XP_HIGHLIGHT, sess, chan, from, text, NULL, 0)
+			 == 1)
+			return 2;
+		if (sess != current_tab && sess->is_tab)
+		{
+			sess->nick_said = TRUE;
+			fe_set_hilight (sess);
+		}
+		return 1;
+	}
+	return 0;
+}
+
+void
+channel_action (struct session *sess, char *tbuf, char *chan, char *from,
+					 char *text, int fromme)
+{
+	int hilight = FALSE;
+	struct session *def = sess;
+
+	if (EMIT_SIGNAL (XP_CHANACTION, sess, chan, from, text, NULL, fromme) == 1)
+		return;
+
+	if (is_channel (chan) || fromme)
+		sess = find_session_from_channel (chan, sess->server);
+	else
+	{
+        	/* it's a private action! */
+		sess = find_session_from_channel (from, sess->server);
+		if (prefs.beepmsg)
+			fe_beep ();
+	}
+
+	if (!sess && !is_channel (chan) && prefs.autodialog)
+		sess = new_session (def->server, from);
+
+	if (!sess)
+		sess = def;
+
+	if (!fromme)
+	{
+		switch (is_hilight (from, text, chan, sess, sess->server))
+		{
+		case 2:
+			return;	/* plugin ate the event */
+		case 1:
+			hilight = TRUE;
+		}
+	}
+
+	sess->highlight_tab = TRUE;
+
+	if (hilight)
+	{
+		EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, NULL, NULL, 0);
+	} else if (prefs.colorednicks)
+	{
+		sprintf (tbuf, "\003%d%s", color_of (from), from);
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, NULL, NULL, 0);
+	} else
+	{
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, NULL, NULL, 0);
+	}
+}
+
 void
 channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 				 char *text, char fromme)
 {
 	struct User *user;
-	char *real_outbuf = outbuf;
 	struct session *sess;
-	int highlight = FALSE;
+	int hilight = FALSE;
 
 	sess = find_session_from_channel (chan, serv);
 	if (!sess)
@@ -263,49 +290,30 @@ channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 	if (EMIT_SIGNAL (XP_CHANMSG, serv, chan, from, text, NULL, fromme) == 1)
 		return;
 
-	if (fromme)
+	if (!fromme)
 	{
-		if (prefs.colorednicks)
-			sprintf (outbuf, "\0032%s\017", from);
-		else
-			strcpy (outbuf, from);
-	} else
-	{
-
-		if (SearchNick (text, serv->nick)
-			 || SearchNick (text, prefs.bluestring))
+		switch (is_hilight (from, text, chan, sess, serv))
 		{
-			if (EMIT_SIGNAL (XP_HIGHLIGHT, sess, chan, from, text, NULL, fromme)
-				 == 1)
-				return;
-			highlight = TRUE;
-
-			if (sess != current_tab && sess->is_tab)
-			{
-				sess->nick_said = TRUE;
-				fe_set_hilight (sess);
-			}
-		}
-		if (prefs.colorednicks)
-			sprintf (outbuf, "\003%d%s\017", color_of (from), from);
-		else
-		{
-			if (highlight)
-				sprintf (outbuf, "\002\003%d%s\002\017", prefs.bt_color, from);
-			else
-				strcpy (outbuf, from);
+		case 2:
+			return;	/* plugin ate the event */
+		case 1:
+			hilight = TRUE;
 		}
 	}
 
 	if (fromme)
-		EMIT_SIGNAL (XP_TE_UCHANMSG, sess, real_outbuf, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_UCHANMSG, sess, from, text, NULL, NULL, 0);
+	else if (sess->is_dialog)
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
+	else if (hilight)
+		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, NULL, NULL, 0);
+	else if (prefs.colorednicks)
+	{
+		sprintf (outbuf, "\003%d%s", color_of (from), from);
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, outbuf, text, NULL, NULL, 0);
+	}
 	else
-	{
-		if (sess && sess->is_dialog)
-			EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, real_outbuf, text, NULL, NULL, 0);
-		else
-			EMIT_SIGNAL (XP_TE_CHANMSG, sess, real_outbuf, text, NULL, NULL, 0);
-	}
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, NULL, NULL, 0);
 }
 
 void
@@ -337,7 +345,6 @@ user_new_nick (struct server *serv, char *outbuf, char *nick, char *newnick,
 			{
 				if (!quiet)
 				{
-
 					if (me)
 						EMIT_SIGNAL (XP_TE_UCHANGENICK, sess, nick, newnick, NULL,
 										 NULL, 0);
@@ -436,7 +443,7 @@ you_joined (struct server *serv, char *outbuf, char *chan, char *nick,
 
 	EMIT_SIGNAL (XP_TE_UJOIN, sess, nick, chan, ip, NULL, 0);
 
-	if (!prefs.nouserhost)
+	if (prefs.userhost)
 	{
 		sprintf (outbuf, "WHO %s\r\n", sess->channel);
 		tcp_send (serv, outbuf);
@@ -613,10 +620,8 @@ static void
 channel_date (struct session *sess, char *tbuf, char *chan, char *timestr)
 {
 	long n = atol (timestr);
-	char *p, *tim = ctime (&n);
-	p = strchr (tim, '\n');
-	if (p)
-		*p = 0;
+	char *tim = ctime (&n);
+	tim[19] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_CHANDATE, sess, chan, tim, NULL, NULL, 0);
 }
 
@@ -626,13 +631,8 @@ topic_nametime (struct server *serv, char *tbuf, char *chan, char *nick,
 {
 	long n = atol (date);
 	char *tim = ctime (&n);
-	char *p;
 	struct session *sess = find_session_from_channel (chan, serv);
-
-	p = strchr (tim, '\n');
-	if (p)
-		*p = 0;
-
+	tim[19] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
 }
 
@@ -1053,7 +1053,7 @@ channel_modes (struct server *serv, char *outbuf, char *word[], char *nick,
 	}
 }
 
-static void
+static int
 end_of_names (struct server *serv, char *outbuf, char *chan, char *text)
 {
 	struct session *sess;
@@ -1072,14 +1072,16 @@ end_of_names (struct server *serv, char *outbuf, char *chan, char *text)
 			}
 			list = list->next;
 		}
-		return;
+		return TRUE;
 	}
 	sess = find_session_from_channel (chan, serv);
 	if (sess)
 	{
 		sess->end_of_names = TRUE;
 		sess->ignore_names = FALSE;
+		return TRUE;
 	}
+	return FALSE;
 }
 
 static void
@@ -1218,6 +1220,15 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		word[4]++;
 		notify_markonline (sess, outbuf, word);
 		break;
+	case 305:
+		serv->is_away = FALSE;
+		fe_set_away (serv);
+		goto def;
+	case 306:
+		serv->is_away = TRUE;
+		serv->away_time = time (0);
+		fe_set_away (serv);
+		goto def;
 	case 312:
 		EMIT_SIGNAL (XP_TE_WHOIS3, sess, word[4], word_eol[5], NULL, NULL, 0);
 		break;
@@ -1233,7 +1244,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		{
 			long n = atol (find_word (pdibuf, 6));
 			long idle = atol (find_word (pdibuf, 5));
-			char *po, *tim = ctime (&n);
+			char *tim;
 			sprintf (outbuf, "%02ld:%02ld:%02ld", idle / 3600, (idle / 60) % 60,
 						idle % 60);
 			if (n == 0)
@@ -1241,8 +1252,8 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 								 find_word (pdibuf, 4), outbuf, NULL, NULL, 0);
 			else
 			{
-				if ((po = strchr (tim, '\n')))
-					*po = 0;
+				tim = ctime (&n);
+				tim[19] = 0; 	/* get rid of the \n */
 				EMIT_SIGNAL (XP_TE_WHOIS4T, serv->front_session,
 								 find_word (pdibuf, 4), outbuf, tim, NULL, 0);
 			}
@@ -1404,7 +1415,21 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		}
 		break;
 	case 366:
-		end_of_names (serv, outbuf, find_word (pdibuf, 4), text);
+		if (!end_of_names (serv, outbuf, find_word (pdibuf, 4), text))
+			goto def;
+		break;
+	case 367: /* banlist entry */
+		{
+			time_t ban_time = atol (word[7]);
+			char *time_str = ctime (&ban_time);
+			time_str[19] = 0;	/* get rid of the \n */
+			if (ban_time == 0)
+				time_str = "";
+			sess = find_session_from_channel (word[4], serv);
+			if (!sess)
+				sess = serv->front_session;
+			EMIT_SIGNAL (XP_TE_BANLIST, sess, word[4], word[5], word[6], time_str, 0);
+		}
 		break;
 	case 376:
 	case 422:						  /* end of motd */
@@ -1514,9 +1539,9 @@ process_line (struct server *serv, char *buf)
 
 	/* split line into words and words_to_end_of_line */
 	if (*buf == ':')
-		process_data_init (pdibuf, buf + 1, word, word_eol);
+		process_data_init (pdibuf, buf + 1, word, word_eol, FALSE);
 	else
-		process_data_init (pdibuf, buf, word, word_eol);
+		process_data_init (pdibuf, buf, word, word_eol, FALSE);
 
 #ifdef USE_PERL
 	/* tell perl that we got a line of data */
@@ -1696,6 +1721,9 @@ process_line (struct server *serv, char *buf)
 					flood_check (nick, ip, serv, sess, 0);
 				if (ignore_check (pdibuf, 0, 0, 0, 1, 0))
 					return;
+				if (strncasecmp (msg + 1, "DCC ", 4) == 0)
+					/* redo this with handle_quotes TRUE */
+					process_data_init (pdibuf, buf, word, word_eol, TRUE);
 				handle_ctcp (sess, outbuf, to, nick, msg + 1, word, word_eol);
 			} else
 			{
@@ -1716,8 +1744,11 @@ process_line (struct server *serv, char *buf)
 	}
 	if (!strcmp ("PONG", cmd))
 	{
+		char *timestring = find_word (pdibuf, 4);
+		if (*timestring == ':')
+			timestring++;
 		got_ping_reply (serv->front_session, outbuf,
-							 find_word (pdibuf, 4) + 1, find_word (pdibuf, 3));
+								timestring, find_word (pdibuf, 3));
 		return;
 	}
 	if (!strcmp ("QUIT", cmd))
