@@ -18,6 +18,10 @@
 
 #include <stdio.h>
 #include <string.h>
+#include "../../config.h"
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -25,7 +29,6 @@
 #include <unistd.h>
 #include <time.h>
 #include "xchat.h"
-#include "util.h"
 #include "notify.h"
 #include "cfgfiles.h"
 #include "plugin.h"
@@ -73,14 +76,14 @@ notify_save (void)
 {
 	int fh;
 	char buf[256];
+	struct notify *notify;
+	GSList *list = notify_list;
 
 	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir ());
 
 	fh = open (buf, O_TRUNC | O_WRONLY | O_CREAT, 0600);
 	if (fh != -1)
 	{
-		struct notify *notify;
-		GSList *list = notify_list;
 		while (list)
 		{
 			notify = (struct notify *) list->data;
@@ -100,7 +103,7 @@ notify_load (void)
 
 	snprintf (buf, sizeof buf, "%s/notify.conf", get_xdir ());
 
-	fh = open (buf, O_RDONLY);
+	fh = open (buf, O_RDONLY | OFLAGS);
 	if (fh != -1)
 	{
 		while (waitline (fh, buf, sizeof buf) != -1)
