@@ -36,11 +36,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include "popup.h"
+#include "cfgfiles.h"			  /* get_xdir() */
 
 
 int handle_command (char *cmd, struct session *sess, int history, int);
 void handle_multiline (struct session *sess, char *cmd, int history, int);
-void check_special_chars (char *cmd);
 
 extern GSList *sess_list;
 extern GSList *serv_list;
@@ -56,8 +56,6 @@ extern void do_dns (struct session *sess, char *tbuf, char *nick, char *host);
 extern struct session *new_session (struct server *serv, char *from);
 extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
-extern struct session *find_session_from_channel (char *chan,
-																  struct server *serv);
 extern int list_delentry (GSList ** list, char *name);
 extern void list_addentry (GSList ** list, char *cmd, char *name);
 extern struct DCC *find_dcc (char *nick, char *file, int type);
@@ -72,6 +70,7 @@ extern void user_new_nick (struct server *serv, char *outbuf, char *nick,
 									char *newnick, int quiet);
 extern void channel_msg (struct server *serv, char *outbuf, char *chan,
 								 char *from, char *text, char fromme);
+extern struct session *find_session_from_channel (char *chan, struct server *serv);
 extern void disconnect_server (struct session *sess, int sendquit, int err);
 extern void notify_showlist (struct session *sess);
 extern void notify_adduser (char *name);
@@ -215,9 +214,88 @@ notc_msg (struct session *sess)
 	PrintText (sess, "Not connected. Try /server <host> [<port>]\n");
 }
 
+static char *
+random_line (char *file_name)
+{
+	FILE *fh;
+	char buf[512];
+	int lines, ran;
+
+	if (!file_name[0])
+		goto nofile;
+
+	snprintf (buf, sizeof (buf), "%s/%s", get_xdir (), file_name);
+	fh = fopen (buf, "r");
+	if (!fh)
+	{
+	 nofile:
+		/* reason is not a file, an actual reason! */
+		return strdup (file_name);
+	}
+
+	/* count number of lines in file */
+	lines = 0;
+	while (fgets (buf, sizeof (buf), fh))
+		lines++;
+
+	if (lines < 1)
+		goto nofile;
+
+	/* go down a random number */
+	rewind (fh);
+	srand (time (0));
+	ran = rand () % lines;
+	do
+	{
+		fgets (buf, sizeof (buf), fh);
+		lines--;
+	}
+	while (lines > ran);
+	fclose (fh);
+	buf[strlen (buf) - 1] = 0;	  /* remove the trailing '\n' */
+	return strdup (buf);
+}
+
+void
+server_sendpart (server * serv, char *channel, char *reason)
+{
+	char tbuf[512];
+
+	if (!reason)
+	{
+		reason = random_line (prefs.partreason);
+		snprintf (tbuf, sizeof (tbuf), "PART %s :%s\r\n", channel, reason);
+		free (reason);
+	} else
+	{
+		/* reason set by /quit, /close argument */
+		snprintf (tbuf, sizeof (tbuf), "PART %s :%s\r\n", channel, reason);
+	}
+	tcp_send (serv, tbuf);
+}
+
+void
+server_sendquit (session * sess)
+{
+	char tbuf[512];
+	char *rea;
+
+	if (!sess->quitreason)
+	{
+		rea = random_line (prefs.quitreason);
+		snprintf (tbuf, sizeof (tbuf), "QUIT :%s\r\n", rea);
+		free (rea);
+	} else
+	{
+		/* reason set by /quit, /close argument */
+		snprintf (tbuf, sizeof (tbuf), "QUIT :%s\r\n", sess->quitreason);
+	}
+	tcp_send (sess->server, tbuf);
+}
+
 void
 process_data_init (unsigned char *buf, char *cmd, char *word[],
-						 char *word_eol[])
+						 char *word_eol[], int handle_quotes)
 {
 	int wordcount = 2;
 	int space = FALSE;
@@ -241,6 +319,8 @@ process_data_init (unsigned char *buf, char *cmd, char *word[],
 			}
 			return;
 		case '\042':
+			if (!handle_quotes)
+				goto def;
 			if (quote)
 				quote = FALSE;
 			else
@@ -266,6 +346,7 @@ process_data_init (unsigned char *buf, char *cmd, char *word[],
 				break;
 			}
 		default:
+def:
 			space = FALSE;
 			buf[j] = *cmd;
 			j++;
@@ -277,9 +358,9 @@ process_data_init (unsigned char *buf, char *cmd, char *word[],
 static int cmd_addbutton (struct session *sess, char *tbuf, char *word[],
 								  char *word_eol[]);
 static int cmd_allchannels (struct session *sess, char *tbuf, char *word[],
-								  char *word_eol[]);
+									 char *word_eol[]);
 static int cmd_allservers (struct session *sess, char *tbuf, char *word[],
-								  char *word_eol[]);
+									char *word_eol[]);
 static int cmd_away (struct session *sess, char *tbuf, char *word[],
 							char *word_eol[]);
 static int cmd_ban (struct session *sess, char *tbuf, char *word[],
@@ -296,14 +377,14 @@ static int cmd_cycle (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
 static int cmd_dcc (struct session *sess, char *tbuf, char *word[],
 						  char *word_eol[]);
-static int cmd_debug (struct session *sess, char *tbuf, char *word[],
-							 char *word_eol[]);
+/*static int cmd_debug (struct session *sess, char *tbuf, char *word[],
+							 char *word_eol[]);*/
 static int cmd_delbutton (struct session *sess, char *tbuf, char *word[],
 								  char *word_eol[]);
 static int cmd_deop (struct session *sess, char *tbuf, char *word[],
 							char *word_eol[]);
 static int cmd_dehop (struct session *sess, char *tbuf, char *word[],
-							char *word_eol[]);
+							 char *word_eol[]);
 static int cmd_devoice (struct session *sess, char *tbuf, char *word[],
 								char *word_eol[]);
 static int cmd_discon (struct session *sess, char *tbuf, char *word[],
@@ -316,11 +397,11 @@ static int cmd_exec (struct session *sess, char *tbuf, char *word[],
 							char *word_eol[]);
 static int cmd_execk (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
+#ifndef __EMX__
 static int cmd_execs (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
 static int cmd_execc (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
-#ifndef __EMX__
 static int cmd_execw (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
 #endif
@@ -329,7 +410,7 @@ static int cmd_gate (struct session *sess, char *tbuf, char *word[],
 int cmd_help (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[]);
 static int cmd_hop (struct session *sess, char *tbuf, char *word[],
-						 char *word_eol[]);
+						  char *word_eol[]);
 static int cmd_ignore (struct session *sess, char *tbuf, char *word[],
 							  char *word_eol[]);
 static int cmd_invite (struct session *sess, char *tbuf, char *word[],
@@ -343,7 +424,7 @@ static int cmd_kickban (struct session *sess, char *tbuf, char *word[],
 static int cmd_lastlog (struct session *sess, char *tbuf, char *word[],
 								char *word_eol[]);
 static int cmd_list (struct session *sess, char *tbuf, char *word[],
-								char *word_eol[]);
+							char *word_eol[]);
 static int cmd_load (struct session *sess, char *tbuf, char *word[],
 							char *word_eol[]);
 int cmd_loaddll (struct session *sess, char *tbuf, char *word[],
@@ -351,7 +432,7 @@ int cmd_loaddll (struct session *sess, char *tbuf, char *word[],
 static int cmd_mdeop (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
 static int cmd_mdehop (struct session *sess, char *tbuf, char *word[],
-							 char *word_eol[]);
+							  char *word_eol[]);
 static int cmd_me (struct session *sess, char *tbuf, char *word[],
 						 char *word_eol[]);
 static int cmd_mkick (struct session *sess, char *tbuf, char *word[],
@@ -408,8 +489,6 @@ static int cmd_timer (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
 static int cmd_topic (struct session *sess, char *tbuf, char *word[],
 							 char *word_eol[]);
-static int cmd_unban (struct session *sess, char *tbuf, char *word[],
-							 char *word_eol[]);
 static int cmd_unignore (struct session *sess, char *tbuf, char *word[],
 								 char *word_eol[]);
 int cmd_unloadall (struct session *sess, char *tbuf, char *word[],
@@ -428,9 +507,9 @@ struct commands cmds[] = {
 	{"ADDBUTTON", cmd_addbutton, 0, 0,
 	 "/ADDBUTTON <name> <action>, adds a button under the user-list\n"},
 	{"ALLCHAN", cmd_allchannels, 0, 0,
-		"/ALLCHAN <cmd>, sends a command to all channels you're in\n"},
+	 "/ALLCHAN <cmd>, sends a command to all channels you're in\n"},
 	{"ALLSERV", cmd_allservers, 0, 0,
-		"/ALLSERV <cmd>, sends a command to all servers you're in\n"},
+	 "/ALLSERV <cmd>, sends a command to all servers you're in\n"},
 	{"AWAY", cmd_away, 1, 0, "/AWAY [<reason>], sets you away\n"},
 	{"BAN", cmd_ban, 1, 1,
 	 "/BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)\n"},
@@ -450,7 +529,7 @@ struct commands cmds[] = {
 	 "/DCC CHAT <nick>         - offer DCC CHAT to someone\n"
 	 "/DCC CLOSE <type> <nick> <file>         example:\n"
 	 "         /dcc close send johnsmith file.tar.gz\n"},
-	{"DEBUG", cmd_debug, 0, 0, 0},
+/*	{"DEBUG", cmd_debug, 0, 0, 0},*/
 
 	{"DELBUTTON", cmd_delbutton, 0, 0,
 	 "/DELBUTTON <name>, deletes a button from under the user-list\n"},
@@ -468,14 +547,13 @@ struct commands cmds[] = {
 	 "/EXEC [-o] <command>, runs the command. If -o flag is used then output is sent to current channel, else is printed to current text box\n"},
 	{"EXECKILL", cmd_execk, 0, 0,
 	 "/EXECKILL [-9], kills a running exec in the current session. If -9 is given the process is SIGKILL'ed\n"},
+#ifndef __EMX__
 	{"EXECSTOP", cmd_execs, 0, 0, "/EXECSTOP, sends the process SIGSTOP\n"},
 	{"EXECCONT", cmd_execc, 0, 0, "/EXECCONT, sends the process SIGCONT\n"},
-#ifndef __EMX__
 
 	{"EXECWRITE", cmd_execw, 0, 0,
 	 "/EXECWRITE, sends data to the processes stdin\n"},
 #endif
-
 	{"GATE", cmd_gate, 0, 0,
 	 "/GATE <host> [<port>], proxies through a host, port defaults to 23\n"},
 	{"HELP", cmd_help, 0, 0, 0},
@@ -516,7 +594,7 @@ struct commands cmds[] = {
 	 "/MKICK, Mass kicks everyone except you in the current channel (needs chanop)\n"},
 	{"MKICKB", cmd_mkickb, 1, 1,
 	 "/MKICKB, Sets a ban of *@* and mass kicks everyone except you in the current channel (needs chanop)\n"},
-	{"ME", cmd_me, 1, 1,
+	{"ME", cmd_me, 0, 0,
 	 "/ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)\n"},
 	{"MSG", cmd_msg, 0, 0, "/MSG <nick> <message>, sends a private message\n"},
 
@@ -570,23 +648,25 @@ struct commands cmds[] = {
 	{"SERVCHAN", cmd_servchan, 0, 0,
 	 "/SERVCHAN <host> <port> <channel> [<ssl>] [<accept_invalid_cert>], connects and joins a channel\n"},
 #else
- 	{"SERVCHAN", cmd_servchan, 0, 0,
+	{"SERVCHAN", cmd_servchan, 0, 0,
 	 "/SERVCHAN <host> <host> <channel>, connects and joins a channel\n"},
 #endif
 #ifdef USE_OPENSSL
-	{"SERVER", cmd_server, 0, 0, "/SERVER <host> [<port>] [<password>] [<ssl>] [<accept_invalid_cert>], connects to a server, the default port is 6667\n"},
+	
+		{"SERVER", cmd_server, 0, 0,
+	 "/SERVER <host> [<port>] [<password>] [<ssl>] [<accept_invalid_cert>], connects to a server, the default port is 6667\n"},
 #else
-	{"SERVER", cmd_server, 0, 0, "/SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667\n"},
+	
+		{"SERVER", cmd_server, 0, 0,
+	 "/SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667\n"},
 #endif
 #ifdef USE_OPENSSL
 	{"SSLSERVER", cmd_sslserver, 0, 0,
-	 "/SSLSERVER <host> [<port>] [<password>] [<accept_invalid_cert>], connects to a server, the default port is 994\n"},
+	 "/SSLSERVER <host> [<port>] [<accept_invalid_cert (y/n)>] [<password>], connects to a server, the default port is 994\n"},
 #endif
 	{"TIMER", cmd_timer, 0, 0, "/TIMER <seconds> <command>\n"},
 	{"TOPIC", cmd_topic, 1, 1,
 	 "/TOPIC [<topic>], sets the topic is one is given, else shows the current topic\n"},
-	{"UNBAN", cmd_unban, 1, 1,
-	 "/UNBAN <mask>, removes a current ban on a mask\n"},
 	{"UNIGNORE", cmd_unignore, 0, 0, "/UNIGNORE <mask> [QUIET]\n"},
 #ifdef USE_PERL
 	{"UNLOADALL", cmd_unloadall, 0, 0, "/UNLOADALL, Unloads all perl scripts\n"},
@@ -631,7 +711,7 @@ help (struct session *sess, char *helpcmd, int quiet)
 
 static void
 send_channel_modes3 (struct session *sess, char *tbuf,
-						  char *word[], int start, int end, char sign, char mode)
+							char *word[], int start, int end, char sign, char mode)
 {
 	int left;
 	int i = start;
@@ -667,7 +747,7 @@ send_channel_modes3 (struct session *sess, char *tbuf,
 
 static void
 send_channel_modes6 (struct session *sess, char *tbuf,
-						  char *word[], int start, int end, char sign, char mode)
+							char *word[], int start, int end, char sign, char mode)
 {
 	int left;
 	int i = start;
@@ -701,14 +781,14 @@ send_channel_modes6 (struct session *sess, char *tbuf,
 			i += 4;
 			break;
 		case 5:
-			sprintf (tbuf, "MODE %s %c%c%c%c%c%c %s %s %s %s %s\r\n", sess->channel,
-						sign, mode, mode, mode, mode, mode,
-						word[i], word[i + 1], word[i + 2], word[i + 3], word[i + 4]);
+			sprintf (tbuf, "MODE %s %c%c%c%c%c%c %s %s %s %s %s\r\n",
+						sess->channel, sign, mode, mode, mode, mode, mode, word[i],
+						word[i + 1], word[i + 2], word[i + 3], word[i + 4]);
 			i += 5;
 			break;
 		default:
-			sprintf (tbuf, "MODE %s %c%c%c%c%c%c%c %s %s %s %s %s %s\r\n", sess->channel,
-						sign, mode, mode, mode, mode, mode, mode,
+			sprintf (tbuf, "MODE %s %c%c%c%c%c%c%c %s %s %s %s %s %s\r\n",
+						sess->channel, sign, mode, mode, mode, mode, mode, mode,
 						word[i], word[i + 1], word[i + 2], word[i + 3], word[i + 4],
 						word[i + 5]);
 			i += 6;
@@ -748,7 +828,7 @@ cmd_addbutton (struct session *sess, char *tbuf, char *word[],
 
 int
 cmd_allchannels (struct session *sess, char *tbuf, char *word[],
-					char *word_eol[])
+					  char *word_eol[])
 {
 	GSList *list = sess_list;
 
@@ -759,7 +839,7 @@ cmd_allchannels (struct session *sess, char *tbuf, char *word[],
 	{
 		sess = list->data;
 		if (!sess->is_server && !sess->is_shell && !sess->is_dialog &&
-				sess->channel[0] && sess->server->connected)
+			 sess->channel[0] && sess->server->connected)
 		{
 			handle_command (word_eol[2], sess, TRUE, FALSE);
 		}
@@ -771,20 +851,20 @@ cmd_allchannels (struct session *sess, char *tbuf, char *word[],
 
 int
 cmd_allservers (struct session *sess, char *tbuf, char *word[],
-					char *word_eol[])
+					 char *word_eol[])
 {
-	GSList *list = sess_list;
+	GSList *list;
+	server *serv;
 
 	if (!*word_eol[2])
 		return FALSE;
 
+	list = serv_list;
 	while (list)
 	{
-		sess = list->data;
-		if (sess->is_server && sess->server->connected)
-		{
-			handle_command (word_eol[2], sess, TRUE, FALSE);
-		}
+		serv = list->data;
+		if (serv->connected)
+			handle_command (word_eol[2], serv->front_session, TRUE, FALSE);
 		list = list->next;
 	}
 
@@ -794,32 +874,41 @@ cmd_allservers (struct session *sess, char *tbuf, char *word[],
 int
 cmd_away (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
+	GSList *list;
 	char *reason = find_word_to_end (cmd, 2);
+	int freeit = FALSE;
+	int back = FALSE;
+	unsigned int gone;
 
-	if (*reason)
+	if (!(*reason) && sess->server->is_away)
 	{
-		sprintf (tbuf, "AWAY :%s\r\n", reason);
-		tcp_send (sess->server, tbuf);
+		/* mark back */
+		tcp_send_len (sess->server, "AWAY\r\n", 6);
+		back = TRUE;
 	} else
 	{
-		tcp_send_len (sess->server, "AWAY\r\n", 6);
+		if (!(*reason))
+		{
+			reason = random_line (prefs.awayreason);
+			freeit = TRUE;
+		}
+		sprintf (tbuf, "AWAY :%s\r\n", reason);
+		tcp_send (sess->server, tbuf);
 	}
+
 	if (prefs.show_away_message)
 	{
-		char *away_me;
-		GSList *list = sess_list;
-
-		if (*reason)
+		if (back)
 		{
-			/* /me is away: + reason + \0 */
-			away_me = malloc (13 + strlen (reason) + 1);
-			sprintf (away_me, "/me is away: %s", reason);
+			gone = time (0) - sess->server->away_time;
+			sprintf (tbuf, "/me is back (gone %.2d:%.2d:%.2d)", gone / 3600,
+						(gone / 60) % 60, gone % 60);
 		} else
 		{
-			away_me = malloc (13);
-			strcpy (away_me, "/me is back");
+			sprintf (tbuf, "/me is away: %s", reason);
 		}
 
+		list = sess_list;
 		while (list)
 		{
 			/* am I the right server and not a dialog box */
@@ -827,14 +916,16 @@ cmd_away (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				 && !((struct session *) list->data)->is_dialog
 				 && !((struct session *) list->data)->is_server)
 			{
-				handle_command (away_me, (struct session *) list->data,
+				handle_command (tbuf, (struct session *) list->data,
 									 FALSE, FALSE);
 			}
 			list = list->next;
 		}
-
-		free (away_me);
 	}
+
+	if (freeit)
+		free (reason);
+
 	return TRUE;
 }
 
@@ -843,7 +934,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr)
 {
 	int bantype;
 	struct User *user;
-	char *at, *dot;
+	char *at, *dot, *lastdot;
 	char username[64], fullhost[128], domain[128];
 
 	user = find_name (sess, mask);
@@ -856,11 +947,18 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr)
 			return;					  /* can't happen? */
 		*at = 0;
 
-		strcpy (username, mask);
+		if (mask[0] == '~' ||
+		    mask[0] == '+' ||
+		    mask[0] == '=' ||
+		    mask[0] == '^' ||
+		    mask[0] == '-')
+			strcpy (username, mask+1);
+		else
+			strcpy (username, mask);
 		*at = '@';
 		strcpy (fullhost, at + 1);
 
-		dot = strchr (mask, '.');
+		dot = strchr (fullhost, '.');
 		if (dot)
 			strcpy (domain, dot);
 		else
@@ -873,7 +971,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr)
 
 		if (inet_addr (fullhost) != -1)	/* "fullhost" is really a IP number */
 		{
-			char *lastdot = strrchr (fullhost, '.');
+			lastdot = strrchr (fullhost, '.');
 			if (!lastdot)
 				return;				  /* can't happen? */
 
@@ -892,12 +990,12 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr)
 				break;
 
 			case 2:
-				sprintf (tbuf, "MODE %s +b *!%s@%s.*\r\n", sess->channel,
+				sprintf (tbuf, "MODE %s +b *!*%s@%s.*\r\n", sess->channel,
 							username, domain);
 				break;
 
 			case 3:
-				sprintf (tbuf, "MODE %s +b *!%s@%s\r\n", sess->channel, username,
+				sprintf (tbuf, "MODE %s +b *!*%s@%s\r\n", sess->channel, username,
 							fullhost);
 				break;
 			}
@@ -914,12 +1012,12 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr)
 				break;
 
 			case 2:
-				sprintf (tbuf, "MODE %s +b *!%s@*%s\r\n", sess->channel, username,
+				sprintf (tbuf, "MODE %s +b *!*%s@*%s\r\n", sess->channel, username,
 							domain);
 				break;
 
 			case 3:
-				sprintf (tbuf, "MODE %s +b *!%s@%s\r\n", sess->channel, username,
+				sprintf (tbuf, "MODE %s +b *!*%s@%s\r\n", sess->channel, username,
 							fullhost);
 				break;
 			}
@@ -977,8 +1075,8 @@ cmd_close (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (*word_eol[2])
 		sess->quitreason = word_eol[2];
-	else
-		sess->quitreason = prefs.quitreason;
+	/*else
+	   sess->quitreason = prefs.quitreason; */
 	fe_close_window (sess);
 
 	return TRUE;
@@ -1112,7 +1210,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 					dcc_get_nick (sess, nick);
 			} else
 			{
-				struct DCC *dcc = find_dcc (nick, file, TYPE_RECV);
+				dcc = find_dcc (nick, file, TYPE_RECV);
 				if (dcc)
 					dcc_get (dcc);
 				else
@@ -1148,7 +1246,7 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
-int
+/*int
 cmd_debug (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	struct session *s;
@@ -1186,10 +1284,10 @@ cmd_debug (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 #ifdef MEMORY_DEBUG
 	sprintf (tbuf, "current mem: %d\n\n", current_mem_usage);
 	PrintText (sess, tbuf);
-#endif /* MEMORY_DEBUG */
+#endif  MEMORY_DEBUG 
 
 	return TRUE;
-}
+}*/
 
 int
 cmd_delbutton (struct session *sess, char *tbuf, char *word[],
@@ -1203,6 +1301,7 @@ cmd_delbutton (struct session *sess, char *tbuf, char *word[],
 	}
 	return FALSE;
 }
+
 int
 cmd_dehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1221,7 +1320,6 @@ cmd_dehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 }
 
-
 int
 cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1239,6 +1337,7 @@ cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		i++;
 	}
 }
+
 int
 cmd_mdehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1428,6 +1527,7 @@ exec_check_process (struct session *sess)
 	}
 }
 
+#ifndef __EMX__
 int
 cmd_execs (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1463,6 +1563,7 @@ cmd_execc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 	return TRUE;
 }
+#endif
 
 int
 cmd_execk (struct session *sess, char *tbuf, char *word[], char *word_eol[])
@@ -1762,8 +1863,8 @@ cmd_quit (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (*word_eol[2])
 		sess->quitreason = word_eol[2];
-	else
-		sess->quitreason = prefs.quitreason;
+	/*else
+	   sess->quitreason = prefs.quitreason; */
 	disconnect_server (sess, TRUE, -1);
 	return 2;
 }
@@ -1852,10 +1953,17 @@ cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			if (t == 6)
 			{
 				t = 1;
-				strcat (buf, "\n  ");
+				strcat (buf, "\n");
+				PrintText (sess, buf);
+				strcpy (buf, "  ");
 			} else
-				for (j = 0; j < (10 - strlen (pop->name)); j++)
-					strcat (buf, " ");
+			{
+				if (strlen (pop->name) < 10)
+				{
+					for (j = 0; j < (10 - strlen (pop->name)); j++)
+						strcat (buf, " ");
+				}
+			}
 			list = list->next;
 		}
 		strcat (buf, "\n");
@@ -1996,8 +2104,13 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 	if (*nick)
 	{
-		/* this will use prefs.bantype, no args allowed in /kickban */
-		ban (sess, tbuf, nick, "");
+		/* if the reason is a 1 digit number, treat it as a bantype */
+		if (isdigit (reason[0]) && reason[1] == 0)
+		{
+			ban (sess, tbuf, nick, reason);
+			reason[0] = 0;
+		} else
+			ban (sess, tbuf, nick, "");
 
 		if (*reason)
 			sprintf (tbuf, "KICK %s %s :%s\r\n", sess->channel, nick, reason);
@@ -2011,10 +2124,9 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 static void
-lastlog (session * sess, char *tbuf, char *search)
+lastlog (session *sess, char *search)
 {
 	session *lastlog_sess;
-	char *buf, *end, *start;
 
 	if (!is_session (sess))
 		return;
@@ -2028,25 +2140,7 @@ lastlog (session * sess, char *tbuf, char *search)
 
 	fe_text_clear (lastlog_sess);
 
-	buf = start = fe_buffer_get (sess);
-	if (buf)
-	{
-		while ((end = strchr (start, '\n')))
-		{
-			strncpy (tbuf, start, end - start);
-			tbuf[end - start] = 0;
-			if (nocasestrstr (tbuf, search))
-			{
-				strcat (tbuf, "\n");
-				PrintText (lastlog_sess, tbuf);
-			}
-			start = ++end;
-		}
-		free (buf);
-	} else
-	{
-		PrintText (lastlog_sess, "Search buffer is empty.\n");
-	}
+	fe_lastlog (sess, lastlog_sess, search);
 }
 
 int
@@ -2054,7 +2148,7 @@ cmd_lastlog (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (*word_eol[2])
 	{
-		lastlog (sess, tbuf, word_eol[2]);
+		lastlog (sess, word_eol[2]);
 		return TRUE;
 	}
 
@@ -2130,20 +2224,35 @@ int
 cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	char *act = find_word_to_end (cmd, 2);
-	if (*act)
+
+	if (!(*act))
+		return FALSE;
+
+	sprintf (tbuf, "\001ACTION %s\001\r", act);
+	/* first try through DCC CHAT */
+	if (dcc_write_chat (sess->channel, tbuf))
 	{
-		channel_action (sess, tbuf, sess->channel, sess->server->nick, act,
-							 TRUE);
-		sprintf (tbuf, "\001ACTION %s\001\r", act);
-		if (!dcc_write_chat (sess->channel, tbuf))
+		/* print it to screen */
+		channel_action (sess, tbuf, sess->channel, sess->server->nick,
+								act, TRUE);
+	} else
+	{
+		/* DCC CHAT failed, try through server */
+		if (sess->server->connected)
 		{
 			sprintf (tbuf, "PRIVMSG %s :\001ACTION %s\001\r\n", sess->channel,
 						act);
 			tcp_send (sess->server, tbuf);
+			/* print it to screen */
+			channel_action (sess, tbuf, sess->channel, sess->server->nick,
+									act, TRUE);
+		} else
+		{
+			notc_msg (sess);
 		}
-		return TRUE;
 	}
-	return FALSE;
+
+	return TRUE;
 }
 
 int
@@ -2151,6 +2260,7 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	char *nick = find_word (pdibuf, 2);
 	char *msg = find_word_to_end (cmd, 3);
+	struct session *newsess;
 
 	if (*nick)
 	{
@@ -2181,7 +2291,12 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				sprintf (tbuf, "PRIVMSG %s :%s\r\n", nick, msg);
 				tcp_send (sess->server, tbuf);
 			}
-			EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
+			newsess = find_session_from_channel (nick, sess->server);
+			if (newsess)
+				channel_msg (newsess->server, tbuf, newsess->channel,
+						newsess->server->nick, msg, TRUE);
+			else
+				EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
 
 			return TRUE;
 		}
@@ -2315,8 +2430,9 @@ cmd_part (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		chan = sess->channel;
 	if ((*chan) && is_channel (chan))
 	{
-		sprintf (tbuf, "PART %s :%s\r\n", chan, reason);
-		tcp_send (sess->server, tbuf);
+		server_sendpart (sess->server, chan, reason);
+		/*sprintf (tbuf, "PART %s :%s\r\n", chan, reason);
+		   tcp_send (sess->server, tbuf); */
 		return TRUE;
 	}
 	return FALSE;
@@ -2489,14 +2605,15 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 #ifdef USE_OPENSSL
+
 int
 cmd_sslserver (struct session *sess, char *tbuf, char *word[],
 					char *word_eol[])
 {
 	char *server = find_word (pdibuf, 2);
 	char *port = find_word (pdibuf, 3);
-	char *pass = find_word (pdibuf, 4);
-	char *accept_invalid_cert = find_word (pdibuf, 5);
+	char *accept_invalid_cert = find_word (pdibuf, 4);
+	char *pass = find_word (pdibuf, 5);
 	char *co;
 
 	if (*server)
@@ -2508,14 +2625,15 @@ cmd_sslserver (struct session *sess, char *tbuf, char *word[],
 		{
 			port = co + 1;
 			*co = 0;
-			pass = find_word (pdibuf, 3);
-			accept_invalid_cert = find_word (pdibuf, 4);
+			accept_invalid_cert = find_word (pdibuf, 3);
+			pass = find_word (pdibuf, 4);
 		}
 		sess->willjoinchannel[0] = 0;
+		sess->server->use_ssl = TRUE;
+		sess->server->accept_invalid_cert =
+			strcmp (accept_invalid_cert, "y") ? FALSE : TRUE;
 		if (pass)
 			strcpy (sess->server->password, pass);
-		sess->server->use_ssl = TRUE;
-		sess->server->accept_invalid_cert = *accept_invalid_cert ? TRUE : FALSE;
 		if (*port)
 			connect_server (sess, server, atoi (port), FALSE);
 		else
@@ -2601,20 +2719,8 @@ cmd_topic (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 int
-cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
-{
-	char *mask = find_word (pdibuf, 2);
-	if (*mask)
-	{
-		sprintf (tbuf, "MODE %s -b %s\r\n", sess->channel, mask);
-		tcp_send (sess->server, tbuf);
-		return TRUE;
-	}
-	return FALSE;
-}
-
-int
-cmd_unignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+cmd_unignore (struct session *sess, char *tbuf, char *word[],
+				  char *word_eol[])
 {
 	char *mask = find_word (pdibuf, 2);
 	char *arg = find_word (pdibuf, 3);
@@ -2631,7 +2737,8 @@ cmd_unignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 int
-cmd_userlist (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+cmd_userlist (struct session *sess, char *tbuf, char *word[],
+				  char *word_eol[])
 {
 	struct User *user;
 	GSList *list;
@@ -2644,7 +2751,9 @@ cmd_userlist (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		lt = time (0) - user->lasttalk;
 		if (!user->lasttalk)
 			lt = 0;
-		sprintf (tbuf, "\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d lt\0033=\017%d\n", user->nick, user->hostname, user->op, user->voice, lt);
+		sprintf (tbuf,
+					"\00306%s\t\00314[\00310%-38s\00314] \017ov\0033=\017%d%d lt\0033=\017%d\n",
+					user->nick, user->hostname, user->op, user->voice, lt);
 		PrintText (sess, tbuf);
 		list = list->next;
 	}
@@ -2849,7 +2958,7 @@ auto_insert (char *dest, char *src, char *word[], char *word_eol[],
 			}
 		} else
 		{
-lamecode:
+		 lamecode:
 			dest[0] = src[0];
 			dest++;
 		}
@@ -2862,7 +2971,7 @@ lamecode:
 }
 
 void
-check_special_chars (char *cmd) /* check for %X */
+check_special_chars (char *cmd, int perc_ascii) /* check for %X */
 {
 	int occur = 0;
 	int len = strlen (cmd);
@@ -2878,9 +2987,10 @@ check_special_chars (char *cmd) /* check for %X */
 			{
 			case '%':
 				occur++;
-				if (j + 3 < len
-					 && (isdigit (cmd[j + 1]) && isdigit (cmd[j + 2])
-						  && isdigit (cmd[j + 3])))
+				if (	perc_ascii &&
+						j + 3 < len &&
+						(isdigit (cmd[j + 1]) && isdigit (cmd[j + 2]) &&
+						isdigit (cmd[j + 3])))
 				{
 					tbuf[0] = cmd[j + 1];
 					tbuf[1] = cmd[j + 2];
@@ -2987,10 +3097,11 @@ check_nick_completion (struct session *sess, char *cmd, char *tbuf)
 }
 
 static void
-user_command (session *sess, char *tbuf, char *cmd, char *word[], char *word_eol[])
+user_command (session * sess, char *tbuf, char *cmd, char *word[],
+				  char *word_eol[])
 {
 	if (!auto_insert (tbuf, cmd, word, word_eol, "", sess->channel, "", "",
-                     sess->server->nick, ""))
+							sess->server->nick, ""))
 	{
 		PrintText (sess, "Bad arguments for user command.\n");
 		return;
@@ -3031,11 +3142,17 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
 		return TRUE;
 #endif
 
-	if (cmd[0] == '/' && !nocommand)
+	if (cmd[0] == prefs.cmdchar[0] && cmd[1] == prefs.cmdchar[0])
+	{
+		nocommand = TRUE;
+		cmd++;
+	}
+
+	if (cmd[0] == prefs.cmdchar[0] && !nocommand)
 	{
 		cmd++;
 
-		process_data_init (pdibuf, cmd, word, word_eol);
+		process_data_init (pdibuf, cmd, word, word_eol, TRUE);
 
 		if (EMIT_SIGNAL (XP_USERCOMMAND, sess, pdibuf, word, word_eol, NULL, 0)
 			 == 1)
@@ -3065,7 +3182,7 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
 		if (user_cmd)
 			return TRUE;
 
-		check_special_chars (cmd);
+		check_special_chars (cmd, prefs.perc_ascii);
 
 		/* now check internal commands */
 		i = 0;
@@ -3109,11 +3226,11 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
 
 	if (strcmp (sess->channel, "(lastlog)") == 0)
 	{
-		lastlog (sess->lastlog_sess, tbuf, cmd);
+		lastlog (sess->lastlog_sess, cmd);
 		return TRUE;
 	}
 
-	check_special_chars (cmd);
+	check_special_chars (cmd, prefs.perc_ascii);
 
 	if (!sess->channel[0] || sess->is_server)
 	{
@@ -3156,7 +3273,7 @@ handle_command (char *cmd, struct session *sess, int history, int nocommand)
 		if (strlen (cmd) > max)
 		{
 			t = cmd[max];
-			cmd[max] = 0;		/* insert a NULL terminator to shorten it */
+			cmd[max] = 0;			  /* insert a NULL terminator to shorten it */
 		}
 
 		channel_msg (sess->server, tbuf, sess->channel,
