@@ -124,8 +124,8 @@ XS (XS_IRC_ignore_list);
 XS (XS_IRC_dcc_list);
 XS (XS_IRC_get_info);
 XS (XS_IRC_get_prefs);
-XS (XS_IRC_user_list_short); /*MAG*/
-XS (XS_IRC_perl_script_list); /*MAG*/
+XS (XS_IRC_user_list_short);
+XS (XS_IRC_perl_script_list);
 
 /* perl module support */
 extern void xs_init _((void));
@@ -307,8 +307,8 @@ perl_init (struct session *default_sess, int autoload)
 	newXS ("IRC::dcc_list", XS_IRC_dcc_list, "IRC");
 	newXS ("IRC::get_info", XS_IRC_get_info, "IRC");
 	newXS ("IRC::get_prefs", XS_IRC_get_prefs, "IRC");
-	newXS ("IRC::user_list_short", XS_IRC_user_list_short, "IRC"); /*MAG*/
-	newXS ("IRC::perl_script_list", XS_IRC_perl_script_list, "IRC"); /*MAG*/
+	newXS ("IRC::user_list_short", XS_IRC_user_list_short, "IRC");
+	newXS ("IRC::perl_script_list", XS_IRC_perl_script_list, "IRC");
 
 	if (autoload)
 		for_files (get_xdir (), "*.pl", perl_autoload_file);
@@ -703,6 +703,10 @@ XS (XS_IRC_get_info)
 		XST_mPV (0, get_xdir ());
 		break;
 
+	case 5:
+		XST_mIV (0, perl_sess->server->is_away);
+		break;
+
 	default:
 		XST_mPV (0, "Error2");
 	}
@@ -966,9 +970,9 @@ XS (XS_IRC_user_list)
 	char *channel, *server;
 	GSList *list = sess_list;
 	int i = 0, junk;
-	int MinStk = 10;	/* MAG */
+	int MinStk = 10;
 	dXSARGS;
-	EXTEND (SP, MinStk); /* MAG */
+	EXTEND (SP, MinStk);
 	items = 0;
 
 	channel = SvPV (ST (0), junk);
@@ -1000,7 +1004,7 @@ XS (XS_IRC_user_list)
 					XST_mPV (i, ":");
 					i++;
 					list = list->next;
-					/* MAG: Make sure there is room on the stack */
+					/* Make sure there is room on the stack */
 					MinStk = i + 10;
 					EXTEND(SP, MinStk);
 				}
@@ -1132,9 +1136,9 @@ XS (XS_IRC_user_list_short)
 	char *channel, *server;
 	GSList *list = sess_list;
 	int i = 0, junk;
-	int MinStk = 10;	/* MAG */
+	int MinStk = 10;
 	dXSARGS;
-	EXTEND (SP, MinStk); /* MAG */
+	EXTEND (SP, MinStk);
 	items = 0;
 
 	channel = SvPV (ST (0), junk);
@@ -1160,7 +1164,7 @@ XS (XS_IRC_user_list_short)
 						XST_mPV (i, "FETCHING");
 					i++;
 					list = list->next;
-					/* MAG: Make sure there is room on the stack */
+					/* Make sure there is room on the stack */
 					MinStk = i + 10;
 					EXTEND(SP, MinStk);
 				}
@@ -1176,11 +1180,12 @@ XS (XS_IRC_user_list_short)
 
 /* MAG030600 BEGIN IRC::perl_script_list() */
 /* return a list of currently loaded perl script names and versions */
-XS (XS_IRC_perl_script_list) {
+XS (XS_IRC_perl_script_list)
+{
 	int i = 0;
 	GSList *handler;
 
-	dXSARGS;
+	dXSARGS; items = 0;
 
 	handler = perl_list;
 	while (handler)
