@@ -31,6 +31,8 @@
 #include <sys/wait.h>
 #include "plugin.h"
 #include "fe.h"
+#include "cfgfiles.h"
+#include "util.h"		/* is_channel() */
 
 #ifdef USE_OPENSSL
 #include <openssl/ssl.h>		  /* SSL_() */
@@ -40,6 +42,11 @@
 
 extern GSList *sess_list;
 extern struct xchatprefs prefs;
+#ifdef USE_OPENSSL
+extern SSL_CTX *ctx;		/* xchat.c */
+/* local variables */
+static struct session *g_sess = NULL;
+#endif
 
 extern int handle_command (char *cmd, struct session *sess, int history, int);
 extern void auto_reconnect (struct server *serv, int send_quit, int err);
@@ -49,13 +56,19 @@ extern void flush_server_queue (struct server *serv);
 extern int tcp_send_len (struct server *serv, char *buf, int len);
 extern int tcp_send (struct server *serv, char *buf);
 extern void PrintText (struct session *sess, unsigned char *text);
-extern void read_data (struct server *serv, gint sok);
 extern char *errorstring (int err);
 extern int waitline (int sok, char *buf, int bufsize);
 extern void notc_msg (struct session *sess);
 extern void notify_cleanup (void);
+extern int is_server (server *serv);
+extern void process_line (struct server *serv, char *buf);
+extern unsigned char *strip_color (unsigned char *text);
+extern void server_sendquit (session *sess);
+extern void serv2user (unsigned char *);
 
 void disconnect_server (struct session *sess, int sendquit, int err);
+void connect_server (struct session *sess, char *server, int port, int no_login);
+static void read_data (struct server *serv, gint sok);
 
 static void
 server_cleanup (server * serv)
@@ -82,6 +95,7 @@ server_cleanup (server * serv)
 static void
 server_connected (server * serv)
 {
+	char hostname[256];
 	char outbuf[512];
 
 	serv->ping_recv = time (0);
@@ -105,10 +119,13 @@ server_connected (server * serv)
 			sprintf (outbuf, "PASS %s\r\n", serv->password);
 			tcp_send (serv, outbuf);
 		}
-		snprintf (outbuf, sizeof (outbuf) - 1,
+		gethostname (hostname, sizeof (hostname) - 1);
+		hostname[sizeof (hostname) - 1] = 0;
+		snprintf (outbuf, sizeof (outbuf),
 					 "NICK %s\r\n"
-					 "USER %s localhost %s :%s\r\n",
-					 serv->nick, prefs.username, serv->servername, prefs.realname);
+					 "USER %s %s %s :%s\r\n",
+						serv->nick, prefs.username, hostname,
+						serv->servername, prefs.realname);
 		tcp_send (serv, outbuf);
 	} else
 	{
@@ -121,7 +138,6 @@ server_connected (server * serv)
 
 #ifdef USE_OPENSSL
 #define	SSLTMOUT	10	/* seconds */
-struct session *g_sess = NULL;
 void
 ssl_cb_info (SSL * s, int where, int ret)
 {
@@ -192,18 +208,63 @@ ssl_do_connect (server * serv)
 
 	if (SSL_is_init_finished (serv->ssl))
 	{
+		struct cert_info cert_info;
+		struct chiper_info *chiper_info;
 		int verify_error;
+		int i;
+
+		if (!_SSL_get_cert_info(&cert_info, serv->ssl)) {
+			snprintf(buf, sizeof(buf), "* Certification info:");
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			snprintf(buf, sizeof(buf), "  Subject:");
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			for (i = 0; cert_info.subject_word[i]; i++) {
+				snprintf(buf, sizeof(buf), "    %s", cert_info.subject_word[i]);
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			}
+			snprintf(buf, sizeof(buf), "  Issuer:");
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			for (i = 0; cert_info.issuer_word[i]; i++) {
+				snprintf(buf, sizeof(buf), "    %s", cert_info.issuer_word[i]);
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			}
+			snprintf(buf, sizeof(buf), "  Public key algorithm: %s (%d bits)",
+				cert_info.algorithm, cert_info.algorithm_bits);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			if (cert_info.rsa_tmp_bits) {
+				snprintf(buf, sizeof(buf), "  Public key algorithm uses ephemeral key with %d bits",
+					cert_info.rsa_tmp_bits);
+				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			}
+			snprintf(buf, sizeof(buf), "  Sign algorithm %s (%d bits)",
+				cert_info.sign_algorithm, cert_info.sign_algorithm_bits);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			snprintf(buf, sizeof(buf), "  Valid since %s to %s",
+				cert_info.notbefore, cert_info.notafter);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+		} else {
+			snprintf(buf, sizeof(buf), " * No Certificate");
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+		}
+
+		chiper_info = _SSL_get_cipher_info(serv->ssl);	/* static buffer */
+		snprintf(buf, sizeof(buf), "* Chiper info:");
+		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+		snprintf(buf, sizeof(buf), "  Version: %s, cipher %s (%u bits)",
+			    chiper_info->version, chiper_info->chiper, chiper_info->chiper_bits);
+		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
 
 		verify_error = SSL_get_verify_result(serv->ssl);
 		switch (verify_error) {
 		    case X509_V_OK:
-			snprintf (buf, sizeof (buf), "* Verify OK (?)");
-			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
+			/* snprintf (buf, sizeof (buf), "* Verify OK (?)"); */
+			/* EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0); */
 			break;
 		    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
 		    case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
+		    case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
 			if (serv->accept_invalid_cert) {
-				snprintf (buf, sizeof (buf), "* Verify error: %s.? (%d) -- Ignored",
+				snprintf (buf, sizeof (buf), "* Verify E: %s.? (%d) -- Ignored",
 					X509_verify_cert_error_string(verify_error), verify_error);
 				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session, buf, NULL, NULL, NULL, 0);
 				break;
@@ -223,11 +284,6 @@ ssl_do_connect (server * serv)
 
 		server_cleanup (serv);	  /* ->connecting = FALSE */
 
-		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session,
-						 _SSL_get_cipher_string (serv->ssl), NULL, NULL, NULL, 0);
-		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->front_session,
-						 _SSL_get_cert_string (serv->ssl), NULL, NULL, NULL, 0);
-
 		/* activate gtk poll */
 		server_connected (serv);
 
@@ -254,6 +310,152 @@ ssl_do_connect (server * serv)
 }
 #endif
 
+static int
+timeout_auto_reconnect (struct server *serv)
+{
+	if (is_server (serv))  /* make sure it hasnt been closed during the delay */
+	{
+		if (!serv->connected && !serv->connecting && serv->front_session)
+		{
+			connect_server (serv->front_session, serv->hostname, serv->port,
+								 FALSE);
+		}
+		serv->recondelay_tag = -1;
+	}
+	return 0;	/* returning 0 should remove the timeout handler */
+}
+
+void
+auto_reconnect (struct server *serv, int send_quit, int err)
+{
+	struct session *s;
+	GSList *list;
+	int del;
+
+	if (serv->front_session == NULL)
+		return;
+
+	list = sess_list;
+	while (list)					  /* make sure auto rejoin can work */
+	{
+		s = (struct session *) list->data;
+		if (is_channel (s->channel))
+		{
+			strcpy (s->waitchannel, s->channel);
+			strcpy (s->willjoinchannel, s->channel);
+		}
+		list = list->next;
+	}
+
+	if (serv->connected)
+		disconnect_server (serv->front_session, send_quit, err);
+
+	del = prefs.recon_delay * 1000;
+	if (del < 1000)
+		del = 500;		/* so it doesn't block the gui */
+
+	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
+}
+
+static void
+read_data (struct server *serv, int sok)
+{
+	int err, i, len;
+	char lbuf[2050];
+	char *temp;
+
+	while (1)
+	{
+		if (!EMIT_SIGNAL (XP_IF_RECV, &len, (void *) sok, &lbuf,
+								(void *) ((sizeof (lbuf)) - 2), NULL, 0))
+		{
+#ifdef USE_OPENSSL
+			if (!serv->ssl)
+#endif
+				len = recv (sok, lbuf, sizeof (lbuf) - 2, 0);
+#ifdef USE_OPENSSL
+			else
+				len = _SSL_recv (serv->ssl, lbuf, sizeof (lbuf) - 2);
+#endif
+		}
+		if (len < 1)
+		{
+			if (len < 0)
+			{
+				if (errno == EAGAIN || errno == EWOULDBLOCK)
+					return;
+				err = errno;
+			} else
+			{
+				err = 0;
+			}
+			if (prefs.autoreconnect)
+				auto_reconnect (serv, FALSE, err);
+			else
+				disconnect_server (serv->front_session, FALSE, err);
+			return;
+		} else
+		{
+			i = 0;
+
+			lbuf[len] = 0;
+
+			while (i < len)
+			{
+				switch (lbuf[i])
+				{
+				case '\r':
+					break;
+
+				case '\n':
+					serv->linebuf[serv->pos] = 0;
+#ifdef USE_TRANS
+					if (prefs.use_trans)
+						serv2user (serv->linebuf);
+#endif
+					if (prefs.stripcolor)
+					{
+						temp = strip_color (serv->linebuf);
+						process_line (serv, temp);
+						free (temp);
+					} else
+					{
+						process_line (serv, serv->linebuf);
+					}
+					serv->pos = 0;
+					break;
+
+				default:
+					serv->linebuf[serv->pos] = lbuf[i];
+					if (serv->pos > 519)
+						fprintf (stderr, "*** XCHAT WARNING: Buffer overflow - shit server!\n");
+					else
+						serv->pos++;
+				}
+				i++;
+			}
+		}
+	}
+}
+
+void
+flush_server_queue (struct server *serv)
+{
+	/*GSList *list = serv->outbound_queue;
+	void *data;*/
+
+	list_free (&serv->outbound_queue);
+
+/*	while (list)
+	{
+		data = list->data;
+		serv->outbound_queue = g_slist_remove (serv->outbound_queue, data);
+		free (data);
+		list = serv->outbound_queue;
+	}*/
+	serv->sendq_len = 0;
+}
+
 static void
 connected_signal (server * serv, int sok)
 {
@@ -269,13 +471,15 @@ connected_signal (server * serv, int sok)
 	{
 	case '1':						  /* unknown host */
 		server_cleanup (serv);
-		close (sok);
+		close (serv->sok);
 		EMIT_SIGNAL (XP_TE_UKNHOST, sess, NULL, NULL, NULL, NULL, 0);
+		if (prefs.autoreconnectonfail)
+			auto_reconnect (serv, FALSE, -1);
 		break;
 	case '2':						  /* connection failed */
 		waitline (serv->childread, tbuf, sizeof tbuf);
 		server_cleanup (serv);
-		close (sok);
+		close (serv->sok);
 		EMIT_SIGNAL (XP_TE_CONNFAIL, sess, errorstring (atoi (tbuf)), NULL,
 						 NULL, NULL, 0);
 		if (prefs.autoreconnectonfail)
@@ -286,7 +490,6 @@ connected_signal (server * serv, int sok)
 		waitline (serv->childread, ip, sizeof ip);
 		waitline (serv->childread, outbuf, sizeof outbuf);
 		EMIT_SIGNAL (XP_TE_CONNECT, sess, host, ip, outbuf, NULL, 0);
-		strcpy (serv->hostname, host);
 		break;
 	case '4':						  /* success */
 #ifdef USE_OPENSSL
@@ -297,8 +500,8 @@ connected_signal (server * serv, int sok)
  
  			/* it'll be a memory leak, if connection doesn't terminated by
  			   server_disconnect() */
- 			serv->ssl = _SSL_socket (serv->sok);
- 			if ((err = _SSL_set_verify (ssl_cb_verify, NULL)))
+			serv->ssl = _SSL_socket (ctx, serv->sok);
+			if ((err = _SSL_set_verify (ctx, ssl_cb_verify, NULL)))
   			{
  				EMIT_SIGNAL (XP_TE_CONNFAIL, serv->front_session, err, NULL,
  								 NULL, NULL, 0);
@@ -337,7 +540,7 @@ connected_signal (server * serv, int sok)
 		prefs.local_ip = inet_addr (tbuf);
 		break;
 	case '7':						  /* gethostbyname (prefs.hostname) failed */
-		sprintf (outbuf, "Cannot resolv hostname %s\nCheck your IP Settings!",
+		sprintf (outbuf, "Cannot resolve hostname %s\nCheck your IP Settings!",
 					prefs.hostname);
 		PrintText (sess, outbuf);
 		break;
@@ -373,12 +576,12 @@ check_connecting (struct session *sess)
 	return FALSE;
 }
 
-int
+/*int
 close_socket (int sok)
 {
 	close (sok);
 	return 0;
-}
+}*/
 
 void
 disconnect_server (struct session *sess, int sendquit, int err)
@@ -428,19 +631,13 @@ disconnect_server (struct session *sess, int sendquit, int err)
 
 	sess = orig_sess;
 	if (sendquit)
-	{
-		char tbuf[256];
-		if (!sess->quitreason)
-			sess->quitreason = prefs.quitreason;
-		snprintf (tbuf, sizeof tbuf, "QUIT :%s\r\n", sess->quitreason);
-		sess->quitreason = 0;
-		tcp_send (serv, tbuf);
-	}
+		server_sendquit (sess);
 
 	/* close it in 5 seconds so the QUIT doesn't get lost. Technically      *
 	 * we should send a QUIT and then wait for the server to disconnect us, *
 	   but that would hold up the GUI                                       */
-	fe_timeout_add (5000, close_socket, (void *) serv->sok);
+	/*fe_timeout_add (5000, close_socket, (void *) serv->sok);*/
+	close (serv->sok);
 
 #ifdef USE_OPENSSL
 	if (serv->ssl)
@@ -512,49 +709,49 @@ traverse_socks (int sok, unsigned long serverAddr, int port)
 
 struct sock5_connect1
 {
-       char version;
-       char nmethods;
-       char method;
+	char version;
+	char nmethods;
+	char method;
 };
 
 struct sock5_connect2
 {
-       char version;
-       char cmd;
-       char rsv;
-       char atyp;
-       unsigned long address;
-       unsigned short port;
+	char version;
+	char cmd;
+	char rsv;
+	char atyp;
+	unsigned long address;
+	unsigned short port;
 };
 
 static int
 traverse_socks5 (int sok, unsigned long serverAddr, int port)
 {
-       struct sock5_connect1 sc1;
-       struct sock5_connect2 sc2;
-       unsigned char buf[10];
+	struct sock5_connect1 sc1;
+	struct sock5_connect2 sc2;
+	unsigned char buf[10];
 
-       sc1.version = 5;
-       sc1.nmethods = 1;
-       sc1.method = 0;
-       send (sok, &sc1, 3, 0);
-       recv (sok, buf, 2, 0);
-       if (buf[0] != 5 && buf[1] != 0)
-               return 1;
+	sc1.version = 5;
+	sc1.nmethods = 1;
+	sc1.method = 0;
+	send (sok, &sc1, 3, 0);
+	recv (sok, buf, 2, 0);
+	if (buf[0] != 5 && buf[1] != 0)
+		return 1;
 
-       sc2.version = 5;
-       sc2.cmd = 1;
-       sc2.rsv = 0;
-       sc2.atyp = 1;
-       sc2.address = serverAddr;
-       sc2.port = htons (port);
-       
-       send (sok, &sc2, 10, 0);
-       recv (sok, buf, 10, 0);
-       if (buf[0] != 5 && buf[1] != 0)
-               return 1;
+	sc2.version = 5;
+	sc2.cmd = 1;
+	sc2.rsv = 0;
+	sc2.atyp = 1;
+	sc2.address = serverAddr;
+	sc2.port = htons (port);
 
-       return 0;
+	send (sok, &sc2, 10, 0);
+	recv (sok, buf, 10, 0);
+	if (buf[0] != 5 && buf[1] != 0)
+		return 1;
+
+	return 0;
 }
 
 static int
@@ -591,6 +788,9 @@ void
 connect_server (struct session *sess, char *server, int port, int no_login)
 {
 	int sok, sw, pid, read_des[2];
+#ifdef USE_LINGER
+	struct linger lin;
+#endif
 
 	if (!server[0])
 		return;
@@ -614,14 +814,23 @@ connect_server (struct session *sess, char *server, int port, int no_login)
 	setsockopt (sok, SOL_SOCKET, SO_REUSEADDR, (char *) &sw, sizeof (sw));
 	sw = 1;
 	setsockopt (sok, SOL_SOCKET, SO_KEEPALIVE, (char *) &sw, sizeof (sw));
+#ifdef USE_LINGER
+	lin.l_onoff = 1;
+	lin.l_linger = 1;
+	setsockopt (sok, SOL_SOCKET, SO_LINGER, (char *) &lin, sizeof (lin));
+#endif
 
 	strcpy (sess->server->servername, server);
+	strcpy (sess->server->hostname, server);
+
 	sess->server->nickcount = 1;
 	sess->server->connecting = TRUE;
 	sess->server->sok = sok;
 	sess->server->port = port;
 	sess->server->end_of_motd = FALSE;
 	sess->server->no_login = no_login;
+	sess->server->is_away = FALSE;
+	fe_set_away (sess->server);
 	flush_server_queue (sess->server);
 
 	if (pipe (read_des) < 0)
