@@ -37,29 +37,15 @@ extern struct xchatprefs prefs;
 
 extern unsigned char *strip_color (unsigned char *text);
 extern void xchat_exec (char *cmd);
-extern void check_special_chars (char *);
+extern void check_special_chars (char *, int perc_ascii);
 extern int child_handler (int pid);
 extern int auto_insert (char *dest, char *src, char *word[], char *word_eol[],
 								char *a, char *c, char *d, char *h, char *n, char *s);
 extern int perl_print (char *cmd, struct session *sess, char *b, char *c,
 							  char *d, char *e);
 
-extern char **environ;
-
 int pevt_build_string (char *input, char **output, int *max_arg);
 
-int
-timecat (char *buf)
-{
-	time_t timval = time (0);
-	char *tim = ctime (&timval) + 10;
-	tim[0] = '[';
-	tim[9] = ']';
-	tim[10] = ' ';
-	tim[11] = 0;
-	strcat (buf, tim);
-	return 11;
-}
 
 #ifdef USE_HEBREW
 
@@ -179,9 +165,8 @@ PrintText (struct session *sess, unsigned char *text)
 		time_t timval = time (0);
 		char *tim = ctime (&timval) + 11;
 		char *temp = strip_color (text);
-		tim[9] = 0;
-		if (prefs.timestamp)
-			write (sess->logfd, tim, 9);	/*strlen (tim)); */
+		if (prefs.timestamp || prefs.timestamp_logs)
+			write (sess->logfd, tim, 9);
 		write (sess->logfd, temp, strlen (temp));
 		free (temp);
 	}
@@ -338,13 +323,13 @@ static char *pevt_join_help[] = {
 };
 
 static char *pevt_chanaction_help[] = {
-	"Nickname (which may contain color)",
+	"Nickname",
 	"The action",
 	NULL
 };
 
 static char *pevt_chanmsg_help[] = {
-	"Nickname (which may contain color)",
+	"Nickname",
 	"The text",
 	NULL
 };
@@ -881,6 +866,14 @@ static char *pevt_uinvite_help[] = {
 	NULL
 };
 
+static char *pevt_banlist_help[] = {
+	"Channel",
+	"Banmask",
+	"Who set the ban",
+	"Ban time",
+	NULL
+};
+
 struct text_event
 {
 	char *name;
@@ -924,10 +917,11 @@ struct text_event te[] = {
 		{"Join", pevt_join_help,
 	 "-%C10-%C11>%O$t%B$1%B %C14(%C10$3%C14)%C has joined $2", 3, NULL},
 	{"Channel Action", pevt_chanaction_help, "%C13*%O$t$1 $2%O", 2, NULL},
+	{"Channel Action Hilight", pevt_chanaction_help, "%C13*%O$t%C8%B$1%B%O $2%O", 2, NULL},
 	{"Channel Message", pevt_chanmsg_help, "%C2<%O$1%C2>%O$t$2%O", 2, NULL},
+	{"Channel Msg Hilight", pevt_chanmsg_help, "%C2<%C8%B$1%B%C2>%O$t$2%O", 2, NULL},
 	{"Private Message", pevt_privmsg_help, "%C12*%C13$1%C12*$t%O$2%O", 2, NULL},
-	
-		{"Change Nick", pevt_changenick_help,
+	{"Change Nick", pevt_changenick_help,
 	 "-%C10-%C11-%O$t$1 is now known as $2", 2, NULL},
 	{"New Topic", pevt_newtopic_help,
 	 "-%C10-%C11-%O$t$1 has changed the topic to: $2%O", 3, NULL},
@@ -1185,7 +1179,7 @@ struct text_event te[] = {
 	{"Your Invitation", pevt_uinvite_help,
 		"-%C10-%C11-%O$tYou're inviting %C11$1%C to %C11$2%C (%C11$3%C)", 3,
 		NULL},
-
+	{"Ban List", pevt_banlist_help, "-%C10-%C11-%O$t$1 Banlist: %C3$4 %C4$2 %C5$3%O", 4, NULL},
 	{NULL, NULL, NULL, 0, NULL}, /* XP_IF_SEND */
 	{NULL, NULL, NULL, 0, NULL}, /* XP_IF_RECV */
 };
@@ -1245,7 +1239,7 @@ pevent_make_pntevts ()
 				abort ();
 			}
 		}
-		check_special_chars (pntevts[i]);
+		check_special_chars (pntevts[i], TRUE);
 	}
 }
 
@@ -1404,10 +1398,8 @@ pevent_check_all_loaded ()
 			continue;
 		if (pntevts_text[i] == NULL)
 		{
-
-			printf ("%s\n", te[i].name);
-
-			/*snprintf(out, sizeof(out), "The data for event %s failed to load. Reverting to defaults.\nThis may be because a new version of XChat is loading an old config file.\n\nCheck all print event texts are correct", evtnames[i]);
+			/*printf ("%s\n", te[i].name);
+			snprintf(out, sizeof(out), "The data for event %s failed to load. Reverting to defaults.\nThis may be because a new version of XChat is loading an old config file.\n\nCheck all print event texts are correct", evtnames[i]);
 			   gtkutil_simpledialog(out); */
 			len = strlen (te[i].def) + 1;
 			pntevts_text[i] = malloc (len);
@@ -1519,7 +1511,7 @@ pevt_build_string (char *input, char **output, int *max_arg)
 	len = strlen (input);
 	i = malloc (len + 1);
 	memcpy (i, input, len + 1);
-	check_special_chars (i);
+	check_special_chars (i, TRUE);
 
 	len = strlen (i);
 
@@ -1611,7 +1603,7 @@ pevt_build_string (char *input, char **output, int *max_arg)
 
 			continue;
 		}
-		if (d < '0' || d > '9')
+		if (d < '1' || d > '9')
 		{
 			snprintf (o, sizeof (o), "Error, invalid argument $%c\n", d);
 			fe_message (o, FALSE);
