@@ -19,16 +19,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "../../config.h"
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include "xchat.h"
 #include "fe.h"
 
 extern struct xchatprefs prefs;
 extern GSList *sess_list;
 
-extern int notify_isnotify (struct session *sess, char *name);
-extern void dcc_send (struct session *sess, char *tbuf, char *to, char *file);
-
-int add_name (struct session *sess, char *name, char *hostname);
+void add_name (struct session *sess, char *name, char *hostname);
 int sub_name (struct session *sess, char *name);
 static void update_entry (struct session *sess, struct User *user);
 
@@ -98,15 +99,9 @@ nick_cmp (struct User *user1, struct User *user2)
 	case 1:
 		return strcasecmp (user1->nick, user2->nick);
 	case 2:
-		if (nick_cmp_az_ops (user1, user2) < 0)
-			return 1;
-		else
-			return -1;
+		return -1 * nick_cmp_az_ops (user1, user2);
 	case 3:
-		if (strcasecmp (user1->nick, user2->nick) < 0)
-			return 1;
-		else
-			return -1;
+		return -1 * strcasecmp (user1->nick, user2->nick);
 	default:
 		return 1;
 	}
@@ -127,7 +122,7 @@ clear_user_list (struct session *sess)
 		if (user->hostname)
 			free (user->hostname);
 		if (user->servername);
-		free (user->servername);
+			free (user->servername);
 		free (user);
 		list = g_slist_remove (list, user);
 	}
@@ -319,12 +314,15 @@ sub_name (struct session *sess, char *name)
 	return TRUE;
 }
 
-int
+void
 add_name (struct session *sess, char *name, char *hostname)
 {
 	struct User *user;
 	int row;
 
+	if (find_name (sess, name))
+		return;
+
 	user = malloc (sizeof (struct User));
 	memset (user, 0, sizeof (struct User));
 
@@ -359,7 +357,7 @@ add_name (struct session *sess, char *name, char *hostname)
 	fe_userlist_insert (sess, user, row);
 	fe_userlist_numbers (sess);
 
-	return row;
+	return;
 }
 
 void
