@@ -27,5 +27,5 @@ extern void ul_op_name (struct session *sess, char *name);
 extern void deop_name (struct session *sess, char *name);
 extern void ul_hop_name (struct session *sess, char *name);
 extern void dehop_name (struct session *sess, char *name);
-extern int add_name (struct session *sess, char *name, char *hostname);
+extern void add_name (struct session *sess, char *name, char *hostname);
 extern int sub_name (struct session *sess, char *name);
