@@ -46,12 +46,6 @@ extern int g_snprintf (char *str, size_t n, const char *format, ...);
 
 #ifdef USE_DEBUG
 
-#define malloc malloc
-#define realloc realloc
-#define free free
-#undef strdup
-#define strdup strdup
-
 int current_mem_usage;
 
 struct mem_block
@@ -166,6 +160,11 @@ xchat_free (void *buf)
 	free (cur);
 }
 
+#define malloc xchat_malloc
+#define realloc xchat_realloc
+#define free xchat_free
+#define strdup xchat_strdup
+
 #endif /* MEMORY_DEBUG */
 
 int
@@ -893,7 +892,8 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
     int i, buflen;
 
     buflen = strlen(s) + 1;
-    bufStart = buf = alloca(buflen);
+/*    bufStart = buf = alloca(buflen);*/
+	 bufStart = buf = malloc (buflen);
     memset(buf, '\0', buflen);
 
     src = s;
@@ -960,6 +960,8 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
     *argvPtr = (char **)argv2;	/* XXX don't change the API */
     *argcPtr = argc;
 
+	 free (bufStart);
+
     return 0;
 }
 
