@@ -23,9 +23,6 @@
 #include "xchat.h"
 #include "fe.h"
 #include "util.h"
-#include <netdb.h>
-#include <arpa/inet.h>
-#include <errno.h>
 #include <signal.h>
 #include <time.h>
 #include "cfgfiles.h"
@@ -59,12 +56,6 @@ int notify_tag = -1;
 int xchat_is_quitting = 0;
 int auto_connect = 1;
 
-extern GSList *ctcp_list;
-extern GSList *popup_list;
-extern GSList *button_list;
-extern GSList *command_list;
-extern GSList *replace_list;
-
 #ifdef USE_PERL
 extern struct session *perl_sess;
 #endif
@@ -73,8 +64,12 @@ struct session *menu_sess = 0;
 struct xchatprefs prefs;
 
 void xchat_cleanup (void);
-struct session *new_session (struct server *serv, char *from);
 void auto_reconnect (struct server *serv, int send_quit, int err);
+int is_server (server * serv);
+
+#ifdef USE_OPENSSL
+SSL_CTX *ctx = NULL;
+#endif
 
 /* trans */
 extern void serv2user (unsigned char *);
@@ -82,10 +77,6 @@ extern void user2serv (unsigned char *);
 extern int load_trans_table (char *full_path);
 /* ~trans */
 
-/* inbound.c */
-
-extern void process_line (struct server *serv, char *buf);
-
 /* plugin.c */
 
 extern void module_setup ();
@@ -93,15 +84,14 @@ extern void signal_setup ();
 
 /* server.c */
 
+extern void flush_server_queue (struct server *serv);
 extern int close_socket (int sok);
 extern void connecting_fin (struct session *);
 extern void connect_server (struct session *sess, char *server, int port,
 									 int quiet);
 extern void disconnect_server (struct session *sess, int sendquit, int err);
-
-/* userlist.c */
-
-extern struct User *find_name (struct session *sess, char *name);
+extern void server_sendquit (session *sess);
+extern void server_sendpart (server *serv, char *channel, char *reason);
 
 /* notify.c */
 
@@ -115,7 +105,6 @@ extern void notify_cleanup (void);
 
 void perl_init (struct session *, int);
 void perl_end (void);
-int perl_inbound (struct session *sess, struct server *serv, char *buf);
 #endif
 
 /* python.c */
@@ -123,13 +112,8 @@ int perl_inbound (struct session *sess, struct server *serv, char *buf);
 extern void pys_init ();
 extern void pys_kill ();
 
-/* editlist.c */
-
-extern void list_loadconf (char *, GSList **, char *);
-
 /* text.c */
 
-extern unsigned char *strip_color (unsigned char *text);
 extern void end_logging (int fd);
 extern void load_text_events ();
 extern void pevent_dialog_save (char *fn);
@@ -141,55 +125,90 @@ extern void ssl_cb_info (SSL * s, int where, int ret);
 #endif
 
 
-void auto_reconnect (struct server *serv, int send_quit, int err);
+/* actually send to the socket. This might do a character translation or
+   send via SSL */
 
-/* anything above SEND_MAX bytes in 1 second is
-   queued and sent QUEUE_TIMEOUT milliseconds later */
+static int
+tcp_send_real (struct server *serv, char *buf, int len)
+{
+	int ret = 1;
+	unsigned char *tbuf = buf;
+#ifdef USE_TRANS
+#define TRANS_STAT_BUF_LEN 1024
+	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
 
-#define SEND_MAX 256
-#define QUEUE_TIMEOUT 2500
+	if (prefs.use_trans)
+	{
+		if (len >= TRANS_STAT_BUF_LEN)
+			tbuf = malloc (len + 1);
+		else
+			tbuf = sbuf;
+		if (!tbuf)
+			return -1;
+		strcpy (tbuf, buf);
+		user2serv (tbuf);
+	}
+#endif
 
-/* this whole throttling system is lame, can anyone suggest
-   a better system? */
+	fe_add_rawlog (serv, tbuf, TRUE);
+
+	if (!EMIT_SIGNAL (XP_IF_SEND, (void *) serv->sok, tbuf, (void *)len,
+							NULL, NULL, 0))
+	{
+#ifdef USE_OPENSSL
+		if (!serv->ssl)
+			ret = send (serv->sok, tbuf, len, 0);
+		else
+			ret = _SSL_send (serv->ssl, tbuf, len);
+#else
+		ret = send (serv->sok, tbuf, len, 0);
+#endif
+	}
+
+#ifdef USE_TRANS
+	if (prefs.use_trans)
+	{
+		if (tbuf != sbuf)
+			free (tbuf);
+	}
+#endif
+
+	return ret;
+}
+
+/* new throttling system, uses the same method as the Undernet
+   ircu2.10 server; under test, a 200-line paste didn't flood
+   off the client */
 
 static int
 tcp_send_queue (struct server *serv)
 {
-	char *buf;
-	int len;
-	GSList *list = serv->outbound_queue;
+	char *buf, *p;
+	int len, i;
+	GSList *list;
 	time_t now = time (0);
 
+	/* did the server close since the timeout was added? */
+	if (!is_server (serv))
+		return 0;
+
+	list = serv->outbound_queue;
+
 	while (list)
 	{
 		buf = (char *) list->data;
 		len = strlen (buf);
 
-		if (serv->last_send != now)
-		{
-			serv->last_send = now;
-			serv->bytes_sent = 0;
-		} else
-		{
-			serv->bytes_sent += len;
-		}
-
-		if (serv->bytes_sent > SEND_MAX)
+		if (serv->next_send < now)
+			serv->next_send = now;
+		if (serv->next_send - now >= 10)
 			return 1;				  /* don't remove the timeout handler */
 
-		fe_add_rawlog (serv, buf, TRUE);
-		if (!EMIT_SIGNAL
-			 (XP_IF_SEND, (void *) serv->sok, buf, (void *) len, NULL, NULL, 0))
-		{
-#ifdef USE_OPENSSL
-			if (!serv->ssl)
-#endif
-				send (serv->sok, buf, len, 0);
-#ifdef USE_OPENSSL
-			else
-				_SSL_send (serv->ssl, buf, len);
-#endif
-		}
+		for (p = buf, i = len; i && *p != ' '; p++, i--);
+			serv->next_send += (2 + i / 120);
+		serv->sendq_len -= len;
+
+		tcp_send_real (serv, buf, len);
 
 		serv->outbound_queue = g_slist_remove (serv->outbound_queue, buf);
 		free (buf);
@@ -202,80 +221,21 @@ tcp_send_queue (struct server *serv)
 int
 tcp_send_len (struct server *serv, char *buf, int len)
 {
-#ifdef USE_TRANS
-	unsigned char *tbuf;
-#define TRANS_STAT_BUF_LEN 1024
-	static unsigned char sbuf[TRANS_STAT_BUF_LEN];
-	int ret;
-#endif
-	time_t now = time (0);
+	char *dbuf;
+	int noqueue = !serv->outbound_queue;
 
-	if (serv->last_send != now)
-	{
-		serv->last_send = now;
-		serv->bytes_sent = 0;
-	} else
-	{
-		serv->bytes_sent += len;
-	}
+	if (!prefs.throttle)
+		return tcp_send_real (serv, buf, len);
 
-	if (prefs.throttle && (serv->bytes_sent > SEND_MAX || serv->outbound_queue))
-	{
-		buf = strdup (buf);
-		if (!serv->outbound_queue)
-			fe_timeout_add (QUEUE_TIMEOUT, tcp_send_queue, serv);
-		serv->outbound_queue = g_slist_append (serv->outbound_queue, buf);
-		return 1;
-	}
-#ifdef USE_TRANS
-	if (prefs.use_trans)
-	{
-		if (len >= TRANS_STAT_BUF_LEN)
-			tbuf = malloc (len + 1);
-		else
-			tbuf = sbuf;
-		if (!tbuf)
-		{
-			return -1;
-		}
+	dbuf = malloc (len + 1);
+	memcpy (dbuf, buf, len);
+	dbuf[len] = 0;			/* tcp_send_queue uses strlen */
+	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
+	serv->sendq_len += len; 
 
-		strcpy (tbuf, buf);
-		user2serv (tbuf);
-		fe_add_rawlog (serv, buf, TRUE);
-		if (!EMIT_SIGNAL
-			 (XP_IF_SEND, (void *) serv->sok, buf, (void *) len, NULL, NULL, 0))
-		{
-#ifdef USE_OPENSSL
-			if (!serv->ssl)
-#endif
-				ret = send (serv->sok, tbuf, len, 0);
-#ifdef USE_OPENSSL
-			else
-				ret = _SSL_send (serv->ssl, tbuf, len);
-#endif
-		} else
-		{
-			ret = 1;
-		}
-		if (tbuf != sbuf)
-			free (tbuf);
-		return ret;
-	}
-#endif
-	fe_add_rawlog (serv, buf, TRUE);
+	if (tcp_send_queue (serv) && noqueue)
+		fe_timeout_add (500, tcp_send_queue, serv);
 
-	if (!EMIT_SIGNAL
-		 (XP_IF_SEND, (void *) serv->sok, buf, (void *) len, NULL, NULL, 0))
-	{
-#ifdef USE_OPENSSL
-		if (!serv->ssl)
-#endif
-			return send (serv->sok, buf, len, 0);
-#ifdef USE_OPENSSL
-		else
-			return _SSL_send (serv->ssl, buf, len);
-#endif
-	}
 	return 1;
 }
 
@@ -366,6 +326,12 @@ find_session_from_nick (char *nick, struct server *serv)
 			return serv->front_session;
 	}
 
+	if (menu_sess && menu_sess->server == serv)
+	{
+		if (find_name (menu_sess, nick))
+			return menu_sess;
+	}
+
 	while (list)
 	{
 		sess = (struct session *) list->data;
@@ -379,146 +345,28 @@ find_session_from_nick (char *nick, struct server *serv)
 	return 0;
 }
 
-static int
-timeout_auto_reconnect (struct server *serv)
+struct session *
+new_session (struct server *serv, char *from)
 {
-	if (is_server (serv))  /* make sure it hasnt been closed during the delay */
+	struct session *sess;
+
+	sess = malloc (sizeof (struct session));
+	memset (sess, 0, sizeof (struct session));
+
+	sess->server = serv;
+	sess->logfd = -1;
+
+	if (from)
 	{
-		if (!serv->connected && !serv->connecting && serv->front_session)
-		{
-			connect_server (serv->front_session, serv->hostname, serv->port,
-								 FALSE);
-		}
-		serv->recondelay_tag = -1;
+		sess->is_dialog = TRUE;
+		strcpy (sess->channel, from);
 	}
-	return 0;	/* returning 0 should remove the timeout handler */
-}
 
-void
-auto_reconnect (struct server *serv, int send_quit, int err)
-{
-	struct session *s;
-	GSList *list;
+	sess_list = g_slist_prepend (sess_list, sess);
 
-	if (serv->front_session == NULL)
-		return;
+	fe_new_window (sess);
 
-	list = sess_list;
-	while (list)					  /* make sure auto rejoin can work */
-	{
-		s = (struct session *) list->data;
-		if (is_channel (s->channel))
-		{
-			strcpy (s->waitchannel, s->channel);
-			strcpy (s->willjoinchannel, s->channel);
-		}
-		list = list->next;
-	}
-	disconnect_server (serv->front_session, send_quit, err);
-
-	if (prefs.recon_delay)
-		serv->recondelay_tag =
-		fe_timeout_add (prefs.recon_delay * 1000, timeout_auto_reconnect, serv);
-	else
-		connect_server (serv->front_session, serv->hostname, serv->port, FALSE);
-}
-
-void
-read_data (struct server *serv, int sok)
-{
-	int err, i, len;
-	char lbuf[2050];
-	char *temp;
-
-	while (1)
-	{
-		if (!EMIT_SIGNAL (XP_IF_RECV, &len, (void *) sok, &lbuf,
-								(void *) ((sizeof (lbuf)) - 2), NULL, 0))
-		{
-#ifdef USE_OPENSSL
-			if (!serv->ssl)
-#endif
-				len = recv (sok, lbuf, sizeof (lbuf) - 2, 0);
-#ifdef USE_OPENSSL
-			else
-				len = _SSL_recv (serv->ssl, lbuf, sizeof (lbuf) - 2);
-#endif
-		}
-		if (len < 1)
-		{
-			if (len < 0)
-			{
-				if (errno == EAGAIN || errno == EWOULDBLOCK)
-					return;
-				err = errno;
-			} else
-			{
-				err = 0;
-			}
-			if (prefs.autoreconnect)
-				auto_reconnect (serv, FALSE, err);
-			else
-				disconnect_server (serv->front_session, FALSE, err);
-			return;
-		} else
-		{
-			i = 0;
-
-			lbuf[len] = 0;
-
-			while (i < len)
-			{
-				switch (lbuf[i])
-				{
-				case '\r':
-					break;
-
-				case '\n':
-					serv->linebuf[serv->pos] = 0;
-#ifdef USE_TRANS
-					if (prefs.use_trans)
-						serv2user (serv->linebuf);
-#endif
-					if (prefs.stripcolor)
-					{
-						temp = strip_color (serv->linebuf);
-						process_line (serv, temp);
-						free (temp);
-					} else
-					{
-						process_line (serv, serv->linebuf);
-					}
-					serv->pos = 0;
-					break;
-
-				default:
-					serv->linebuf[serv->pos] = lbuf[i];
-					if (serv->pos > 519)
-						fprintf (stderr, "*** XCHAT WARNING: Buffer overflow - shit server!\n");
-					else
-						serv->pos++;
-				}
-				i++;
-			}
-		}
-	}
-}
-
-void
-flush_server_queue (struct server *serv)
-{
-	GSList *list = serv->outbound_queue;
-	void *data;
-
-	while (list)
-	{
-		data = list->data;
-		serv->outbound_queue = g_slist_remove (serv->outbound_queue, data);
-		free (data);
-		list = serv->outbound_queue;
-	}
-	serv->last_send = 0;
-	serv->bytes_sent = 0;
+	return sess;
 }
 
 struct server *
@@ -551,30 +399,6 @@ new_server (void)
 	return serv;
 }
 
-struct session *
-new_session (struct server *serv, char *from)
-{
-	struct session *sess;
-
-	sess = malloc (sizeof (struct session));
-	memset (sess, 0, sizeof (struct session));
-
-	sess->server = serv;
-	sess->logfd = -1;
-
-	if (from)
-	{
-		sess->is_dialog = TRUE;
-		strcpy (sess->channel, from);
-	}
-
-	sess_list = g_slist_prepend (sess_list, sess);
-
-	fe_new_window (sess);
-
-	return sess;
-}
-
 static void
 kill_server_callback (server * serv)
 {
@@ -582,7 +406,8 @@ kill_server_callback (server * serv)
 	{
 		if (serv->iotag != -1)
 			fe_input_remove (serv->iotag);
-		fe_timeout_add (5000, close_socket, (void *) serv->sok);
+		/*fe_timeout_add (5000, close_socket, (void *) serv->sok);*/
+		close (serv->sok);
 	}
 	if (serv->connecting)
 	{
@@ -601,7 +426,7 @@ kill_server_callback (server * serv)
 		close (serv->childwrite);
 		close (serv->sok);
 	}
-	fe_server_callback (serv);
+/*	fe_server_callback (serv);*/
 
 	serv_list = g_slist_remove (serv_list, serv);
 
@@ -667,7 +492,6 @@ static void
 send_quit_or_part (session * killsess)
 {
 	int willquit = TRUE;
-	char tbuf[256];
 	GSList *list;
 	session *sess;
 	server *killserv = killsess->server;
@@ -695,9 +519,7 @@ send_quit_or_part (session * killsess)
 			if (!killserv->sent_quit)
 			{
 				flush_server_queue (killserv);
-				snprintf (tbuf, sizeof tbuf, "QUIT :%s\r\n",
-							 killsess->quitreason);
-				tcp_send (killserv, tbuf);
+				server_sendquit (killsess);
 				killserv->sent_quit = TRUE;
 			}
 		} else
@@ -705,8 +527,7 @@ send_quit_or_part (session * killsess)
 			if (!killsess->is_dialog && !killsess->is_server
 				 && killsess->channel[0])
 			{
-				snprintf (tbuf, sizeof tbuf, "PART %s\r\n", killsess->channel);
-				tcp_send (killserv, tbuf);
+				server_sendpart (killserv, killsess->channel, 0);
 			}
 		}
 	}
@@ -719,9 +540,6 @@ kill_session_callback (session * killsess)
 	session *sess;
 	GSList *list;
 
-	if (!killsess->quitreason)
-		killsess->quitreason = prefs.quitreason;
-
 	if (current_tab == killsess)
 		current_tab = NULL;
 
@@ -784,7 +602,6 @@ free_sessions (void)
 	while (list)
 	{
 		sess = (struct session *) list->data;
-		/*send_quit_or_part (sess); */
 		fe_close_window (sess);
 		list = sess_list;
 	}
@@ -822,7 +639,7 @@ save_away_message (struct server *serv, char *nick, char *msg)
 		if (away)
 		{
 			away->server = serv;
-			strcpy (away->nick, nick);
+			safe_strcpy (away->nick, nick, sizeof (away->nick));
 			away->message = strdup (msg);
 			away_list = g_slist_prepend (away_list, away);
 		}
@@ -863,7 +680,7 @@ xchat_mail_check (void)
 	maildir = getenv ("MAIL");
 	if (!maildir)
 	{
-		snprintf (buf, 511, "/var/spool/mail/%s", g_get_user_name ());
+		snprintf (buf, sizeof (buf) - 1, "/var/spool/mail/%s", g_get_user_name ());
 		maildir = buf;
 	}
 
@@ -882,7 +699,7 @@ xchat_mail_check (void)
 	{
 		sprintf (buf, "%d", mail_items (maildir));
 		sprintf (buf + 16, "%d", size);
-		if (menu_sess && !menu_sess->is_server)
+		if (menu_sess && !menu_sess->is_shell)
 			EMIT_SIGNAL (XP_TE_NEWMAIL, menu_sess, buf, buf + 16, NULL, NULL, 0);
 	}
 
@@ -932,12 +749,10 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 
 	count++;
 
-	if (count == 5 && prefs.lagometer)
-	{
+	if (count == 7 && prefs.lagometer)	/* every 30 seconds */
 		lag_check ();
-	}
 
-	if (count == 10)
+	if (count == 15)							/* every 30 seconds */
 	{
 		count = 0;
 		if (prefs.mail_check)
@@ -948,78 +763,87 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 }
 
 #define defaultconf_ctcp \
-	"NAME TIME\n"		"CMD /nctcp %s TIME %t\n\n"\
-	"NAME PING\n"		"CMD /nctcp %s PING %d\n\n"
+	"NAME TIME\n"				"CMD /nctcp %s TIME %t\n\n"\
+	"NAME PING\n"				"CMD /nctcp %s PING %d\n\n"
 
 #define defaultconf_popup \
-	"NAME SUB\n"					"CMD CTCP\n\n"\
-	"NAME Version\n"				"CMD /ctcp %s VERSION\n\n"\
-	"NAME Userinfo\n"				"CMD /ctcp %s USERINFO\n\n"\
-	"NAME Clientinfo\n"			"CMD /ctcp %s CLIENTINFO\n\n"\
-	"NAME Ping\n"					"CMD /ping %s\n\n"\
-	"NAME Time\n"					"CMD /ctcp %s TIME\n\n"\
-	"NAME Finger\n"				"CMD /ctcp %s FINGER\n\n"\
-	"NAME XDCC List\n"			"CMD /ctcp %s XDCC LIST\n\n"\
-	"NAME CDCC List\n"			"CMD /ctcp %s CDCC LIST\n\n"\
-	"NAME ENDSUB\n"				"CMD \n\n"\
-	"NAME SUB\nCMD DCC\n\n"\
-	"NAME Send File\nCMD /dcc send %s\n\n"\
-	"NAME Offer Chat\nCMD /dcc chat %s\n\n"\
-	"NAME Abort Chat\nCMD /dcc close chat %s\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\nCMD Oper\n\n"\
-	"NAME Kill\nCMD /quote KILL %s :die!\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\nCMD Mode\n\n"\
-	"NAME Give Voice\nCMD /voice %s\n\n"\
-	"NAME Take Voice\nCMD /devoice %s\n"\
-	"NAME SEP\nCMD \n\n"\
-	"NAME Give Ops\nCMD /op %s\n\n"\
-	"NAME Take Ops\nCMD /deop %s\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\nCMD Ignore\n\n"\
-	"NAME Ignore User\nCMD /ignore %s!*@* ALL\n\n"\
-	"NAME UnIgnore User\nCMD /unignore %s!*@*\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\nCMD Kick/Ban\n\n"\
-	"NAME Kick\nCMD /kick %s\n\n"\
-	"NAME Ban *!*@*.host\nCMD /ban %s 0\n\n"\
-	"NAME Ban *!*@domain\nCMD /ban %s 1\n\n"\
-	"NAME Ban *!user@*.host\nCMD /ban %s 2\n\n"\
-	"NAME Ban *!user@domain\nCMD /ban %s 3\n\n"\
-	"NAME Kick&Ban\nCMD /kickban %s\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\nCMD Info\n\n"\
-	"NAME Who\nCMD /quote WHO %s\n\n"\
-	"NAME Whois\nCMD /quote WHOIS %s\n\n"\
-	"NAME DNS Lookup\nCMD /dns %s\n\n"\
-	"NAME Trace\nCMD /quote TRACE %s\n\n"\
-	"NAME UserHost\nCMD /quote USERHOST %s\n\n"\
-	"NAME ENDSUB\nCMD \n\n"\
-	"NAME SUB\n"			"CMD External\n\n"\
-	"NAME Traceroute\n"	"CMD !xterm -e /bin/sh -c \"traceroute %h ; sleep 30\"\n\n"\
-	"NAME Ping\n"			"CMD !xterm -e /bin/sh -c \"ping -c 4 %h ; sleep 30\"\n\n"\
-	"NAME Telnet\n"		"CMD !xterm -e telnet %h\n\n"\
-	"NAME ENDSUB\n"		"CMD \n\n"\
-	"NAME Open Query\n"	"CMD /query %s\n\n"
+	"NAME SUB\n"				"CMD CTCP\n\n"\
+	"NAME Version\n"			"CMD /ctcp %s VERSION\n\n"\
+	"NAME Userinfo\n"			"CMD /ctcp %s USERINFO\n\n"\
+	"NAME Clientinfo\n"		"CMD /ctcp %s CLIENTINFO\n\n"\
+	"NAME Ping\n"				"CMD /ping %s\n\n"\
+	"NAME Time\n"				"CMD /ctcp %s TIME\n\n"\
+	"NAME Finger\n"			"CMD /ctcp %s FINGER\n\n"\
+	"NAME XDCC List\n"		"CMD /ctcp %s XDCC LIST\n\n"\
+	"NAME CDCC List\n"		"CMD /ctcp %s CDCC LIST\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD DCC\n\n"\
+	"NAME Send File\n"		"CMD /dcc send %s\n\n"\
+	"NAME Offer Chat\n"		"CMD /dcc chat %s\n\n"\
+	"NAME Abort Chat\n"		"CMD /dcc close chat %s\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD Oper\n\n"\
+	"NAME Kill\n"				"CMD /quote KILL %s :die!\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD Mode\n\n"\
+	"NAME Give Voice\n"		"CMD /voice %s\n\n"\
+	"NAME Take Voice\n"		"CMD /devoice %s\n"\
+	"NAME SEP\n"				"CMD \n\n"\
+	"NAME Give Ops\n"			"CMD /op %s\n\n"\
+	"NAME Take Ops\n"			"CMD /deop %s\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD Ignore\n\n"\
+	"NAME Ignore User\n"		"CMD /ignore %s!*@* ALL\n\n"\
+	"NAME UnIgnore User\n"	"CMD /unignore %s!*@*\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD Kick/Ban\n\n"\
+	"NAME Kick\n"				"CMD /kick %s\n\n"\
+	"NAME Ban\n"				"CMD /ban %s\n\n"\
+	"NAME SEP\n"				"CMD \n\n"\
+	"NAME Ban *!*@*.host\n"	"CMD /ban %s 0\n\n"\
+	"NAME Ban *!*@domain\n"	"CMD /ban %s 1\n\n"\
+	"NAME Ban *!*user@*.host\n""CMD /ban %s 2\n\n"\
+	"NAME Ban *!*user@domain\n""CMD /ban %s 3\n\n"\
+	"NAME SEP\n"				"CMD \n\n"\
+	"NAME KickBan *!*@*.host\n""CMD /kickban %s 0\n\n"\
+	"NAME KickBan *!*@domain\n""CMD /kickban %s 1\n\n"\
+	"NAME KickBan *!*user@*.host\n""CMD /kickban %s 2\n\n"\
+	"NAME KickBan *!*user@domain\n""CMD /kickban %s 3\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD Info\n\n"\
+	"NAME Who\n"				"CMD /quote WHO %s\n\n"\
+	"NAME Whois\n"				"CMD /quote WHOIS %s\n\n"\
+	"NAME DNS Lookup\n"		"CMD /dns %s\n\n"\
+	"NAME Trace\n"				"CMD /quote TRACE %s\n\n"\
+	"NAME UserHost\n"			"CMD /quote USERHOST %s\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME SUB\n"				"CMD External\n\n"\
+	"NAME Traceroute\n"		"CMD !rxvt -e /bin/sh -c \"traceroute %h ; sleep 30\"\n\n"\
+	"NAME Ping\n"				"CMD !rxvt -e /bin/sh -c \"ping -c 4 %h ; sleep 30\"\n\n"\
+	"NAME Telnet\n"			"CMD !rxvt -e telnet %h\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
+	"NAME Open Query\n"		"CMD /query %s\n\n"
 
-#define defaultconf_buttons   "NAME Op\nCMD /op %a\n\n"\
-                              "NAME DeOp\nCMD /deop %a\n\n"\
-                              "NAME Ban\nCMD /ban %s\n\n"\
-                              "NAME Kick\nCMD /kick %s\n\n"\
-                              "NAME Send\nCMD /dcc send %s\n\n"\
-                              "NAME Dialog\nCMD /query %s\n\n"\
-                              "NAME Lookup\nCMD /dns %s\n\n"\
-                              "NAME Whois\nCMD /whois %s\n"
+#define defaultconf_buttons \
+	"NAME Op\n"					"CMD /op %a\n\n"\
+	"NAME DeOp\n"				"CMD /deop %a\n\n"\
+	"NAME Ban\n"				"CMD /ban %s\n\n"\
+	"NAME Kick\n"				"CMD /kick %s\n\n"\
+	"NAME Send\n"				"CMD /dcc send %s\n\n"\
+	"NAME Dialog\n"			"CMD /query %s\n\n"\
+	"NAME Lookup\n"			"CMD /dns %s\n\n"\
+	"NAME Whois\n"				"CMD /whois %s\n"
 
-#define defaultconf_replace "NAME teh\nCMD the\n\n"\
-                            "NAME r\nCMD are\n\n"\
-                            "NAME u\nCMD you\n\n"
+#define defaultconf_replace \
+	"NAME teh\n"				"CMD the\n\n"\
+	"NAME r\n"					"CMD are\n\n"\
+	"NAME u\n"					"CMD you\n\n"
 
 #define defaultconf_commands \
    "NAME ACTION\n"		"CMD /me &2\n\n"\
 	"NAME AME\n"			"CMD /allchan /me &2\n\n"\
 	"NAME AMSG\n"			"CMD /allchan &2\n\n"\
+	"NAME BACK\n"			"CMD /away\n\n"\
    "NAME BANLIST\n"		"CMD /quote MODE %c +b\n\n"\
    "NAME CHAT\n"			"CMD /dcc chat %2\n\n"\
    "NAME DIALOG\n"		"CMD /query %2\n\n"\
@@ -1035,6 +859,7 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME SPING\n"			"CMD /ping\n\n"\
    "NAME SV\n"				"CMD /echo xchat %v %m\n\n"\
    "NAME UMODE\n"			"CMD /mode %n &2\n\n"\
+	"NAME UNBAN\n"			"CMD /quote MODE %c -b &2\n\n"\
    "NAME UPTIME\n"		"CMD /quote STATS u\n\n"\
    "NAME VER\n"			"CMD /ctcp %2 VERSION\n\n"\
    "NAME VERSION\n"		"CMD /ctcp %2 VERSION\n\n"\
@@ -1071,16 +896,31 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
    "NAME ENDSUB\n"				"CMD \n\n"
 
 #define defaultconf_urlhandlers \
-   "NAME -> Netscape (Existing)\n"		"CMD !netscape -remote 'openURL(%s)'\n\n"\
-   "NAME -> Netscape (New Window)\n"	"CMD !netscape -remote 'openURL(%s,new-window)'\n\n"\
-   "NAME -> Netscape (Run New)\n"		"CMD !netscape %s\n\n"\
-   "NAME -> Lynx\n"							"CMD !xterm -e lynx %s\n\n"\
-   "NAME -> NcFTP\n" 						"CMD !xterm -e ncftp %s\n\n"\
-   "NAME -> gFTP\n"							"CMD !gftp %s\n\n"\
-   "NAME -> Mozilla\n"						"CMD !mozilla %s\n\n"\
-   "NAME -> KFM\n"							"CMD !kfmclient openURL %s\n\n"\
-   "NAME -> Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
-   "NAME -> Connect as IRC Server\n"	"CMD /newserver %s\n\n"
+	"NAME SUB\n"								"CMD Netscape...\n\n"\
+   	"NAME Open in existing\n"			"CMD !netscape -remote 'openURL(%s)'\n\n"\
+   	"NAME Open in new window\n"		"CMD !netscape -remote 'openURL(%s,new-window)'\n\n"\
+   	"NAME Run new Netscape\n"			"CMD !netscape %s\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
+	"NAME SUB\n"								"CMD Mozilla...\n\n"\
+   	"NAME Open in existing\n"			"CMD !mozilla -remote 'openURL(%s)'\n\n"\
+   	"NAME Open in new window\n"		"CMD !mozilla -remote 'openURL(%s,new-window)'\n\n"\
+   	"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
+	"NAME SUB\n"								"CMD Galeon...\n\n"\
+		"NAME Open in existing\n"			"CMD !galeon -x '%s'\n\n"\
+		"NAME Run new Galeon\n"				"CMD !galeon '%s'\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
+	"NAME SUB\n"								"CMD Send URL to...\n\n"\
+   	"NAME Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
+   	"NAME Lynx\n"							"CMD !rxvt -e lynx %s\n\n"\
+   	"NAME Links\n"							"CMD !rxvt -e links %s\n\n"\
+   	"NAME NcFTP\n" 						"CMD !rxvt -e ncftp %s\n\n"\
+   	"NAME gFTP\n"							"CMD !gftp %s\n\n"\
+		"NAME KFM\n"							"CMD !kfmclient openURL %s\n\n"\
+		"NAME Telnet\n"						"CMD !rxvt -e telnet %s\n\n"\
+		"NAME Ping\n"							"CMD !rxvt -e ping -c 4 %s\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
+	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n"
 
 static void
 xchat_init (void)
@@ -1089,6 +929,7 @@ xchat_init (void)
 	struct server *serv;
 	struct sigaction act;
 
+	/* ignore SIGPIPE's */
 	act.sa_handler = SIG_IGN;
 	act.sa_flags = 0;
 	sigemptyset (&act.sa_mask);
@@ -1157,7 +998,7 @@ xchat_cleanup (void)
 	fe_exit ();
 }
 
-int
+static int
 child_handler (int pid)
 {
 	if (waitpid (pid, 0, WNOHANG) == pid)
@@ -1185,7 +1026,10 @@ main (int argc, char *argv[])
 #endif
 
 #ifdef USE_OPENSSL
-	_SSL_context_init (ssl_cb_info);
+	if (!(ctx = _SSL_context_init (ssl_cb_info, FALSE))) {
+		fprintf(stderr, "_SSL_context_init failed\n");
+		exit(1);
+	}
 #endif
 
 	if (!fe_args (argc, argv))
@@ -1200,7 +1044,7 @@ main (int argc, char *argv[])
 	fe_main ();
 
 #ifdef USE_OPENSSL
-	_SSL_context_free ();
+	_SSL_context_free (ctx);
 #endif
 
 #ifdef USE_PYTHON
