@@ -1,10 +1,8 @@
 #include "../../config.h"
 
-#ifndef PACKAGE				/* fixes prob compiling on RH5.2, *shrug* */
+#ifndef VERSION	/* fixes prob compiling on RH5.2, *shrug* */
 #define PACKAGE "xchat"
-#endif
-#ifndef VERSION
-#define VERSION "1.5.3"
+#define VERSION "1.6.3"
 #endif
 
 #ifdef USE_MYGLIB
@@ -45,6 +43,10 @@
 
 #ifdef __EMX__						  /* for o/s 2 */
 #define OFLAGS O_BINARY
+#define strcasecmp stricmp
+#define strncasecmp strnicmp
+#define PATH_MAX MAXPATHLEN
+#define FILEPATH_LEN_MAX MAXPATHLEN
 #else
 #define OFLAGS 0
 #endif
@@ -76,6 +78,7 @@ struct xchatprefs
 	char nick_suffix[4];			/* Only ever holds a one-character string. */
 	char awayreason[256];
 	char quitreason[256];
+	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
 	char dialog_font_normal[FONTNAMELEN + 1];
 	char doubleclickuser[256];
@@ -84,9 +87,10 @@ struct xchatprefs
 	char background[PATHLEN + 1];
 	char background_dialog[PATHLEN + 1];
 	char dccdir[PATHLEN + 1];
-	char bluestring[64];
+	char bluestring[300];
 	char dnsprogram[72];
 	char hostname[127];
+	char cmdchar[4];
 	char trans_file[256];
 	char logmask[256];
 
@@ -123,10 +127,11 @@ struct xchatprefs
 	int bantype;
 	int userlist_sort;
 	int nu_color;
-	int bt_color;
 	unsigned long local_ip;
 	unsigned long dcc_ip;
 
+	unsigned int perc_ascii;
+	unsigned int hilightnick;
 	unsigned int use_trans;
 	unsigned int nopanel;
 	unsigned int autosave;
@@ -138,22 +143,21 @@ struct xchatprefs
 	unsigned int servernotice;
 	unsigned int wallops;
 	unsigned int skipmotd;
-	unsigned int away;
 	unsigned int autorejoin;
 	unsigned int colorednicks;
-	unsigned int nochanmodebuttons;
-	unsigned int nouserlistbuttons;
+	unsigned int chanmodebuttons;
+	unsigned int userlistbuttons;
    unsigned int showhostname_in_userlist;
 	unsigned int nickcompletion;
 	unsigned int old_nickcompletion;
 	unsigned int tabchannels;
-	unsigned int nopaned;
+	unsigned int paned_userlist;
 	unsigned int autodccchat;
 	unsigned int autodccsend;
 	unsigned int autoresume;
-	unsigned int noautoopendccsendwindow;
-	unsigned int noautoopendccrecvwindow;
-	unsigned int noautoopendccchatwindow;
+	unsigned int autoopendccsendwindow;
+	unsigned int autoopendccrecvwindow;
+	unsigned int autoopendccchatwindow;
 	unsigned int transparent;
 	unsigned int tint;
 	unsigned int dialog_transparent;
@@ -166,6 +170,7 @@ struct xchatprefs
 	unsigned int beepmsg;
 	unsigned int privmsgtab;
 	unsigned int logging;
+	unsigned int timestamp_logs;
 	unsigned int newtabstofront;
 	unsigned int dccwithnick;
 	unsigned int hilitenotify;
@@ -174,10 +179,9 @@ struct xchatprefs
 	unsigned int panelize_hide;
 	unsigned int panel_vbox;
 	unsigned int raw_modes;
-	unsigned int no_server_logs;
 	unsigned int show_away_once;
 	unsigned int show_away_message;
-	unsigned int nouserhost;
+	unsigned int userhost;
 	unsigned int use_server_tab;
 	unsigned int style_namelistgad;
 	unsigned int style_inputbox;
@@ -193,7 +197,6 @@ struct xchatprefs
 	unsigned int wordwrap;
 	unsigned int dialog_wordwrap;
 	unsigned int mail_check;
-	unsigned int double_buffer;
 	unsigned int throttle;
 	unsigned int fudgeservernotice;
 	unsigned int topicbar;
@@ -260,7 +263,7 @@ struct session
 	int ignore_names:1;
 	int end_of_names:1;
 	int doing_who:1;		/* /who sent on this channel */
- 	int highlight_tab:1;	/* Highlight the channel tab (red-style) */
+	int highlight_tab:1;	/* Highlight the channel tab (red-style) */
 };
 
 typedef struct session session;
@@ -290,8 +293,8 @@ struct server
 	int nickcount;
 
 	GSList *outbound_queue;
-	time_t last_send;
-	int bytes_sent;				  /* this second only */
+	int next_send;						/* cptr->since in ircu */
+	int sendq_len;						/* queue size */
 
 	struct session *front_session;
 
@@ -306,6 +309,7 @@ struct server
 	/*time_t connect_time;*/				/* when did it connect? */
 	time_t lag_sent;
 	time_t ping_recv;					/* when we last got a ping reply */
+	time_t away_time;					/* when we were marked away */
 
 	int motd_skipped:1;
 	int connected:1;
@@ -318,6 +322,7 @@ struct server
 	int sent_quit:1;				  /* sent a QUIT already? */
 	int is_newtype:1;		/* undernet and dalnet need /list >0,<10000 */
 	int six_modes:1;		/* undernet allows 6 modes per line */
+	int is_away:1;
 };
 
 typedef struct server server;
