@@ -36,25 +36,8 @@
 #include "gtkutil.h"
 
 extern GdkColor colors[];
-
-static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT", };
-
-/* The name and color of each dcc status. */
-
-struct dccstat_info
-{
-	char *name;						  /* Display name */
-	int color;						  /* Display color (index into colors[] ) */
-};
-
-static struct dccstat_info dccstat[] = {
-	{"Queued", 1 /*black */ },
-	{"Active", 12 /*cyan */ },
-	{"Failed", 4 /*red */ },
-	{"Done", 3 /*green */ },
-	{"Connect", 1 /*black */ },
-	{"Aborted", 4 /*red */ },
-};
+extern struct dccstat_info dccstat[];	/* defined in common/dcc.c */
+extern char *dcctypes[];
 
 struct dccwindow
 {
@@ -78,12 +61,13 @@ extern GSList *dcc_list;
 
 extern int child_handler (int pid);
 extern void dcc_resume (struct DCC *dcc);
+extern void xchat_exec (char *cmd);
 
 
 static void
 dcc_send_filereq_done (struct session *sess, char *nick, char *file)
 {
-	char tbuf[1024];
+	char tbuf[400];
 
 	if (file)
 	{
@@ -380,7 +364,6 @@ open_clicked (void)
 	char *mime_type;
 	char *mime_prog;
 	char *tmp;
-	int pid;
 
 	row = gtkutil_clist_selection (dccrwin.list);
 	if (row != -1)
@@ -401,13 +384,7 @@ open_clicked (void)
 						tmp[1] = 's';
 						tmp = malloc (strlen (dcc->destfile) + strlen (mime_prog));
 						sprintf (tmp, mime_prog, dcc->destfile);
-
-						pid = fork ();
-						if (pid == 0)
-							execl ("/bin/sh", "sh", "-c", tmp, 0);
-						else
-							fe_timeout_add (1000, child_handler, (void *) pid);
-
+						xchat_exec (tmp);
 						free (tmp);
 					}
 					free (mime_prog);
@@ -680,18 +657,7 @@ accept_chat_clicked (GtkWidget * wid, gpointer none)
 	{
 		dcc = gtk_clist_get_row_data (GTK_CLIST (dcccwin.list), row);
 		gtk_clist_unselect_row (GTK_CLIST (dcccwin.list), row, 0);
-		switch (dcc->dccstat)
-		{
-		case STAT_QUEUED:
-			if (dcc->type == TYPE_CHATRECV)
-				dcc_connect (0, dcc);
-			break;
-		case STAT_DONE:
-		case STAT_FAILED:
-		case STAT_ABORTED:
-			dcc_abort (dcc);
-			break;
-		}
+		dcc_get (dcc);
 	}
 }
 
