@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
@@ -30,7 +31,6 @@
 
 extern void maingui_updatebuttons (struct session *sess);
 extern int buf_get_line (char *, char **, int *, int);
-extern void list_free (GSList **);
 extern void list_loadconf (char *, GSList **, char *);
 extern void fe_buttons_update (struct session *sess);
 
