@@ -18,19 +18,29 @@
 
 #include <stdio.h>
 #include <string.h>
+#include "../../config.h"
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 #include "../common/xchat.h"
 #include "../common/fe.h"
+#include "../common/util.h"
 #include "fe-gtk.h"
 #include "gtkutil.h"
 #include "pixmaps.h"
 #include "xtext.h"
+
+#ifdef USE_XLIB
 #include <gdk/gdkx.h>
 #include <gtk/gtkinvisible.h>
+#endif
+
 #ifdef USE_GDK_PIXBUF
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
+
 #ifdef USE_PANEL
 #include <applet-widget.h>
 extern int nopanel;
@@ -58,11 +68,9 @@ extern GdkColor colors[];
 extern int is_session (session * sess);
 extern char *get_cpu_str (int color);
 extern void userlist_hide (GtkWidget * igad, struct session *sess);
-extern void PrintTextRaw (GtkWidget * textwidget, unsigned char *text);
+extern void PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent);
 extern void notify_gui_update (void);
 extern void update_all_of (char *name);
-extern void my_gtk_entry_set_text (GtkWidget * wid, char *text,
-											  struct session *sess);
 extern void palette_load (void);
 extern void palette_save (void);
 extern void key_init (void);
@@ -74,6 +82,9 @@ extern void serverlist_autoconnect (struct session *sess, int);
 extern void tree_blue_style (struct session *);
 extern void tree_red_style (struct session *);
 extern void menu_newshell_set_palette (session *sess);
+
+#ifdef USE_XLIB
+
 extern Window get_desktop_window (Window the_window);
 
 
@@ -112,6 +123,8 @@ static gboolean handle_property_notify (GtkWidget *widget,
 	return FALSE;
 }
 
+#endif
+
 int
 fe_args (int argc, char *argv[])
 {
@@ -198,6 +211,7 @@ fe_args (int argc, char *argv[])
 #endif
 #endif
 
+#ifdef USE_XLIB
 	proxy_invisible = gtk_invisible_new ();
 	gtk_widget_show (proxy_invisible);
 
@@ -208,6 +222,7 @@ fe_args (int argc, char *argv[])
 	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);
 
 	gtk_signal_connect(GTK_OBJECT(proxy_invisible), "property-notify-event", GTK_SIGNAL_FUNC(handle_property_notify), NULL);
+#endif
 
 	return 1;
 }
@@ -277,7 +292,6 @@ static void
 init_sess (void)
 {
 	char buf[512];
-	time_t tim = time (0);
 	struct session *sess = sess_list->data;
 
 	if (done_intro)
@@ -293,28 +307,28 @@ init_sess (void)
 		if (!prefs.skipserverlist)
 			open_server_list (0, sess, 0);
 	}
-	snprintf (buf, sizeof buf,	
-				"\n\n"
-				" \037\0031                                                  \n"
-				" \037\0038,2                xchat "VERSION" dEVEL!              \n"
-				"\n"
-				"\0033  GTK/X Version\0032..\0035:\017 %d.%d.%d / %d (%d.%d)\n"
-				"\0033  System Type\0032....\0035:\017 %s\n"
-				"\0033  Launch Time\0032....\0035:\017 %.19s\n"
-				"\0033  Binary Date\0032....\0035:\017 "__DATE__"\n"
-				"\0033  Current Pid\0032....\0035:\017 %d\n"
-				"\0033  Unix Username\0032..\0035:\017 %s\n"
-				"\0033  Your UserId\0032....\0035:\017 %d\n",
-					gtk_major_version, gtk_minor_version, gtk_micro_version,
-					VendorRelease (GDK_DISPLAY ()),
-					ProtocolVersion (GDK_DISPLAY ()),
-					ProtocolRevision (GDK_DISPLAY ()),
-					get_cpu_str (1), ctime (&tim), getpid (), g_get_user_name (),
-					getuid ()
-				 );
+
+	snprintf (buf, sizeof (buf),	
+   	"\n"
+		"\00314-\00315-\00301-\00310-\00311 xchat "VERSION""
+		" \00310-\00301-\00315-\00314-\017\n"
+		"\n"
+		"\00314[ \0036 system  \00314] \017 %s\n"
+		"\00314[ \0036  gui    \00314] \017 X %d GTK %d.%d.%d\n"
+		"\00314[ \0036nickname \00314] \017 %s\n"
+		"\00314[ \0036compiled \00314] \017 "__DATE__"\n",
+				get_cpu_str (1),
+#ifdef USE_XLIB
+				VendorRelease (GDK_DISPLAY ()),
+#else
+				666,
+#endif
+				gtk_major_version, gtk_minor_version, gtk_micro_version,
+				prefs.nick1
+				);
 	PrintText (sess, buf);
 
-	strcpy (buf, "\0033  Features\0032.......\0035: \017");
+	strcpy (buf, "\00314[ \0036features \00314] \017 ");
 #ifdef USE_GDK_PIXBUF
 	strcat (buf, "Gdk-Pixbuf ");
 #endif
@@ -354,10 +368,9 @@ init_sess (void)
 	strcat (buf, "\n\n");
 #endif
 	PrintText (sess, buf);
-	if (getuid () == 0)
-	{
-		PrintText (sess, "\nYour move root!\n");
-	}
+
+	PrintText (sess, "\n\0033*\00315 Type \017/help\00315 for a list of commands\017\n\n");
+
 	while (gtk_events_pending ())
 		gtk_main_iteration ();
 }
@@ -476,8 +489,8 @@ fe_set_hilight (struct session *sess)
 void
 fe_update_channel_key (struct session *sess)
 {
-	my_gtk_entry_set_text (sess->gui->key_entry, sess->channelkey, sess);
-
+	if (sess->gui->flag_wid[0])	/* channel mode buttons enabled? */
+		gtk_entry_set_text ((GtkEntry*)sess->gui->key_entry, sess->channelkey);
 	fe_set_title (sess);
 }
 
@@ -485,8 +498,12 @@ void
 fe_update_channel_limit (struct session *sess)
 {
 	char tmp[16];
-	sprintf (tmp, "%d", sess->limit);
-	my_gtk_entry_set_text (sess->gui->limit_entry, tmp, sess);
+
+	if (sess->gui->flag_wid[0])	/* channel mode buttons enabled? */
+	{
+		sprintf (tmp, "%d", sess->limit);
+		gtk_entry_set_text ((GtkEntry*)sess->gui->limit_entry, tmp);
+	}
 }
 
 int
@@ -516,7 +533,7 @@ void
 fe_text_clear (struct session *sess)
 {
 	GtkXText *xtext = (GtkXText *)sess->gui->textgad;
-	gtk_xtext_remove_lines (xtext, -1, TRUE);
+	gtk_xtext_clear (xtext);
 	if (xtext->auto_indent)
 		xtext->indent = 1;
 }
@@ -598,7 +615,14 @@ fe_progressbar_end (struct session *sess)
 void
 fe_print_text (struct session *sess, char *text)
 {
-	PrintTextRaw (sess->gui->textgad, text);
+	int indent;
+
+	if (sess->is_dialog)
+		indent = prefs.dialog_indent_nicks;
+	else
+		indent = prefs.indent_nicks;
+
+	PrintTextRaw (sess->gui->textgad, text, indent);
 
 	if (prefs.limitedtabhighlight && !sess->highlight_tab)
 		return;
@@ -645,10 +669,25 @@ fe_userlist_hide (struct session *sess)
 	userlist_hide (0, sess);
 }
 
-char *
-fe_buffer_get (session * sess)
+void
+fe_lastlog (session *sess, session *lastlog_sess, char *sstr)
 {
-	return gtk_xtext_get_chars (GTK_XTEXT (sess->gui->textgad));
+	textentry *ent;
+
+	ent = GTK_XTEXT(sess->gui->textgad)->text_first;
+	if (!ent)
+	{
+		PrintText (lastlog_sess, "Search buffer is empty.\n");
+	} else
+	{
+		do
+		{
+			if (nocasestrstr (ent->str, sstr))
+				PrintText (lastlog_sess, ent->str);
+			ent = ent->next;
+		}
+		while (ent);
+	}
 }
 
 void
