@@ -18,10 +18,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
+#include "../../config.h"
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include <fcntl.h>
 #include <ctype.h>
 #include "../common/xchat.h"
@@ -35,7 +39,6 @@
 
 extern GSList *fkey_list;
 extern GtkWidget *main_window, *main_book;
-extern int waitline (int sok, char *buf, int bufsize);
 extern int handle_multiline (struct session *sess, char *cmd, int history,
 									  int nocommand);
 extern void handle_inputgad (GtkWidget * igad, struct session *sess);
@@ -53,8 +56,6 @@ extern struct xchatprefs prefs;
 
 extern void PrintText (struct session *sess, unsigned char *text);
 extern int buf_get_line (char *, char **, int *, int len);
-static void history_up (GtkWidget * widget, struct history *his);
-static void history_down (GtkWidget * widget, struct history *his);
 static int tab_nick_comp (GtkWidget * t, int shift);
 static void nick_comp_chng (GtkWidget * t, int updown);
 static void replace_handle (GtkWidget * wid);
@@ -78,7 +79,7 @@ void maingui_moveright (GtkWidget * wid);
  */
 
 /* Remember that the *number* of actions is this *plus* 1 --AGL */
-#define KEY_MAX_ACTIONS 13
+#define KEY_MAX_ACTIONS 14
 /* These are cp'ed from history.c --AGL */
 #define STATE_SHIFT     GDK_SHIFT_MASK
 #define	STATE_ALT	GDK_MOD1_MASK
@@ -105,7 +106,7 @@ struct key_action
 int key_load_kbs (char *);
 static void key_load_defaults ();
 void key_save_kbs (char *);
-void PrintTextRaw (GtkWidget * textwidget, unsigned char *text);
+void PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent);
 
 static int key_action_delete_backward_word(GtkWidget * wid, GdkEventKey * evt, char *d1,
 														 char *d2, struct session *sess);
@@ -138,6 +139,9 @@ static int key_action_move_tab_left (GtkWidget * wid, GdkEventKey * evt,
 static int key_action_move_tab_right (GtkWidget * wid, GdkEventKey * evt,
 												  char *d1, char *d2,
 												  struct session *sess);
+static int key_action_put_history (GtkWidget * wid, GdkEventKey * evt,
+												  char *d1, char *d2,
+												  struct session *sess);
 
 static GtkWidget *key_dialog;
 static struct key_binding *keys_root = NULL;
@@ -161,7 +165,7 @@ static struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 	{key_action_history_up, "Last Command",
 	 "The \002Last Command\002 command sets the entry to contain the last command entered - the same as pressing up in a shell"},
 	{key_action_history_down, "Next Command",
-	 "The \002Last Command\002 command sets the entry to contain the next command entered - the same as pressing down in a shell"},
+	 "The \002Next Command\002 command sets the entry to contain the next command entered - the same as pressing down in a shell"},
 	{key_action_tab_comp, "Complete nick/command",
 	 "This command changes the text in the entry to finish an incomplete nickname or command. If Data 1 is set then double-tabbing in a string will select the last nick, not the next"},
 	{key_action_comp_chng, "Change Selected Nick",
@@ -172,6 +176,8 @@ static struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 	 "This command move the front tab left by one"},
 	{key_action_move_tab_right, "Move front tab right",
 	 "This command move the front tab right by one"},
+	{key_action_put_history, "Push input line into history",
+	 "Push input line into history but doesn't send to server"},
 };
 
 void
@@ -237,10 +243,6 @@ key_handle_key_press (GtkWidget * wid, GdkEventKey * evt,
 	int mod, n;
 
 	mod = evt->state & (STATE_CTRL | STATE_ALT | STATE_SHIFT);
-	/* First check if it's a return or enter */
-	if ((evt->keyval == GDK_Return)
-		 || (evt->keyval == GDK_KP_Enter /*&& mod == 0*/))
-		handle_inputgad (wid, sess);
 
 	kb = keys_root;
 	while (kb)
@@ -273,6 +275,12 @@ key_handle_key_press (GtkWidget * wid, GdkEventKey * evt,
 		last = kb;
 		kb = kb->next;
 	}
+
+	/* check if it's a return or enter */
+	if ((evt->keyval == GDK_Return)
+		 || (evt->keyval == GDK_KP_Enter /*&& mod == 0*/))
+		handle_inputgad (wid, sess);
+
 	if (evt->keyval == GDK_KP_Enter)
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (wid), "key_press_event");
 	return 1;
@@ -439,8 +447,8 @@ key_dialog_delete (GtkWidget * button, GtkCList * list)
 			cur = cur->next;
 		}
 		printf ("*** key_dialog_delete: couldn't find kb in list!\n");
-		if (getenv ("XCHAT_DEBUG"))
-			abort ();
+		/*if (getenv ("XCHAT_DEBUG"))
+			abort ();*/
 	}
 }
 
@@ -458,8 +466,8 @@ key_dialog_sel_act (GtkWidget * un, int num)
 								  key_actions[num].name);
 		if (key_actions[num].help)
 		{
-			gtk_xtext_remove_lines (GTK_XTEXT (key_dialog_text), -1, TRUE);
-			PrintTextRaw (key_dialog_text, key_actions[num].help);
+			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
+			PrintTextRaw (key_dialog_text, key_actions[num].help, 0);
 		}
 	}
 }
@@ -481,8 +489,8 @@ key_dialog_sel_row (GtkWidget * clist, gint row, gint column,
 											  kb->action);
 		if (key_actions[kb->action].help)
 		{
-			gtk_xtext_remove_lines (GTK_XTEXT (key_dialog_text), -1, TRUE);
-			PrintTextRaw (key_dialog_text, key_actions[kb->action].help);
+			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
+			PrintTextRaw (key_dialog_text, key_actions[kb->action].help, 0);
 		}
 	}
 	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (key_dialog_tog_c),
@@ -1072,8 +1080,8 @@ key_load_kbs (char *filename)
 	return 0;
 
  corrupt_file:
-	if (getenv ("XCHAT_DEBUG"))
-		abort ();
+	/*if (getenv ("XCHAT_DEBUG"))
+		abort ();*/
 	free (ibuf);
 	buf = malloc (1024);
 	snprintf (buf, 1024,
@@ -1315,7 +1323,12 @@ static int
 key_action_history_up (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 							  struct session *sess)
 {
-	history_up (wid, &sess->history);
+	char *new_line;
+
+	new_line = history_up (&sess->history);
+	if (new_line)
+		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
+
 	return 2;
 }
 
@@ -1323,7 +1336,12 @@ static int
 key_action_history_down (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								 char *d2, struct session *sess)
 {
-	history_down (wid, &sess->history);
+	char *new_line;
+
+	new_line = history_down (&sess->history);
+	if (new_line)
+		gtk_entry_set_text (GTK_ENTRY (wid), new_line);
+
 	return 2;
 }
 
@@ -1382,12 +1400,21 @@ key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 	return 2;						  /* -''- */
 }
 
+static int
+key_action_put_history (GtkWidget * wid, GdkEventKey * ent, char *d1,
+									char *d2, struct session *sess)
+{
+	history_add (&sess->history, gtk_entry_get_text (GTK_ENTRY (wid)));
+	gtk_entry_set_text (GTK_ENTRY (wid), "");
+	return 2;						  /* -''- */
+}
+
 
 /* -------- */
 
 
-#define STATE_SHIFT     GDK_SHIFT_MASK
-#define	STATE_ALT	GDK_MOD1_MASK
+#define STATE_SHIFT	GDK_SHIFT_MASK
+#define STATE_ALT		GDK_MOD1_MASK
 #define STATE_CTRL	GDK_CONTROL_MASK
 
 static void
@@ -1414,6 +1441,8 @@ replace_handle (GtkWidget *t)
 	xlen = c;
 	if (len - c >= (sizeof (word) - 12))
 		return;
+	if (len - c < 1)
+		return;
 	memcpy (word, &text[c], len - c);
 	word[len - c] = 0;
 	len = strlen (word);
@@ -1459,22 +1488,19 @@ static struct session *
 find_session_from_inputgad (GtkWidget * w)
 {
 	GSList *list = sess_list;
-	struct session *sess = 0;
+	struct session *sess;
 
 	/* First find the session from the widget */
 	while (list)
 	{
 		sess = (struct session *) list->data;
 		if (sess->gui->inputgad == w)
-			break;
+			return sess;
 		list = list->next;
 	}
-	if (!list)
-	{
-		/*Erm, we didn't find ANY valid session, HELP! */
-		return NULL;
-	}
-	return sess;
+
+	/*Erm, we didn't find ANY valid session, HELP! */
+	return NULL;
 }
 
 static int
@@ -1801,6 +1827,11 @@ tab_nick_comp (GtkWidget *t, int shift)
 		}
 
 		j = GTK_EDITABLE (t)->current_pos;
+
+		/* !! FIXME !! */
+		if (len - j < 0)
+			return 0;
+
 		b4 = (char *) malloc (len + 1);
 		c5 = (char *) malloc (len + 1);
 		memmove (c5, &text[j], len - j);
@@ -1947,71 +1978,3 @@ tab_nick_comp (GtkWidget *t, int shift)
 
 	return 0;
 }
-
-static void
-history_down (GtkWidget * widget, struct history *his)
-{
-	if (his->pos == his->realpos)	/* allow down only after up */
-		return;
-	if (his->realpos == 0)
-	{
-		if (his->pos == HISTORY_SIZE - 1)
-		{
-			his->pos++;
-			gtk_entry_set_text (GTK_ENTRY (widget), "");
-			return;
-		}
-	} else
-	{
-		if (his->pos == his->realpos - 1)
-		{
-			his->pos++;
-			gtk_entry_set_text (GTK_ENTRY (widget), "");
-			return;
-		}
-	}
-
-	his->pos++;
-	if (his->pos == HISTORY_SIZE)
-		his->pos = 0;
-	if (his->lines[his->pos])
-	{
-		gtk_entry_set_text (GTK_ENTRY (widget), his->lines[his->pos]);
-	} else
-	{
-		if (his->pos != 0)
-			his->pos--;
-		else
-			his->pos = (HISTORY_SIZE - 1);
-	}
-}
-
-static void
-history_up (GtkWidget * widget, struct history *his)
-{
-	if (his->realpos == HISTORY_SIZE - 1)
-	{
-		if (his->pos == 0)
-			return;
-	} else
-	{
-		if (his->pos == his->realpos + 1)
-			return;
-	}
-
-	if (his->pos == 0)
-		his->pos = (HISTORY_SIZE - 1);
-	else
-		his->pos--;
-
-	if (his->lines[his->pos])
-	{
-		gtk_entry_set_text (GTK_ENTRY (widget), his->lines[his->pos]);
-	} else
-	{
-		if (his->pos == (HISTORY_SIZE - 1))
-			his->pos = 0;
-		else
-			his->pos++;
-	}
-}
