@@ -19,14 +19,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include "../common/xchat.h"
 #include "fe-gtk.h"
+#ifdef USE_XLIB
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
+#endif
 
 /* gtkutil.c, just some gtk wrappers */
 
@@ -46,7 +49,7 @@ struct file_req
 	int write;
 };
 
-char last_dir[256] = "";
+static char last_dir[256] = "";
 
 void
 gtkutil_file_req_cancel (GtkWidget * wid, struct file_req *freq)
@@ -155,8 +158,9 @@ gtkutil_window_new (char *title, char *wmclass, int x, int y, void *callback,
 	gtk_widget_set_usize (win, x, y);
 	gtk_window_set_title (GTK_WINDOW (win), title);
 	gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_MOUSE);
-	gtk_signal_connect (GTK_OBJECT (win), "destroy",
-							  GTK_SIGNAL_FUNC (callback), userdata);
+	if (callback)
+		gtk_signal_connect (GTK_OBJECT (win), "destroy",
+							  		GTK_SIGNAL_FUNC (callback), userdata);
 	return win;
 }
 
@@ -334,12 +338,14 @@ gtkutil_null_this_var (GtkWidget * unused, GtkWidget ** dialog)
 void
 gtkutil_set_icon (GdkWindow * window, GdkPixmap * pixmap, GdkBitmap * mask)
 {
+#ifdef USE_XLIB
 	Atom icon_atom;
 	glong data[2];
 
-	data[0] = ((GdkPixmapPrivate *) pixmap)->xwindow;
-	data[1] = ((GdkPixmapPrivate *) mask)->xwindow;
+	data[0] = GDK_WINDOW_XWINDOW (pixmap);
+	data[1] = GDK_WINDOW_XWINDOW (mask);
 	icon_atom = gdk_atom_intern ("KWM_WIN_ICON", FALSE);
 	gdk_property_change (window, icon_atom, icon_atom,
 								32, GDK_PROP_MODE_REPLACE, (guchar *) data, 2);
+#endif
 }
