@@ -18,12 +18,12 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include "../../config.h"
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
 #include <stdio.h>
 #include <fcntl.h>
-#include <arpa/inet.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -47,6 +47,7 @@ GtkWidget *main_window = 0;
 GtkWidget *main_book;
 GtkWidget *main_menu;
 GtkWidget *main_menu_bar;
+GtkWidget *main_menu_away_item = 0;
 
 GtkStyle *normaltab_style = 0;
 GtkStyle *redtab_style;
@@ -66,12 +67,12 @@ extern GdkFont *font_normal;
 extern GdkColor colors[];
 extern gint xchat_is_quitting;
 
-extern void menu_showhide (session *sess, int force_hide);
+extern void menu_showhide (void);
 extern void menu_newshell_set_palette (session *sess);
 extern void nick_command_parse (session * sess, char *cmd, char *nick,
 										  char *allnick);
 extern void userlist_button_cb (GtkWidget * button, char *cmd);
-extern void goto_url (void *unused, char *url);
+extern void goto_url (char *url);
 extern void ascii_open (void);
 extern int key_action_insert (GtkWidget * wid, GdkEventKey * evt, char *d1,
 										char *d2, struct session *sess);
@@ -99,8 +100,6 @@ extern void menu_nickmenu (session *sess, GdkEventButton *event, char *nick);
 extern void clear_user_list (struct session *sess);
 extern GtkWidget *createmenus (void *app, struct session *sess, int bar);
 extern void handle_inputgad (GtkWidget * igad, struct session *sess);
-extern void my_gtk_entry_set_text (GtkWidget * wid, char *text,
-											  struct session *sess);
 extern struct session *new_dialog (struct session *sess);
 extern void create_panel_widget ();
 int key_handle_key_press (GtkWidget *, GdkEventKey *, gpointer);
@@ -108,8 +107,7 @@ int key_handle_key_press (GtkWidget *, GdkEventKey *, gpointer);
 static void userlist_button (GtkWidget * box, char *label, char *cmd,
 									  struct session *sess, int a, int b, int c,
 									  int d);
-static void my_gtk_toggle_state (GtkWidget * wid, int state, char which,
-											struct session *sess);
+static void my_gtk_togglebutton_state (GtkWidget * wid, int state);
 static void tree_update ();
 void tree_default_style (struct session *sess);
 
@@ -146,6 +144,26 @@ maingui_set_icon (GtkWidget *win)
 	gdk_window_set_icon_name (win->window, PACKAGE);
 }
 
+void
+fe_set_away (server *serv)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->server == serv)
+		{
+			if (!sess->gui->awaymenuitem)
+				sess->gui->awaymenuitem = main_menu_away_item;
+			if (sess->gui->awaymenuitem)
+				((GtkCheckMenuItem*)sess->gui->awaymenuitem)->active = serv->is_away;
+		}
+		list = list->next;
+	}
+}
+
 void
 fe_set_nonchannel (struct session *sess, int state)
 {
@@ -204,7 +222,7 @@ fe_buttons_update (struct session *sess)
 		gtk_widget_destroy (sess->gui->button_box);
 		sess->gui->button_box = 0;
 	}
-	if (!prefs.nouserlistbuttons)
+	if (prefs.userlistbuttons)
 		maingui_createbuttons (sess);
 }
 
@@ -282,8 +300,8 @@ fe_set_channel (struct session *sess)
 void
 fe_clear_channel (struct session *sess)
 {
+	gtk_widget_hide (sess->gui->namelistinfo);
 	gtk_entry_set_text (GTK_ENTRY (sess->gui->topicgad), "");
-	gtk_label_set_text (GTK_LABEL (sess->gui->namelistinfo), " ");
 	gtk_label_set_text (GTK_LABEL (sess->gui->changad), "<none>");
 #ifdef USE_PANEL
 	if (sess->gui->panel_button)
@@ -296,10 +314,9 @@ fe_clear_channel (struct session *sess)
 	{
 		int i;
 		for (i = 0; i < 8; i++)
-			my_gtk_toggle_state (sess->gui->flag_wid[i], FALSE, chan_flags[i],
-										sess);
-		my_gtk_entry_set_text (sess->gui->limit_entry, "", sess);
-		my_gtk_entry_set_text (sess->gui->key_entry, "", sess);
+			my_gtk_togglebutton_state (sess->gui->flag_wid[i], FALSE);
+		gtk_entry_set_text ((GtkEntry*)sess->gui->limit_entry, "");
+		gtk_entry_set_text ((GtkEntry*)sess->gui->key_entry, "");
 	}
 	if (sess->gui->op_xpm)
 		gtk_widget_destroy (sess->gui->op_xpm);
@@ -476,42 +493,6 @@ change_channel_flag (GtkWidget * wid, struct session *sess, char flag)
 	}
 }
 
-static void
-flagt_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 't');
-}
-
-static void
-flagn_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 'n');
-}
-
-static void
-flags_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 's');
-}
-
-static void
-flagi_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 'i');
-}
-
-static void
-flagp_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 'p');
-}
-
-static void
-flagm_hit (GtkWidget * wid, struct session *sess)
-{
-	change_channel_flag (wid, sess, 'm');
-}
-
 static void
 flagl_hit (GtkWidget * wid, struct session *sess)
 {
@@ -568,6 +549,31 @@ flagk_hit (GtkWidget * wid, struct session *sess)
 	}
 }
 
+static void
+channelmode_button_cb (GtkWidget *but, char *flag)
+{
+	session *sess;
+	char mode;
+
+	sess = gtk_object_get_user_data (GTK_OBJECT (but));
+	if (!sess)	/* can be zero if called from my_gtk_togglebutton_state() */
+		return;
+
+	mode = tolower (flag[0]);
+
+	switch (mode)
+	{
+	case 'l':
+		flagl_hit (but, sess);
+		break;
+	case 'k':
+		flagk_hit (but, sess);
+		break;
+	default:
+		change_channel_flag (but, sess, mode);
+	}
+}
+
 static void
 key_entry (GtkWidget * igad, struct session *sess)
 {
@@ -611,17 +617,21 @@ show_and_unfocus (GtkWidget * wid)
 	gtk_widget_show (wid);
 }
 
-static void
-add_flag_wid (GtkWidget ** wid, char *tip, GtkWidget * box, char *face,
-				  void *callback, gpointer userdata)
+static GtkWidget *
+add_flag_wid (char *tip, GtkWidget *box, char *face, session *sess)
 {
-	*wid = gtk_toggle_button_new_with_label (face);
-	gtk_widget_set_usize (*wid, 18, -1);
-	add_tip (*wid, tip);
-	gtk_box_pack_end (GTK_BOX (box), *wid, 0, 0, 0);
-	gtk_signal_connect (GTK_OBJECT (*wid), "toggled",
-							  GTK_SIGNAL_FUNC (callback), userdata);
-	show_and_unfocus (*wid);
+	GtkWidget *wid;
+
+	wid = gtk_toggle_button_new_with_label (face);
+	gtk_widget_set_usize (wid, 18, -1);
+	add_tip (wid, tip);
+	gtk_box_pack_end (GTK_BOX (box), wid, 0, 0, 0);
+	gtk_object_set_user_data (GTK_OBJECT (wid), sess);
+	gtk_signal_connect (GTK_OBJECT (wid), "toggled",
+							  GTK_SIGNAL_FUNC (channelmode_button_cb), face);
+	show_and_unfocus (wid);
+
+	return wid;
 }
 
 static void
@@ -774,7 +784,7 @@ maingui_word_clicked (GtkXText *xtext, char *word, GdkEventButton *even,
 			{
 			case WORD_URL:
 			case WORD_HOST:
-				goto_url (0, word);
+				goto_url (word);
 			}
 		}
 		return;
@@ -821,17 +831,27 @@ maingui_word_clicked (GtkXText *xtext, char *word, GdkEventButton *even,
 }
 
 void
-maingui_configure (GtkWidget * unused)
+maingui_configure (session *sess)
 {
-	if (menu_sess)
+	if (sess == 0)			/* for the main_window */
+		sess = menu_sess;
+
+	if (sess)
 	{
-		if (!menu_sess->is_shell)
+#ifdef USE_ZVT
+		if (sess->is_shell)
 		{
-			if (((GtkXText *) menu_sess->gui->textgad)->transparent)
+			gtk_widget_queue_draw (sess->gui->textgad);
+		} else
+		{
+#endif
+			if (((GtkXText *) sess->gui->textgad)->transparent)
 			{
-				gtk_widget_queue_draw (menu_sess->gui->textgad);
+				gtk_widget_queue_draw (sess->gui->textgad);
 			}
+#ifdef USE_ZVT
 		}
+#endif
 	}
 }
 
@@ -844,7 +864,6 @@ maingui_create_textlist (struct session *sess, GtkWidget * leftpane)
 
 	gtk_object_set_user_data (GTK_OBJECT (sess->gui->textgad), sess);
 
-	((GtkXText *) sess->gui->textgad)->double_buffer = prefs.double_buffer;
 	((GtkXText *) sess->gui->textgad)->wordwrap = prefs.wordwrap;
 	((GtkXText *) sess->gui->textgad)->max_auto_indent = prefs.max_auto_indent;
 	((GtkXText *) sess->gui->textgad)->auto_indent = prefs.auto_indent;
@@ -866,8 +885,8 @@ maingui_create_textlist (struct session *sess, GtkWidget * leftpane)
 									  channelwin_pix, prefs.transparent, prefs.tint);
 
 	gtk_widget_set_usize (sess->gui->textgad,
-								 prefs.mainwindow_width - 115,
-								 prefs.mainwindow_height - 105);
+								 prefs.mainwindow_width - 138,
+								 prefs.mainwindow_height - 106);
 	gtk_container_add (GTK_CONTAINER (leftpane), sess->gui->textgad);
 	show_and_unfocus (sess->gui->textgad);
 
@@ -881,7 +900,7 @@ maingui_create_textlist (struct session *sess, GtkWidget * leftpane)
 		gtk_signal_connect_object (GTK_OBJECT (sess->gui->window),
 											"configure_event",
 											GTK_SIGNAL_FUNC (maingui_configure),
-											GTK_OBJECT (sess->gui->textgad));
+											GTK_OBJECT (sess));
 
 	gtk_signal_connect (GTK_OBJECT (sess->gui->textgad), "word_click",
 							  GTK_SIGNAL_FUNC (maingui_word_clicked), sess);
@@ -892,23 +911,29 @@ gui_new_tab (session * sess)
 {
 	current_tab = sess;
 	menu_sess = sess;
+
 	if (!prefs.use_server_tab)
 		sess->server->front_session = sess;
+
 	fe_set_title (sess);
+
 	if (!sess->is_shell)
+	{
+		fe_set_away (sess->server);
 		gtk_widget_grab_focus (sess->gui->inputgad);
+	}
 
 #ifdef USE_PANEL
 	if (sess->gui->panel_button)
 		gtk_widget_set_style (GTK_BIN (sess->gui->panel_button)->child,
 									 normaltab_style);
 #endif
+
 	if (sess->new_data || sess->nick_said)
 	{
 		sess->nick_said = FALSE;
 		sess->new_data = FALSE;
 		gtk_widget_set_rc_style (sess->gui->changad);
-		/*gtk_widget_set_style (sess->gui->changad, normaltab_style); */
 	}
 }
 
@@ -1403,7 +1428,8 @@ gui_make_tab_window (struct session *sess)
 #else
 		main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 #endif
-
+		gtk_widget_set_usize (main_window, prefs.mainwindow_width,
+										prefs.mainwindow_height);
 		gtk_widget_realize (main_window);
 		maingui_set_icon (main_window);
 		gtk_signal_connect ((GtkObject *) main_window, "destroy",
@@ -1432,8 +1458,10 @@ gui_make_tab_window (struct session *sess)
 		main_menu_bar = wid = gtk_handle_box_new ();
 		gtk_container_add (GTK_CONTAINER (wid), main_menu);
 		gtk_box_pack_start (GTK_BOX (main_box), wid, FALSE, TRUE, 0);
-		gtk_widget_show (wid);
+		if (!prefs.hidemenu)
+			gtk_widget_show (wid);
 #endif
+		main_menu_away_item = sess->gui->awaymenuitem;
 		if (prefs.treeview)
 		{
 			main_hbox = gtk_hpaned_new ();
@@ -1484,8 +1512,8 @@ maingui_refresh (session * sess)
 int
 relink_window (GtkWidget * w, struct session *sess)
 {
-	GtkWidget *old, *wid;
-	int page, num, need = 0;
+	GtkWidget *old, *wid = NULL;
+	int num, need = 0;
 
 	if (sess->is_tab)
 	{
@@ -1511,6 +1539,8 @@ relink_window (GtkWidget * w, struct session *sess)
 #else
 		sess->gui->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 #endif
+		gtk_widget_set_usize (sess->gui->window, prefs.mainwindow_width,
+										prefs.mainwindow_height);
 		gtk_widget_realize (sess->gui->window);
 		maingui_set_icon (sess->gui->window);
 		if (!sess->is_dialog && !sess->is_shell)
@@ -1521,13 +1551,15 @@ relink_window (GtkWidget * w, struct session *sess)
 			wid = gtk_handle_box_new ();
 			gtk_box_pack_start (GTK_BOX (sess->gui->vbox), wid, FALSE, FALSE, 0);
 			gtk_box_reorder_child (GTK_BOX (sess->gui->vbox), wid, 0);
-			gtk_widget_show (wid);
 
 			sess->gui->menu = createmenus (0, sess, 1);
 			gtk_container_add (GTK_CONTAINER (wid), sess->gui->menu);
-			gtk_widget_show (sess->gui->menu);
+			gtk_widget_show_all (wid);
 #endif
 		}
+		gtk_widget_realize (sess->gui->window);
+		maingui_set_icon (sess->gui->window);
+
 		fe_set_title (sess);
 		gtk_signal_connect ((GtkObject *) sess->gui->window, "destroy",
 								  GTK_SIGNAL_FUNC (gtk_kill_session_callback), sess);
@@ -1540,8 +1572,13 @@ relink_window (GtkWidget * w, struct session *sess)
 		gtk_widget_reparent (sess->gui->vbox, wid);
 		gnome_app_set_contents (GNOME_APP (sess->gui->window), wid);
 #else
+		gtk_widget_show_all (sess->gui->vbox);
 		gtk_widget_reparent (sess->gui->vbox, sess->gui->window);
+		if (sess->gui->toolbox)
+			gtk_widget_hide (sess->gui->toolbox);
 #endif
+		gtk_widget_hide (sess->gui->tabright_button);
+		gtk_widget_hide (sess->gui->tableft_button);
 		if (!sess->is_dialog)
 		{
 			if (sess->channel[0] == 0)
@@ -1554,7 +1591,17 @@ relink_window (GtkWidget * w, struct session *sess)
 			gtk_widget_show (sess->gui->changad);
 		} else
 			sess->gui->changad = NULL;
-		gtk_widget_show_all (sess->gui->window);
+
+		gtk_widget_show (sess->gui->window);
+
+		/* Hide menu if it is turned off */
+		if (prefs.hidemenu)
+#ifdef USE_GNOME
+			menu_showhide ();
+#else
+			if (wid)
+				gtk_widget_hide (wid);
+#endif
 
 #ifdef USE_ZVT
 		if (sess->is_shell)
@@ -1563,8 +1610,6 @@ relink_window (GtkWidget * w, struct session *sess)
 #endif
 			gtk_xtext_refresh (GTK_XTEXT (sess->gui->textgad), 1);
 
-		if (sess->gui->toolbox)
-			gtk_widget_hide (sess->gui->toolbox);
 		num = gtk_notebook_page_num (GTK_NOTEBOOK (main_book), old);
 		gtk_notebook_remove_page (GTK_NOTEBOOK (main_book), num);
 		if (prefs.mainwindow_left || prefs.mainwindow_top)
@@ -1615,14 +1660,19 @@ relink_window (GtkWidget * w, struct session *sess)
 		gtk_widget_show (sess->gui->changad);
 		gtk_notebook_append_page (GTK_NOTEBOOK (main_book), sess->gui->window,
 										  sess->gui->changad);
-		gtk_widget_show_all (sess->gui->window);
+		gtk_widget_show (sess->gui->window);
+		gtk_widget_show (sess->gui->tabright_button);
+		gtk_widget_show (sess->gui->tableft_button);
+
 		if (need)
-			gtk_widget_show_all (main_window);
-		if (sess->gui->toolbox)
-			gtk_widget_hide (sess->gui->toolbox);
-		page =
-			gtk_notebook_page_num (GTK_NOTEBOOK (main_book), sess->gui->window);
-		maingui_pagetofront (page);
+			gtk_widget_show (main_window);
+
+#ifdef USE_GNOME
+		menu_showhide ();
+#endif
+
+		num = gtk_notebook_page_num (GTK_NOTEBOOK (main_book), sess->gui->window);
+		maingui_pagetofront (num);
 		gtk_idle_add ((GtkFunction) maingui_refresh, (gpointer) sess);
 
 		/* dialog windows don't have this box */
@@ -1832,13 +1882,6 @@ gui_create_toolbox (session *sess, GtkWidget *box)
 	else
 		gtk_widget_show (wid);
 	add_tip (wid, "Open Toolbox");
-/*
-	wid = gtk_button_new_with_label ("^");
-	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
-							  GTK_SIGNAL_FUNC (maingui_showhide_topic), sess);
-	gtk_box_pack_start (GTK_BOX (box), wid, 0, 0, 1);
-	gtk_widget_show (wid);
-	add_tip (wid, "Show/Hide topic bar");*/
 }
 
 void
@@ -1890,6 +1933,7 @@ gui_create_standard_buttons (session * sess, GtkWidget * win,
 	wid =
 		gtkutil_button (win, GNOME_STOCK_PIXMAP_BACK, 0, maingui_moveleft, 0,
 							 0);
+	sess->gui->tableft_button = wid;
 	gtk_box_pack_start (GTK_BOX (tbox), wid, 0, 0, 0);
 	show_and_unfocus (wid);
 	add_tip (wid, "Move tab left");
@@ -1897,11 +1941,13 @@ gui_create_standard_buttons (session * sess, GtkWidget * win,
 	wid =
 		gtkutil_button (win, GNOME_STOCK_PIXMAP_FORWARD, 0, maingui_moveright,
 							 0, 0);
+	sess->gui->tabright_button = wid;
 	gtk_box_pack_start (GTK_BOX (tbox), wid, 0, 0, 0);
 	show_and_unfocus (wid);
 	add_tip (wid, "Move tab right");
 #else
 	wid = gtk_button_new_with_label ("<");
+	sess->gui->tableft_button = wid;
 	gtk_box_pack_start (GTK_BOX (tbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (maingui_moveleft), 0);
@@ -1909,6 +1955,7 @@ gui_create_standard_buttons (session * sess, GtkWidget * win,
 	add_tip (wid, "Move tab left");
 
 	wid = gtk_button_new_with_label (">");
+	sess->gui->tabright_button = wid;
 	gtk_box_pack_start (GTK_BOX (tbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (maingui_moveright), 0);
@@ -1937,11 +1984,39 @@ ul_keypress_cb (GtkWidget *widget, GdkEventKey *event, session *sess)
 	}
 }*/
 
+static GtkWidget *
+create_infoframe (GdkPixmap *pix, GdkPixmap *mask, GtkWidget *box)
+{
+	GtkWidget *frame, *label, *hbox, *p;
+
+	frame = gtk_frame_new (0);
+	gtk_frame_set_shadow_type ((GtkFrame*)frame, GTK_SHADOW_IN);
+	gtk_container_add (GTK_CONTAINER (box), frame);
+	gtk_widget_show (frame);
+
+	hbox = gtk_hbox_new (0, 0);
+	gtk_container_add (GTK_CONTAINER (frame), hbox);
+	gtk_widget_show (hbox);
+
+	if (pix)
+	{
+		p = gtk_pixmap_new (pix, mask);
+		gtk_container_add (GTK_CONTAINER (hbox), p);
+		gtk_widget_show (p);
+	}
+
+	label = gtk_label_new (" ");
+	gtk_container_add (GTK_CONTAINER (hbox), label);
+	gtk_widget_show (label);
+
+	return label;
+}
+
 void
 create_window (struct session *sess)
 {
 	GtkWidget *leftpane, *rightpane;
-	GtkWidget *vbox, *tbox, *bbox, *nlbox, *wid;
+	GtkWidget *vbox, *tbox, *bbox, *nlbox, *wid, *infbox;
 	int justopened = FALSE;
 
 	if (sess->is_dialog)
@@ -1978,6 +2053,8 @@ create_window (struct session *sess)
 #else
 		sess->gui->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 #endif
+		gtk_widget_set_usize (sess->gui->window, prefs.mainwindow_width,
+										prefs.mainwindow_height);
 		fe_set_title (sess);
 		gtk_widget_realize (sess->gui->window);
 		maingui_set_icon (sess->gui->window);
@@ -2016,7 +2093,8 @@ create_window (struct session *sess)
 		wid = gtk_handle_box_new ();
 		gtk_container_add (GTK_CONTAINER (wid), sess->gui->menu);
 		gtk_box_pack_start (GTK_BOX (vbox), wid, FALSE, TRUE, 0);
-		gtk_widget_show (wid);
+		if (!prefs.hidemenu)
+			gtk_widget_show (wid);
 #endif
 	}
 	tbox = gtk_hbox_new (FALSE, 0);
@@ -2030,6 +2108,8 @@ create_window (struct session *sess)
 
 	if (!prefs.tabchannels)
 	{
+		gtk_widget_hide (sess->gui->tableft_button);
+		gtk_widget_hide (sess->gui->tabright_button);
 		sess->gui->changad = gtk_label_new ("<none>");
 		gtk_box_pack_start (GTK_BOX (tbox), sess->gui->changad, FALSE, FALSE, 5);
 		gtk_widget_show (sess->gui->changad);
@@ -2066,7 +2146,7 @@ create_window (struct session *sess)
 	show_and_unfocus (wid);
 	add_tip (wid, "Hide/Show Userlist");
 
-	if (!prefs.nochanmodebuttons)
+	if (prefs.chanmodebuttons)
 	{
 		sess->gui->key_entry = gtk_entry_new_with_max_length (16);
 		gtk_widget_set_usize (sess->gui->key_entry, 30, -1);
@@ -2078,8 +2158,7 @@ create_window (struct session *sess)
 		else
 			gtk_widget_show (sess->gui->key_entry);
 
-		add_flag_wid (&(sess->gui->flag_k), "Keyword", tbox, "K", flagk_hit,
-						  sess);
+		sess->gui->flag_k = add_flag_wid ("Keyword", tbox, "K", sess);
 
 		sess->gui->limit_entry = gtk_entry_new_with_max_length (10);
 		gtk_widget_set_usize (sess->gui->limit_entry, 30, -1);
@@ -2091,26 +2170,13 @@ create_window (struct session *sess)
 		else
 			gtk_widget_show (sess->gui->limit_entry);
 
-		add_flag_wid (&(sess->gui->flag_l), "User Limit", tbox, "L", flagl_hit,
-						  sess);
-
-		add_flag_wid (&(sess->gui->flag_m), "Moderated", tbox, "M", flagm_hit,
-						  sess);
-
-		add_flag_wid (&(sess->gui->flag_p), "Private", tbox, "P", flagp_hit,
-						  sess);
-
-		add_flag_wid (&(sess->gui->flag_i), "Invite Only", tbox, "I", flagi_hit,
-						  sess);
-
-		add_flag_wid (&(sess->gui->flag_s), "Secret", tbox, "S", flags_hit,
-						  sess);
-
-		add_flag_wid (&(sess->gui->flag_n), "No outside messages", tbox, "N",
-						  flagn_hit, sess);
-
-		add_flag_wid (&(sess->gui->flag_t), "Topic Protection", tbox, "T",
-						  flagt_hit, sess);
+		sess->gui->flag_l = add_flag_wid ("User Limit", tbox, "L", sess);
+		sess->gui->flag_m = add_flag_wid ("Moderated", tbox, "M", sess);
+		sess->gui->flag_p = add_flag_wid ("Private", tbox, "P", sess);
+		sess->gui->flag_i = add_flag_wid ("Invite Only", tbox, "I", sess);
+		sess->gui->flag_s = add_flag_wid ("Secret", tbox, "S", sess);
+		sess->gui->flag_n = add_flag_wid ("No outside messages", tbox, "N", sess);
+		sess->gui->flag_t = add_flag_wid ("Topic Protection", tbox, "T", sess);
 
 	} else
 		sess->gui->flag_wid[0] = 0;
@@ -2118,7 +2184,7 @@ create_window (struct session *sess)
 	leftpane = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (leftpane);
 
-	if (!prefs.nopaned)
+	if (prefs.paned_userlist)
 	{
 		sess->gui->paned = gtk_hpaned_new ();
 		gtk_container_add (GTK_CONTAINER (vbox), sess->gui->paned);
@@ -2128,7 +2194,7 @@ create_window (struct session *sess)
 	gtk_widget_show (rightpane);
 	sess->gui->userlistbox = rightpane;
 
-	if (!prefs.nopaned)
+	if (prefs.paned_userlist)
 	{
 		gtk_paned_pack1 (GTK_PANED (sess->gui->paned), leftpane, TRUE, TRUE);
 		gtk_paned_pack2 (GTK_PANED (sess->gui->paned), rightpane, FALSE, TRUE);
@@ -2146,13 +2212,12 @@ create_window (struct session *sess)
 	gtk_container_add (GTK_CONTAINER (rightpane), nlbox);
 	gtk_widget_show (nlbox);
 
-	wid = gtk_frame_new (0);
-	gtk_box_pack_start (GTK_BOX (nlbox), wid, 0, 0, 0);
-	gtk_widget_show (wid);
+	sess->gui->namelistinfo = infbox = gtk_hbox_new (0, 0);
+	gtk_box_pack_start (GTK_BOX (nlbox), infbox, 0, 0, 0);
 
-	sess->gui->namelistinfo = gtk_label_new (" ");
-	gtk_container_add (GTK_CONTAINER (wid), sess->gui->namelistinfo);
-	gtk_widget_show (sess->gui->namelistinfo);
+	sess->gui->namelistinfo_o = create_infoframe (pix_op, mask_op, infbox);
+	sess->gui->namelistinfo_v = create_infoframe (pix_voice, mask_voice, infbox);
+	sess->gui->namelistinfo_t = create_infoframe (0, 0, infbox);
 
 	maingui_create_textlist (sess, leftpane);
 	sess->gui->leftpane = leftpane;
@@ -2213,7 +2278,7 @@ create_window (struct session *sess)
 		gtk_widget_show (wid);
 	}
 
-	if (!prefs.nouserlistbuttons)
+	if (prefs.userlistbuttons)
 		maingui_createbuttons (sess);
 	else
 		sess->gui->button_box = 0;
@@ -2303,11 +2368,9 @@ create_window (struct session *sess)
 			sess->userlisthidden = GTK_PANED (sess->gui->paned)->handle_xpos;
 	}
 
-	if (prefs.hidemenu)
 #ifdef USE_GNOME
-		menu_showhide (sess, 1);
-#else
-		gtk_widget_hide (main_menu_bar);
+	if (!sess->is_tab || justopened)
+		menu_showhide ();
 #endif
 
 	fe_set_nonchannel (sess, FALSE);
@@ -2340,8 +2403,8 @@ maingui_delink_nontab (GtkWidget * wid, GtkWidget * box)
 		rld->win = win;
 		gtk_window_set_title (GTK_WINDOW (win), rld->ltitle);
 		gtk_notebook_remove_page (GTK_NOTEBOOK (main_book), page);
+		gtk_widget_show (win);
 
-		gtk_widget_show_all (win);
 		rld->hbox = NULL;
 	} else
 	{
@@ -2356,17 +2419,15 @@ maingui_delink_nontab (GtkWidget * wid, GtkWidget * box)
 		gtk_widget_reparent (box, win);
 		gtk_signal_disconnect_by_data (GTK_OBJECT (rld->win), box);
 		gtk_widget_destroy (GTK_WIDGET (rld->win));
+
 		label = gtk_label_new (rld->stitle);
 		gtk_widget_show (label);
+
 		gtk_notebook_append_page (GTK_NOTEBOOK (main_book), win, label);
 		if (prefs.newtabstofront)
-		{
-			gtk_idle_add ((GtkFunction) maingui_pagetofront, (gpointer)
-							  gtk_notebook_page_num (GTK_NOTEBOOK (main_book), box));
-		}
-		gtk_widget_show_all (win);
-		if (need)
-			gtk_widget_show_all (main_window);
+			maingui_pagetofront (gtk_notebook_page_num (GTK_NOTEBOOK (main_book), box));
+		gtk_widget_show (main_window);
+
 		rld->win = NULL;
 		rld->hbox = win;
 	}
@@ -2493,73 +2554,30 @@ fe_session_callback (struct session *sess)
 	}
 }
 
-void
+/*void
 fe_server_callback (struct server *serv)
 {
-	/*if (serv->gui->rawlog_window)
-	   { */
-	/* we don't want the "destroy" callback to be called */
-	/*gtk_signal_disconnect_by_data (GTK_OBJECT (serv->gui->rawlog_window), serv);
-	   gtk_widget_destroy (serv->gui->rawlog_window);
-	   } */
-}
-
-void
-my_gtk_entry_set_text (GtkWidget * wid, char *text, struct session *sess)
-{
-	void *callback;
-	if (sess->gui->flag_wid[0] == 0)
-		return;
-	if (wid == sess->gui->limit_entry)
-		callback = (void *) limit_entry;
-	else
-		callback = (void *) key_entry;
-	gtk_signal_disconnect_by_data (GTK_OBJECT (wid), sess);
-	gtk_entry_set_text (GTK_ENTRY (wid), text);
-	gtk_signal_connect (GTK_OBJECT (wid), "activate",
-							  GTK_SIGNAL_FUNC (callback), sess);
-}
+	if (serv->gui->rawlog_window)
+	{ 
+		we don't want the "destroy" callback to be called
+		gtk_signal_disconnect_by_data (GTK_OBJECT (serv->gui->rawlog_window), serv);
+		gtk_widget_destroy (serv->gui->rawlog_window);
+	} 
+}*/
 
 static void
-my_gtk_toggle_state (GtkWidget * wid, int state, char which,
-							struct session *sess)
+my_gtk_togglebutton_state (GtkWidget *wid, int state)
 {
-	void *callback;
-	switch (which)
-	{
-	case 'k':
-		callback = flagk_hit;
-		break;
-	case 'l':
-		callback = flagl_hit;
-		break;
-	case 'm':
-		callback = flagm_hit;
-		break;
-	case 'p':
-		callback = flagp_hit;
-		break;
-	case 'i':
-		callback = flagi_hit;
-		break;
-	case 's':
-		callback = flags_hit;
-		break;
-	case 'n':
-		callback = flagn_hit;
-		break;
-	case 't':
-		callback = flagt_hit;
-		break;
-	default:
-		gtk_toggle_button_set_active ((GtkToggleButton *) wid, state);
-		return;
-	}
-	gtk_signal_disconnect_by_data (GTK_OBJECT (wid), sess);
-	gtk_toggle_button_set_active ((GtkToggleButton *) wid, state);
-	if (callback)
-		gtk_signal_connect (GTK_OBJECT (wid), "toggled",
-								  GTK_SIGNAL_FUNC (callback), sess);
+	gpointer udata;
+
+	/* remove the userdata so the callback ignores this event */
+	udata = gtk_object_get_user_data (GTK_OBJECT (wid));
+	gtk_object_set_user_data (GTK_OBJECT (wid), 0);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid), state);
+
+	/* now put it back */
+	gtk_object_set_user_data (GTK_OBJECT (wid), udata);
 }
 
 void
@@ -2591,8 +2609,7 @@ fe_update_mode_buttons (struct session *sess, char mode, char sign)
 			if (chan_flags[i] == mode)
 			{
 				if (GTK_TOGGLE_BUTTON (sess->gui->flag_wid[i])->active != state)
-					my_gtk_toggle_state (sess->gui->flag_wid[i], state,
-												chan_flags[i], sess);
+					my_gtk_togglebutton_state (sess->gui->flag_wid[i], state);
 			}
 		}
 	}
