@@ -44,7 +44,7 @@ extern GdkColor colors[];
 extern GSList *sess_list;
 extern GdkFont *font_normal;
 extern GtkWidget *main_window, *main_book;
-extern GtkWidget *main_menu_bar;
+extern GtkWidget *main_menu_bar, *main_menu;
 
 extern int cfg_get_bool (char *var);
 extern void maingui_showhide_topic (GtkWidget * button, session * sess);
@@ -55,7 +55,7 @@ extern void search_open (session * sess);
 extern void xchat_exec (char *cmd);
 extern void show_and_unfocus (GtkWidget * wid);
 extern char *default_file (void);
-extern void menu_about (GtkWidget * wid, gpointer sess);
+extern void menu_about (GtkWidget * wid, gpointer none);
 extern void userlist_hide (GtkWidget * igad, struct session *sess);
 extern void palette_edit (void);
 extern GdkFont *my_font_load (char *fontname);
@@ -98,42 +98,9 @@ extern GSList *usermenu_list;
 extern GSList *urlhandler_list;
 static GSList *submenu_list;
 
-/* escapes \ $ ` ' " & | ; */
-
-/*static char *
-url_escape_chars (char *url)
-{
-	char *buf, *orig_buf;
-
-	orig_buf = buf = malloc ((strlen (url) * 2) + 1);
-
-	while (1)
-	{
-		switch (*url)
-		{
-		case 0:
-			*buf = '\0';
-			return orig_buf;
-		case '\\':
-		case '`':
-		case '\'':
-		case '\"':
-		case '$':
-		case '&':
-		case '|':
-		case ';':
-			*buf = '\\';
-			buf++;
-		default:
-			*buf = *url;
-			buf++;
-		}
-		url++;
-	}
-}*/
 
 void
-goto_url (void *unused, char *url)
+goto_url (char *url)
 {
 #ifdef USE_GNOME
 	gnome_url_show (url);
@@ -251,7 +218,10 @@ popup_menu_cb (GtkWidget * item, char *cmd)
 	/* the userdata is set in menu_quick_item() */
 	nick = gtk_object_get_user_data (GTK_OBJECT (item));
 
-	nick_command_parse (menu_sess, cmd, nick, nick);
+	if (!menu_sess)	/* for url grabber window */
+		nick_command_parse (sess_list->data, cmd, nick, nick);
+	else
+		nick_command_parse (menu_sess, cmd, nick, nick);
 }
 
 static void
@@ -259,10 +229,10 @@ menu_toggle_item (char *label, GtkWidget *menu, void *callback, void *userdata,
 						int state)
 {
 	GtkWidget *item;
-	if (!label)
-		item = gtk_check_menu_item_new ();
-	else
-		item = gtk_check_menu_item_new_with_label (label);
+
+	item = gtk_check_menu_item_new_with_label (label);
+	/* Always show checkbox, not just when it is moused over. */
+	gtk_check_menu_item_set_show_toggle ((GtkCheckMenuItem*)item, TRUE);
 	gtk_check_menu_item_set_state ((GtkCheckMenuItem*)item, state);
 	gtk_menu_append (GTK_MENU (menu), item);
 	gtk_signal_connect (GTK_OBJECT (item), "activate",
@@ -449,32 +419,52 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick)
 }
 
 void
-menu_showhide (session *sess, int force_hide)
+menu_showhide (void)
 {
+	session *sess;
+	GSList *list;
 	GtkWidget *menu;
 #ifdef USE_GNOME
 	GtkWidget *win;
+	int resize_done = FALSE;	/* resize main_window only once */
 #endif
 
-	if (menu_sess->is_tab)
-		menu = main_menu_bar;
-	else
-		menu = menu_sess->gui->menu->parent;
+	list = sess_list;
+	while (list)
+	{
+		sess = list->data;
 
-	if (GTK_WIDGET_VISIBLE (menu) || force_hide)
-	{
-		gtk_widget_hide (menu);
-		prefs.hidemenu = 1;
-	} else
-	{
-		gtk_widget_show (menu);
-		prefs.hidemenu = 0;
-	}
+		if (sess->is_tab)
+		{
+			menu = main_menu_bar;
+		} else
+		{
+			if (!sess->gui->menu)
+				goto cont;
+			menu = sess->gui->menu->parent;	/* really the handlebox */
+		}
+
+		if (prefs.hidemenu)
+		{
+			gtk_widget_hide (menu);
 #ifdef USE_GNOME
-	win = gtk_widget_get_toplevel (menu);
-	gdk_window_resize (win->window, win->allocation.width-1, win->allocation.height);
-	gdk_window_resize (win->window, win->allocation.width, win->allocation.height);
+			/* needed to hide the menubar properly under gnome. */
+			if (!resize_done || !sess->is_tab)
+			{
+				win = gtk_widget_get_toplevel (menu);
+				gdk_window_resize (win->window, win->allocation.width-1, win->allocation.height);
+				gdk_window_resize (win->window, win->allocation.width, win->allocation.height);
+				if (sess->is_tab)
+					resize_done = TRUE;
+			}
 #endif
+		} else
+		{
+			gtk_widget_show (menu);
+		}
+cont:
+		list = list->next;
+	}
 }
 
 static void
@@ -483,7 +473,15 @@ menu_middle_cb (GtkWidget *item, int n)
 	switch (n)
 	{
 	case 0:
-		menu_showhide (menu_sess, 0);
+		if (prefs.hidemenu)
+		{
+			prefs.hidemenu = 0;
+			menu_showhide ();
+		} else
+		{
+			prefs.hidemenu = 1;
+			menu_showhide ();
+		}
 		break;
 	case 1:
 		maingui_showhide_topic (0, menu_sess);
@@ -501,7 +499,6 @@ menu_middlemenu (session *sess, GdkEventButton *event)
 
 	menu = createmenus (0, sess, 0);
 
-	/*menu_toggle_item ("Menu Bar", menu, (void*)menu_middle_cb, 0, GTK_WIDGET_VISIBLE (main_menu_bar));*/
 	menu_toggle_item ("Menu Bar", menu, (void*)menu_middle_cb, 0, !prefs.hidemenu);
 	menu_toggle_item ("Topic Bar", menu, (void*)menu_middle_cb, (void*)1, prefs.topicbar);
 	menu_toggle_item ("User List", menu, (void*)menu_middle_cb, (void*)2, !prefs.hideuserlist);
@@ -513,14 +510,41 @@ void
 menu_urlmenu (session *sess, GdkEventButton *event, char *url)
 {
 	GtkWidget *menu;
+	int i;
+	char *tmp;
 
 	if (str_copy)
 		free (str_copy);
-	str_copy = strdup (url);
-/*	str_copy = url_escape_chars (url);*/
+
+	/* replace all commas with %2c */
+	str_copy = malloc ((strlen (url) * 3) + 1);
+	i = 0;
+	while (*url)
+	{
+		if (*url == ',')
+		{
+			str_copy[i++] = '%';
+			str_copy[i++] = '2';
+			str_copy[i] = 'c';
+		} else
+			str_copy[i] = *url;
+		i++;
+		url++;
+	}
+	str_copy[i] = 0;
 
 	menu = gtk_menu_new ();
-	menu_quick_item (0, url, menu, 1, 0);
+	if (strlen (str_copy) > 51)
+	{
+		tmp = strdup (str_copy);
+		tmp[47] = tmp[48] = tmp[49] = '.';
+		tmp[50] = 0;
+		menu_quick_item (0, tmp, menu, 1, 0);
+		free (tmp);
+	} else
+	{
+		menu_quick_item (0, str_copy, menu, 1, 0);
+	}
 	menu_quick_item (0, 0, menu, 1, 0);
 
 	menu_create (menu, urlhandler_list, str_copy);
@@ -616,7 +640,7 @@ menu_open_server_list (GtkWidget *wid, gpointer none)
 }
 
 static void
-menu_settings (GtkWidget * wid, gpointer sess)
+menu_settings (GtkWidget * wid, gpointer none)
 {
 	settings_opengui (menu_sess);
 }
@@ -641,7 +665,9 @@ menu_shell_title (ZvtTerm * term, int type, char *newtitle, session * sess)
 	}
 }
 
+#ifdef USE_XLIB
 #include <gdk/gdkx.h>
+#endif
 
 /*                  30 31 32 33 34 35 36  37  30  31 32 33 34  35  36  37 */
 static
@@ -676,13 +702,13 @@ menu_newshell_set_palette (session *sess)
 static void
 menu_newshell_tab (GtkWidget * wid, struct session *sess)
 {
-	int oldb = prefs.nochanmodebuttons;
-	int oldp = prefs.nopaned;
+	int oldb = prefs.chanmodebuttons;
+	int oldp = prefs.paned_userlist;
 	struct server *serv;
 	char *shell, *name;
 	char buf[16];
 
-	prefs.nochanmodebuttons = prefs.nopaned = 1;
+	prefs.chanmodebuttons = prefs.paned_userlist = 0;
 	serv = new_server ();
 	if (!serv)
 		return;
@@ -691,8 +717,8 @@ menu_newshell_tab (GtkWidget * wid, struct session *sess)
 	else
 		sess = new_session (serv, 0);
 	sess->is_shell = 1;
-	prefs.nochanmodebuttons = oldb;
-	prefs.nopaned = oldp;
+	prefs.chanmodebuttons = oldb;
+	prefs.paned_userlist = oldp;
 
 	gtk_widget_destroy (sess->gui->textgad);
 	gtk_widget_destroy (sess->gui->vscrollbar);
@@ -743,10 +769,12 @@ menu_newshell_tab (GtkWidget * wid, struct session *sess)
 		break;
 
 	case 0:
-#ifdef HAVE_SETENV				  /* solaris is lame */
+#ifdef USE_XLIB
 		sprintf (buf, "%d",
 					(int) GDK_WINDOW_XWINDOW (GTK_WIDGET (sess->gui->textgad)->
 													  window));
+#endif
+#ifdef HAVE_SETENV				  /* solaris is lame */
 		setenv ("WINDOWID", buf, 1);
 		setenv ("COLORTERM", "zterm", 0);
 #endif
@@ -754,15 +782,15 @@ menu_newshell_tab (GtkWidget * wid, struct session *sess)
 		_exit (127);
 	}
 
-	g_free (shell);				/* FIXME: is this a race condition ???? */
-
 	if (main_book)
 		gdk_window_set_back_pixmap (main_book->window, 0, 0);
+
+	g_free (shell);				/* FIXME: is this a race condition ???? */
 }
 #endif
 
 static void
-menu_newserver_window (GtkWidget * wid, gpointer sess)
+menu_newserver_window (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
 	struct server *serv;
@@ -775,7 +803,7 @@ menu_newserver_window (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_newchannel_window (GtkWidget * wid, gpointer sess)
+menu_newchannel_window (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
 
@@ -785,7 +813,7 @@ menu_newchannel_window (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_newserver_tab (GtkWidget * wid, gpointer sess)
+menu_newserver_tab (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
 	struct server *serv;
@@ -798,7 +826,7 @@ menu_newserver_tab (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_newchannel_tab (GtkWidget * wid, gpointer sess)
+menu_newchannel_tab (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
 
@@ -808,23 +836,13 @@ menu_newchannel_tab (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_rawlog (GtkWidget * wid, gpointer sess)
+menu_rawlog (GtkWidget * wid, gpointer none)
 {
 	open_rawlog (menu_sess->server);
 }
 
-#ifdef USE_GNOME
-
 static void
-menu_toggle_option (GtkWidget *wid, int *var)
-{
-	*var = !(*var);
-}
-
-#endif
-
-static void
-menu_autodccsend (GtkWidget * wid, gpointer sess)
+menu_autodccsend (GtkWidget * wid, gpointer none)
 {
 	prefs.autodccsend = !prefs.autodccsend;
 	if (prefs.autodccsend)
@@ -840,7 +858,7 @@ menu_autodccsend (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_close (GtkWidget * wid, gpointer sess)
+menu_close (GtkWidget * wid, gpointer none)
 {
 	gtk_widget_destroy (menu_sess->gui->window);
 }
@@ -852,65 +870,49 @@ menu_search ()
 }
 
 static void
-menu_flushbuffer (GtkWidget * wid, gpointer sess)
+menu_flushbuffer (GtkWidget * wid, gpointer none)
 {
 	if (!menu_sess->is_shell)
 		fe_text_clear (menu_sess);
 }
 
-static GtkWidget *freq = 0;
-
 static void
-close_savebuffer (void)
+savebuffer_req_done (session *sess, void *arg2, char *file)
 {
-	gtk_widget_destroy (freq);
-	freq = 0;
-}
+	int fh, newlen;
+	char *buf;
+	textentry *ent;
 
-static void
-savebuffer_req_done (GtkWidget * wid, struct session *sess)
-{
-	int fh;
-	char *buf, *sbuf;
-
-	fh = open (gtk_file_selection_get_filename (GTK_FILE_SELECTION (freq)),
-				  O_TRUNC | O_WRONLY | O_CREAT, 0600);
-
-	close_savebuffer ();
+	if (!file)
+		return;
 
+	fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
 	if (fh != -1)
 	{
-		buf = gtk_xtext_get_chars (GTK_XTEXT (sess->gui->textgad));
-		if (buf)
+		ent = GTK_XTEXT(sess->gui->textgad)->text_first;
+		while (ent)
 		{
-			sbuf = gtk_xtext_strip_color (buf, strlen (buf), NULL, NULL);
-			g_free (buf);
-			write (fh, sbuf, strlen (sbuf));
-			free (sbuf);
+			buf = gtk_xtext_strip_color (ent->str, ent->str_len, NULL,
+								&newlen, GTK_XTEXT(sess->gui->textgad)->fonttype);
+			write (fh, buf, strlen (buf));
+			write (fh, "\n", 1);
+			free (buf);
+			ent = ent->next;
 		}
-
 		close (fh);
-	} else
-		gtkutil_simpledialog ("Cannot write to that file.");
+	}
+	free (file);
 }
 
 static void
-menu_savebuffer (GtkWidget * wid, gpointer sess)
+menu_savebuffer (GtkWidget * wid, gpointer none)
 {
-	if (freq)
-		close_savebuffer ();
-	freq = gtk_file_selection_new ("Select an output filename");
-	gtk_signal_connect_object (GTK_OBJECT
-										(GTK_FILE_SELECTION (freq)->cancel_button),
-										"clicked", (GtkSignalFunc) close_savebuffer, 0);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (freq)->ok_button),
-							  "clicked", (GtkSignalFunc) savebuffer_req_done,
-							  menu_sess);
-	gtk_widget_show (freq);
+	gtkutil_file_req ("Select an output filename", savebuffer_req_done,
+							menu_sess, 0, TRUE);
 }
 
 static void
-menu_wallops (GtkWidget * wid, gpointer sess)
+menu_wallops (GtkWidget * wid, gpointer none)
 {
 	char tbuf[128];
 	prefs.wallops = !prefs.wallops;
@@ -925,7 +927,7 @@ menu_wallops (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_servernotice (GtkWidget * wid, gpointer sess)
+menu_servernotice (GtkWidget * wid, gpointer none)
 {
 	char tbuf[128];
 	prefs.servernotice = !prefs.servernotice;
@@ -940,25 +942,13 @@ menu_servernotice (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_away (GtkWidget * wid, gpointer sess)
+menu_away (GtkWidget * wid, gpointer none)
 {
-	char tbuf[256];
-	prefs.away = !prefs.away;
-	if (menu_sess->server->connected)
-	{
-		if (prefs.away)
-		{
-			snprintf (tbuf, 255, "/away %s", prefs.awayreason);
-			handle_command (tbuf, menu_sess, FALSE, FALSE);
-		} else
-		{
-			handle_command ("/away", menu_sess, FALSE, FALSE);
-		}
-	}
+	handle_command ("/away", menu_sess, FALSE, FALSE);
 }
 
 static void
-menu_invisible (GtkWidget * wid, gpointer sess)
+menu_invisible (GtkWidget * wid, gpointer none)
 {
 	char tbuf[128];
 	prefs.invisible = !prefs.invisible;
@@ -973,13 +963,13 @@ menu_invisible (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_help (GtkWidget * wid, gpointer sess)
+menu_help (GtkWidget * wid, gpointer none)
 {
 	cmd_help (menu_sess, 0, 0, 0);
 }
 
 static void
-menu_savedefault (GtkWidget * wid, gpointer sess)
+menu_savedefault (GtkWidget * wid, gpointer none)
 {
 	palette_save ();
 	if (save_config ())
@@ -987,7 +977,7 @@ menu_savedefault (GtkWidget * wid, gpointer sess)
 }
 
 static void
-menu_chanlist (GtkWidget * wid, gpointer sess)
+menu_chanlist (GtkWidget * wid, gpointer none)
 {
 	chanlist_opengui (menu_sess->server);
 }
@@ -1140,7 +1130,7 @@ menu_pythonlist (void)
                            "%s  =  selected nick\n"\
 									"%t  =  time/date\n"
 
-#define ulbutton_help       "Userlist Popup - Special codes:\n\n"\
+#define ulbutton_help       "Userlist Buttons - Special codes:\n\n"\
                            "%a  =  all selected nicks\n"\
                            "%c  =  current channel\n"\
                            "%h  =  selected nick's hostname\n"\
@@ -1236,6 +1226,54 @@ menu_reload (void)
 	free (buf);
 }
 
+static void
+menu_autorejoin (GtkWidget *wid, gpointer none)
+{
+	prefs.autorejoin = !prefs.autorejoin;
+}
+
+static void
+menu_autoreconnect (GtkWidget *wid, gpointer none)
+{
+	prefs.autoreconnect = !prefs.autoreconnect;
+}
+
+static void
+menu_autoreconnectonfail (GtkWidget *wid, gpointer none)
+{
+	prefs.autoreconnectonfail = !prefs.autoreconnectonfail;
+}
+
+static void
+menu_autodialog (GtkWidget *wid, gpointer none)
+{
+	prefs.autodialog = !prefs.autodialog;
+}
+
+static void
+menu_autodccchat (GtkWidget *wid, gpointer none)
+{
+	prefs.autodccchat = !prefs.autodccchat;
+}
+
+static void
+menu_saveexit (GtkWidget *wid, gpointer none)
+{
+	prefs.autosave = !prefs.autosave;
+}
+
+static void
+menu_docs (GtkWidget *wid, gpointer none)
+{
+	goto_url ("http://xchat.org/docs.html");
+}
+
+static void
+menu_webpage (GtkWidget *wid, gpointer none)
+{
+	goto_url ("http://xchat.org");
+}
+
 #ifdef USE_GNOME
 
 #include "../pixmaps/win.xpm"
@@ -1389,24 +1427,24 @@ static GnomeUIInfo usermodesmenu[] = {
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Auto Rejoin on Kick"), 0,
-	 menu_toggle_option, &prefs.autorejoin, 0, 0, 0, 0, 0, 0},
+	 menu_autorejoin, 0, 0, 0, 0, 0, 0, 0},
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Auto ReConnect to Server"), 0,
-	 menu_toggle_option, &prefs.autoreconnect, 0, 0, 0, 0, 0, 0},
+	 menu_autoreconnect, 0, 0, 0, 0, 0, 0, 0},
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Never-give-up ReConnect"), 0,
-	 menu_toggle_option, &prefs.autoreconnectonfail, 0, 0, 0, 0, 0, 0},
+	 menu_autoreconnectonfail, 0, 0, 0, 0, 0, 0, 0},
 	GNOMEUIINFO_SEPARATOR,
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Auto Open Dialog Windows"), 0,
-	 menu_toggle_option, &prefs.autodialog, 0, 0, 0, 0, 0, 0},
+	 menu_autodialog, 0, 0, 0, 0, 0, 0, 0},
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Auto Accept DCC Chat"), 0,
-	 menu_toggle_option, &prefs.autodccchat, 0, 0, 0, 0, 0, 0},
+	 menu_autodccchat, 0, 0, 0, 0, 0, 0, 0},
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Auto Accept DCC Send"), 0,
@@ -1492,7 +1530,9 @@ static GnomeUIInfo settingsmenu[] = {
 	{
 	 GNOME_APP_UI_TOGGLEITEM,
 	 N_("Save Settings on exit"), 0,
-	 menu_toggle_option, &prefs.autosave, 0, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 0, 0, 0},
+	 menu_saveexit, 0, 0,
+	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
+	 0, 0, 0},
 	GNOMEUIINFO_END
 };
 
@@ -1507,13 +1547,13 @@ static GnomeUIInfo helpmenu[] = {
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("X-Chat Homepage.."), 0,
-	 goto_url, "http://xchat.org", 0,
+	 menu_webpage, 0, 0,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
 	 0, 0, 0},
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("Online Docs.."), 0,
-	 goto_url, "http://xchat.org/docs.html", 0,
+	 menu_docs, 0, 0,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
 	 0, 0, 0},
 	GNOMEUIINFO_SEPARATOR,
@@ -1615,18 +1655,17 @@ createmenus (void *app, struct session *sess, int bar)
 	{
 		gnome_app_create_menus (GNOME_APP (app), mainmenu);
 		menu = GNOME_APP (app)->menubar;
+		gtk_menu_item_right_justify (GTK_MENU_ITEM (mainmenu[6].widget));
 	} else
 	{
 		menu = gnome_popup_menu_new (mainmenu);
 	}
 
-	gtk_menu_item_right_justify (GTK_MENU_ITEM (mainmenu[6].widget));
-
 	/* is it legal to poke in the initial values?? */
 	((GtkCheckMenuItem *) (usermodesmenu[0].widget))->active = prefs.invisible;
 	((GtkCheckMenuItem *) (usermodesmenu[1].widget))->active = prefs.wallops;
 	((GtkCheckMenuItem *) (usermodesmenu[2].widget))->active = prefs.servernotice;
-	((GtkCheckMenuItem *) (usermodesmenu[4].widget))->active = prefs.away;
+	((GtkCheckMenuItem *) (usermodesmenu[4].widget))->active = sess->server->is_away;
 	((GtkCheckMenuItem *) (usermodesmenu[6].widget))->active = prefs.autorejoin;
 	((GtkCheckMenuItem *) (usermodesmenu[7].widget))->active = prefs.autoreconnect;
 	((GtkCheckMenuItem *) (usermodesmenu[8].widget))->active = prefs.autoreconnectonfail;
@@ -1654,7 +1693,12 @@ createmenus (void *app, struct session *sess, int bar)
 	gtk_widget_set_sensitive (xchatmenu[7].widget, FALSE);
 #endif
 
-	sess->gui->usermenu = GTK_MENU_ITEM (mainmenu[5].widget)->submenu;
+	if (bar)	/* popup menus can't alter sess->gui, only a menubar! */
+	{
+		sess->gui->usermenu = GTK_MENU_ITEM (mainmenu[5].widget)->submenu;
+		sess->gui->awaymenuitem = usermodesmenu[4].widget;
+	}
+
 	usermenu_create (GTK_MENU_ITEM (mainmenu[5].widget)->submenu);
 
 	return menu;
@@ -1662,54 +1706,6 @@ createmenus (void *app, struct session *sess, int bar)
 
 #else
 
-static void
-menu_docs ()
-{
-	goto_url (0, "http://xchat.org/docs.html");
-}
-
-static void
-menu_webpage ()
-{
-	goto_url (0, "http://xchat.org");
-}
-
-static void
-menu_autorejoin (GtkWidget *wid, gpointer sess)
-{
-	prefs.autorejoin = !prefs.autorejoin;
-}
-
-static void
-menu_autoreconnect (GtkWidget *wid, gpointer sess)
-{
-	prefs.autoreconnect = !prefs.autoreconnect;
-}
-
-static void
-menu_autoreconnectonfail (GtkWidget *wid, gpointer sess)
-{
-	prefs.autoreconnectonfail = !prefs.autoreconnectonfail;
-}
-
-static void
-menu_autodialog (GtkWidget *wid, gpointer sess)
-{
-	prefs.autodialog = !prefs.autodialog;
-}
-
-static void
-menu_autodccchat (GtkWidget *wid, gpointer sess)
-{
-	prefs.autodccchat = !prefs.autodccchat;
-}
-
-static void
-menu_saveexit (GtkWidget *wid, gpointer sess)
-{
-	prefs.autosave = !prefs.autosave;
-}
-
 static struct mymenu mymenu[] = {
 	{M_NEWMENU, N_("X-Chat"), 0, 0, 1},
 	{M_MENU, N_("Server List.."), (menucallback) menu_open_server_list, 0, 1},
@@ -1842,6 +1838,7 @@ createmenus (void *app, struct session *sess, int bar)
 	mymenu[27-menuoffset].state = prefs.invisible;
 	mymenu[28-menuoffset].state = prefs.wallops;
 	mymenu[29-menuoffset].state = prefs.servernotice;
+	mymenu[31-menuoffset].state = sess->server->is_away;
 	mymenu[33-menuoffset].state = prefs.autorejoin;
 	mymenu[34-menuoffset].state = prefs.autoreconnect;
 	mymenu[35-menuoffset].state = prefs.autoreconnectonfail;
@@ -1882,6 +1879,9 @@ createmenus (void *app, struct session *sess, int bar)
 			break;
 		case M_MENUTOG:
 			item = gtk_check_menu_item_new_with_label (mymenu[i].text);
+			/* Always show checkbox, not just when it is moused over. */
+			gtk_check_menu_item_set_show_toggle ((GtkCheckMenuItem*)item, TRUE);
+
 			gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (item),
 													 mymenu[i].state);
 			if (mymenu[i].callback)
@@ -1891,6 +1891,8 @@ createmenus (void *app, struct session *sess, int bar)
 			gtk_menu_append (GTK_MENU (menu), item);
 			gtk_widget_show (item);
 			gtk_widget_set_sensitive (item, mymenu[i].activate);
+			if (bar && i == 31 - menuoffset)
+				sess->gui->awaymenuitem = item;
 			break;
 		case M_SEP:
 			item = gtk_menu_item_new ();
@@ -1903,7 +1905,8 @@ createmenus (void *app, struct session *sess, int bar)
 				gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
 			if (usermenu)
 				usermenu_create (usermenu);
-			sess->gui->usermenu = usermenu;
+			if (bar)
+				sess->gui->usermenu = usermenu;
 			return (menu_bar);
 		}
 		i++;
