@@ -39,26 +39,26 @@ extern GSList *sess_list;
 extern GtkStyle *channelwin_style;
 
 GdkColor colors[] = {
-	{0, 0, 0, 0},					  /* 0  black */
-	{0, 0xcccc, 0xcccc, 0xcccc}, /* 1  white */
-	{0, 0, 0, 0xcccc},			  /* 2  blue */
-	{0, 0, 0xcccc, 0},			  /* 3  green */
-	{0, 0xcccc, 0, 0},			  /* 4  red */
-	{0, 0xbbbb, 0xbbbb, 0},		  /* 5  yellow/brown */
-	{0, 0xbbbb, 0, 0xbbbb},		  /* 6  purple */
-	{0, 0xffff, 0xaaaa, 0},		  /* 7  orange */
-	{0, 0xffff, 0xffff, 0},		  /* 8  yellow */
-	{0, 0, 0xffff, 0},			  /* 9  green */
-	{0, 0, 0xcccc, 0xcccc},		  /* 10 aqua */
-	{0, 0, 0xffff, 0xffff},		  /* 11 light aqua */
-	{0, 0, 0, 0xffff},			  /* 12 blue */
-	{0, 0xffff, 0, 0xffff},		  /* 13 pink */
+	{0, 0x0000, 0x0000, 0x0000}, /* 0  black */
+	{0, 0xcf3c, 0xcf3c, 0xcf3c}, /* 1  white */
+	{0, 0x0000, 0x0000, 0xcccc}, /* 2  blue */
+	{0, 0x0000, 0xcccc, 0x0000}, /* 3  green */
+	{0, 0xcccc, 0x0000, 0x0000}, /* 4  red */
+	{0, 0xbbbb, 0xbbbb, 0x0000}, /* 5  yellow/brown */
+	{0, 0xbbbb, 0x0000, 0xbbbb}, /* 6  purple */
+	{0, 0xffff, 0xaaaa, 0x0000}, /* 7  orange */
+	{0, 0xeeee, 0xdddd, 0x2222}, /* 8  yellow */
+	{0, 0x3333, 0xdede, 0x5555}, /* 9  green */
+	{0, 0x0000, 0xcccc, 0xcccc}, /* 10 aqua */
+	{0, 0x3333, 0xdddd, 0xeeee}, /* 11 light aqua */
+	{0, 0x0000, 0x0000, 0xffff}, /* 12 blue */
+	{0, 0x9999, 0x2222, 0xeeee}, /* 13 light purple */
 	{0, 0x7777, 0x7777, 0x7777}, /* 14 grey */
 	{0, 0x9999, 0x9999, 0x9999}, /* 15 light grey */
-	{0, 0, 0, 0xcccc},			  /* 16 blue markBack */
-	{0, 0xeeee, 0xeeee, 0xeeee}, /* 17 white markFore */
-	{0, 0xcccc, 0xcccc, 0xcccc}, /* 18 foreground (white) */
-	{0, 0, 0, 0},					  /* 19 background (black) */
+	{0, 0xbe00, 0xbe00, 0xbe00}, /* 16 marktext Back (white) */
+	{0, 0x0000, 0x0000, 0x0000}, /* 17 marktext Fore (black) */
+	{0, 0xcf3c, 0xcf3c, 0xcf3c}, /* 18 foreground (white) */
+	{0, 0x0000, 0x0000, 0x0000}, /* 19 background (black) */
 };
 
 static GtkWidget *palwin = 0;
@@ -107,13 +107,13 @@ palette_load (void)
 
 			for (i = 0; i < 20; i++)
 			{
-				snprintf (prefname, sizeof prefname, "color_%d_red ", i);
+				snprintf (prefname, sizeof prefname, "color_%d_red", i);
 				red = cfg_get_int (cfg, prefname);
 
-				snprintf (prefname, sizeof prefname, "color_%d_grn ", i);
+				snprintf (prefname, sizeof prefname, "color_%d_grn", i);
 				green = cfg_get_int (cfg, prefname);
 
-				snprintf (prefname, sizeof prefname, "color_%d_blu ", i);
+				snprintf (prefname, sizeof prefname, "color_%d_blu", i);
 				blue = cfg_get_int_with_result (cfg, prefname, &res);
 
 				if (res)
@@ -141,13 +141,13 @@ palette_save (void)
 	{
 		for (i = 0; i < 20; i++)
 		{
-			snprintf (prefname, sizeof prefname, "color_%d_red ", i);
+			snprintf (prefname, sizeof prefname, "color_%d_red", i);
 			cfg_put_int (fh, colors[i].red, prefname);
 
-			snprintf (prefname, sizeof prefname, "color_%d_grn ", i);
+			snprintf (prefname, sizeof prefname, "color_%d_grn", i);
 			cfg_put_int (fh, colors[i].green, prefname);
 
-			snprintf (prefname, sizeof prefname, "color_%d_blu ", i);
+			snprintf (prefname, sizeof prefname, "color_%d_blu", i);
 			cfg_put_int (fh, colors[i].blue, prefname);
 		}
 		close (fh);
