@@ -26,8 +26,9 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #endif
 
-#include <gdk/gdkprivate.h>
-
+#ifdef USE_XLIB
+#include <gdk/gdkx.h>
+#endif
 
 #include "../pixmaps/globe.xpm"
 #include "../pixmaps/server.xpm"
@@ -58,7 +59,11 @@ GdkBitmap *mask_xchat;
 static void
 pixmap_load_from_data (GdkPixmap **pix, GdkBitmap **mask, char **xpm_data)
 {
-	*pix = gdk_pixmap_create_from_xpm_d ((GdkWindow*)&gdk_root_parent, mask, NULL, xpm_data);
+#ifdef USE_XLIB
+	*pix = gdk_pixmap_create_from_xpm_d (GDK_ROOT_PARENT(), mask, NULL, xpm_data);
+#else
+	*pix = gdk_pixmap_create_from_xpm_d (NULL, mask, NULL, xpm_data);
+#endif
 }
 
 GdkPixmap *
@@ -77,7 +82,12 @@ pixmap_load_from_file (char *file)
 	return pixmap;
 #else
 
-	return gdk_pixmap_create_from_xpm ((GdkWindow*)&gdk_root_parent, NULL, NULL, file);
+#ifdef USE_XLIB
+	return gdk_pixmap_create_from_xpm (GDK_ROOT_PARENT(), NULL, NULL, file);
+#else
+	return gdk_pixmap_create_from_xpm (NULL, NULL, NULL, file);
+#endif
+
 #endif
 }
 
