@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <pwd.h>
 #include <fcntl.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
@@ -46,6 +47,7 @@ static GtkWidget *quickview;
 static GtkWidget *entry_port;
 static GtkWidget *entry_comment;
 static GtkWidget *entry_password;
+static GtkWidget *entry_nick;
 static GtkWidget *entry_autoconnect;
 static GtkWidget *entry_use_ssl;
 static GtkWidget *entry_accept_invalid_cert;
@@ -119,6 +121,7 @@ add_server_entry (struct slentry *serv)
 		strcpy (new->server, serv->server);
 		new->channel[0] = 0;
 		new->password[0] = 0;
+		new->nick[0] = 0;
 		new->flags = 0;
 		new->port = 0;
 		if (serv->port)
@@ -149,6 +152,7 @@ add_server_entry (struct slentry *serv)
 		strcpy (new->channel, serv->channel);
 		strcpy (new->server, serv->server);
 		strcpy (new->password, serv->password);
+		strcpy (new->nick, serv->nick);
 		text[0] = new->comment;
 		node = gtk_ctree_insert_node (GTK_CTREE (sltree), parent,
 												NULL, text, 4, pix_server, mask_server,
@@ -172,6 +176,7 @@ add_defaults (void)
 	int i = 0;
 
 	serv.password[0] = 0;
+	serv.nick[0] = 0;
 	while (dserv[i].server != 0)
 	{
 		strcpy (serv.channel, dserv[i].channel);
@@ -198,21 +203,23 @@ read_next_server_entry (char *my_cfg, struct slentry *serv)
 	else
 		return 0; /* EOF */
 
-	if ((my_cfg_tmp = cfg_get_str (my_cfg, "port ", serv->channel)))
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "port", serv->channel)))
 	{
 		my_cfg = my_cfg_tmp;
 		serv->port = atoi (serv->channel);
 	}
-	if ((my_cfg_tmp = cfg_get_str (my_cfg, "flags ", serv->channel)))
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "flags", serv->channel)))
 	{
 		my_cfg = my_cfg_tmp;
 		serv->flags = atoi (serv->channel);
 	}
-	if ((my_cfg_tmp = cfg_get_str (my_cfg, "channel ", serv->channel)))
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "channel", serv->channel)))
 		my_cfg = my_cfg_tmp;
-	if ((my_cfg_tmp = cfg_get_str (my_cfg, "password ", serv->password)))
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "password", serv->password)))
 		my_cfg = my_cfg_tmp;
-	if ((my_cfg_tmp = cfg_get_str (my_cfg, "comment ", serv->comment)))
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "nick", serv->nick)))
+		my_cfg = my_cfg_tmp;
+	if ((my_cfg_tmp = cfg_get_str (my_cfg, "comment", serv->comment)))
 		my_cfg = my_cfg_tmp;
 
 	return my_cfg;
@@ -248,16 +255,18 @@ load_serverentrys (int select_row)
 				cfg[i] = '\0';
 			do
 			{
+				serv.nick[0] = 0;
 				my_cfg = read_next_server_entry (my_cfg, &serv);
 				if (my_cfg)
 					add_server_entry (&serv);
 			}
 			while (my_cfg);
 			free (cfg);
-			while (g_slist_last (server_parents))
+			while (server_parents)
+			{
 				server_parents = g_slist_remove (server_parents,
-															((g_slist_last
-															  (server_parents))->data));
+															server_parents->data);
+			}
 		}
 		close (fh);
 	} else
@@ -284,9 +293,10 @@ write_serverentry (struct slentry *entry, int fh)
 				 "flags = %d\n"
 				 "channel = %s\n"
 				 "password = %s\n"
+				 "nick = %s\n"
 				 "comment = %s\n\n",
 				 entry->server, entry->port, entry->flags,
-				 entry->channel, entry->password, entry->comment);
+				 entry->channel, entry->password, entry->nick, entry->comment);
 	write (fh, buf, strlen (buf));
 }
 
@@ -319,6 +329,7 @@ parse_serverentrys (GtkCTree * ctree, GtkCTreeNode * node, int *fh)
 		buf->flags = 0;
 		*(buf->channel) = 0;
 		*(buf->password) = 0;
+		*(buf->nick) = 0;
 		*(buf->comment) = 0;
 		strcpy (buf->server, "ENDSUB");
 		write_serverentry (buf, *fh);
@@ -328,6 +339,7 @@ parse_serverentrys (GtkCTree * ctree, GtkCTreeNode * node, int *fh)
 	strcpy (buf->channel, temporary->channel);
 	strcpy (buf->server, temporary->server);
 	strcpy (buf->password, temporary->password);
+	strcpy (buf->nick, temporary->nick);
 	strcpy (buf->comment, temporary->comment);
 	buf->port = temporary->port;
 	buf->flags = temporary->flags;
@@ -341,6 +353,7 @@ parse_serverentrys (GtkCTree * ctree, GtkCTreeNode * node, int *fh)
 		buf->flags = 0;
 		*(buf->channel) = 0;
 		*(buf->password) = 0;
+		*(buf->nick) = 0;
 		*(buf->comment) = 0;
 		strcpy (buf->server, "ENDSUB");
 		write_serverentry (buf, *fh);
@@ -398,12 +411,10 @@ reload_servers (void)
 		while (gtk_ctree_node_nth (GTK_CTREE (sltree), 0))
 			gtk_ctree_remove_node (GTK_CTREE (sltree),
 										  gtk_ctree_node_nth (GTK_CTREE (sltree), 0));
-		while (g_slist_last (server_malloc_list))
+		while (server_malloc_list)
 		{
-			free ((g_slist_last (server_malloc_list))->data);
-			server_malloc_list = g_slist_remove (server_malloc_list,
-															 ((g_slist_last
-																(server_malloc_list))->data));
+			free (server_malloc_list->data);
+			server_malloc_list = g_slist_remove (server_malloc_list, server_malloc_list->data);
 		}
 		load_serverentrys (focus_row);
 		gtk_clist_thaw (GTK_CLIST (sltree));
@@ -473,6 +484,8 @@ done_edit_entry (void)
 				  gtk_entry_get_text (GTK_ENTRY (entry_channel)));
 		strcpy (editentry->password,
 				  gtk_entry_get_text (GTK_ENTRY (entry_password)));
+		strcpy (editentry->nick,
+				  gtk_entry_get_text (GTK_ENTRY (entry_nick)));
 		editentry->port = atoi (gtk_entry_get_text (GTK_ENTRY (entry_port)));
 
 		if (!gtk_toggle_button_get_active
@@ -498,7 +511,9 @@ done_edit_entry (void)
 	close_edit_entry ();
 }
 
-void
+#ifdef USE_OPENSSL
+
+static void
 entry_use_ssl_toggled (GtkWidget * w, gpointer entry_accept_invalid_cert)
 {
 	gtk_widget_set_sensitive (GTK_WIDGET (entry_accept_invalid_cert),
@@ -507,6 +522,8 @@ entry_use_ssl_toggled (GtkWidget * w, gpointer entry_accept_invalid_cert)
 									  FALSE);
 }
 
+#endif
+
 /*
  * edit_entry_clicked:
  * called when the users clicks on the "edit" button in the serverlist GUI
@@ -596,6 +613,11 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 		gtk_entry_set_text (GTK_ENTRY (entry_channel), editentry->channel);
 		gtk_widget_show (entry_channel);
 
+		gtkutil_label_new ("Nickname:", box1);
+		entry_nick = gtkutil_entry_new (201, box2, NULL, NULL);
+		gtk_entry_set_text (GTK_ENTRY (entry_nick), editentry->nick);
+		gtk_widget_show (entry_nick);
+
 		box1 = gtk_hbox_new (FALSE, 0);
 		entry_autoconnect = gtk_check_button_new_with_label ("Autoconnect");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry_autoconnect),
@@ -669,6 +691,7 @@ new_server_clicked (GtkWidget * wid, struct session *sess)
 	new->channel[0] = 0;
 	new->server[0] = 0;
 	new->password[0] = 0;
+	new->nick[0] = 0;
 	text[0] = new->comment;
 
 	if (!parent)
@@ -721,6 +744,7 @@ new_group_clicked (GtkWidget * wid, struct session *sess)
 	new->channel[0] = 0;
 	strcpy (new->server, "SUB");
 	new->password[0] = 0;
+	new->nick[0] = 0;
 	text[0] = new->comment;
 
 	if (!sibling)
@@ -811,12 +835,10 @@ close_server_list (GtkWidget * win, int destroy)
 			gtk_widget_destroy (sleditwin);
 		slwin = 0;
 		sleditwin = 0;
-		while (g_slist_last (server_malloc_list))
+		while (server_malloc_list)
 		{
-			free ((g_slist_last (server_malloc_list))->data);
-			server_malloc_list = g_slist_remove (server_malloc_list,
-															 ((g_slist_last
-																(server_malloc_list))->data));
+			free (server_malloc_list->data);
+			server_malloc_list = g_slist_remove (server_malloc_list, server_malloc_list->data);
 		}
 		sltree = 0;
 	}
@@ -834,11 +856,13 @@ connect_auto (struct slentry *conn, struct session *sess)
 	char channel[CHANLEN];
 	char server[132];
 	char password[86];
+	char nick[NICKLEN];
 
 	if (!(strlen (conn->server)))
 		return;
 	safe_strcpy (server, conn->server, sizeof (server));
 	safe_strcpy (password, conn->password, sizeof (password));
+	safe_strcpy (nick, conn->nick, NICKLEN);
 	safe_strcpy (channel, conn->channel, CHANLEN);
 	port = conn->port;
 	if (!sess)
@@ -851,6 +875,10 @@ connect_auto (struct slentry *conn, struct session *sess)
 	}
 	strcpy (sess->willjoinchannel, channel);
 	strcpy (sess->server->password, password);
+	if (nick[0] == 0)
+		strcpy (sess->server->nick, prefs.nick1);
+	else
+		strcpy (sess->server->nick, nick);
 #ifdef USE_OPENSSL
 	sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
 	sess->server->accept_invalid_cert =
@@ -913,6 +941,7 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 	char channel[CHANLEN];
 	char server[132];
 	char password[86];
+	char nick[NICKLEN];
 	int port;
 
 	selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
@@ -925,6 +954,7 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 		return;
 	safe_strcpy (server, conn->server, sizeof (server));
 	safe_strcpy (password, conn->password, sizeof (password));
+	safe_strcpy (nick, conn->nick, NICKLEN);
 	safe_strcpy (channel, conn->channel, CHANLEN);
 	port = conn->port;
 	serv = new_server ();
@@ -939,7 +969,10 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 		{
 			strcpy (sess->server->password, password);
 			strcpy (sess->willjoinchannel, channel);
-			strcpy (sess->server->nick, prefs.nick1);
+			if (nick[0] == 0)
+				strcpy (sess->server->nick, prefs.nick1);
+			else
+				strcpy (sess->server->nick, nick);
 #ifdef USE_OPENSSL
 			sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
 			sess->server->accept_invalid_cert =
@@ -961,8 +994,17 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 	GtkCTreeNode *selected;
 	struct slentry *conn;
 	char server[132];
+	char nick[NICKLEN];
 	int port;
 
+	if (sess->is_shell)
+	{
+		gtkutil_simpledialog ("You want to connect in a shell-tab eh? "
+										"Well I don't think so!\n\n"
+										"Try the CONNECT NEW button instead.");
+		return;
+	}
+
 	/* clueless user filter! */
 	if (!strcmp ("root", gtk_entry_get_text ((GtkEntry *) nick1gad)))
 	{
@@ -976,20 +1018,32 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 		return;
 
 	conn = gtk_ctree_node_get_row_data (GTK_CTREE (sltree), selected);
-	if (!(strlen (conn->server)))
+	if (conn->server[0] == 0)
 		return;
 	safe_strcpy (server, conn->server, sizeof (server));
+	safe_strcpy (nick, conn->nick, NICKLEN);
 	safe_strcpy (sess->server->password, conn->password, 85);
 	safe_strcpy (sess->willjoinchannel, conn->channel, CHANLEN);
 	port = conn->port;
 
 	close_server_list (0, TRUE);
 
-	if (strcmp (sess->server->nick, prefs.nick1))
+	if (nick[0] == 0)
 	{
-		fe_set_nick (sess->server, prefs.nick1);
-		strcpy (sess->server->nick, prefs.nick1);
+		if (strcmp (sess->server->nick, prefs.nick1))
+		{
+			fe_set_nick (sess->server, prefs.nick1);
+			strcpy (sess->server->nick, prefs.nick1);
+		}
+	} else
+	{
+		if (strcmp (sess->server->nick, nick))
+		{
+			fe_set_nick (sess->server, nick);
+			strcpy (sess->server->nick, nick);
+		}
 	}
+
 #ifdef USE_OPENSSL
 	sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
 	sess->server->accept_invalid_cert =
@@ -1153,11 +1207,10 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	gtk_widget_show (scroll);
 	sltree = gtk_ctree_new (1, 0);
 	gtk_clist_freeze (GTK_CLIST (sltree));
-	gtk_ctree_set_line_style (GTK_CTREE (sltree), GTK_CTREE_LINES_DOTTED);
+	gtk_ctree_set_line_style (GTK_CTREE (sltree), GTK_CTREE_LINES_SOLID);
+	gtk_ctree_set_expander_style (GTK_CTREE (sltree), GTK_CTREE_EXPANDER_SQUARE);
 	gtk_clist_set_reorderable (GTK_CLIST (sltree), TRUE);
 	gtk_clist_set_selection_mode (GTK_CLIST (sltree), GTK_SELECTION_BROWSE);
-	gtk_ctree_set_expander_style (GTK_CTREE (sltree),
-											GTK_CTREE_EXPANDER_CIRCULAR);
 	gtk_widget_show (sltree);
 	gtk_signal_connect (GTK_OBJECT (GTK_CLIST (sltree)), "select_row",
 							  GTK_SIGNAL_FUNC (row_selected), sess);
