@@ -57,6 +57,63 @@ static GtkWidget *fontdialog = 0;
 
 /* Generic interface creation functions */
 
+static void
+settings_create_optmenu (GtkWidget *table, char *label, char **text,
+								void *callback, void *userdata, int row_index,
+								int history)
+{
+	GtkWidget *wid, *menu, *align;
+	int i;
+
+	align = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
+	gtk_table_attach (GTK_TABLE (table), align, 0, 1, row_index, row_index + 1,
+							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
+	gtk_widget_show (align);
+
+	wid = gtk_label_new (label);
+	gtk_container_add (GTK_CONTAINER (align), wid);
+	gtk_widget_show (wid);
+
+	wid = gtk_option_menu_new ();
+	menu = gtk_menu_new ();
+
+	i = 0;
+	while (text[i])
+	{
+		menu_quick_item_with_callback (callback, text[i], menu, (void *) i);
+		i++;
+	}
+
+	gtk_object_set_user_data (GTK_OBJECT (menu), userdata);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (wid), menu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (wid), history);
+	gtk_widget_show (menu);
+	align = gtk_alignment_new (0.0, 1.0, 0.0, 0.0);
+	gtk_table_attach_defaults (GTK_TABLE (table), align, 2, 3, row_index,
+										row_index + 1);
+	gtk_widget_show (align);
+	gtk_container_add (GTK_CONTAINER (align), wid);
+	gtk_widget_show (wid);
+}
+
+static void
+settings_create_tworows (GtkWidget *box, GtkWidget **left_box, GtkWidget **right_box)
+{
+	GtkWidget *ac_box;
+
+	ac_box = gtk_hbox_new (0, 0);
+	gtk_container_add (GTK_CONTAINER (box), ac_box);
+	gtk_widget_show (ac_box);
+
+	*left_box = gtk_vbox_new (0, 0);
+	gtk_container_add (GTK_CONTAINER (ac_box), *left_box);
+	gtk_widget_show (*left_box);
+
+	*right_box = gtk_vbox_new (0, 0);
+	gtk_container_add (GTK_CONTAINER (ac_box), *right_box);
+	gtk_widget_show (*right_box);
+}
+
 static GtkWidget *
 settings_create_group (GtkWidget * vvbox, gchar * title)
 {
@@ -120,7 +177,8 @@ settings_create_entry (char *text, int maxlen, GtkWidget * table,
 
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
-	gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 0);
+	gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (wid), 2, 3, row,
+							row + 1, GTK_FILL, GTK_FILL, 0, 0);
 	gtk_widget_show (hbox);
 
 	if (suffixcallback || suffix)
@@ -137,12 +195,56 @@ settings_create_entry (char *text, int maxlen, GtkWidget * table,
 			swid = gtk_label_new (suffix);
 		}
 		gtk_widget_show (swid);
-		gtk_box_pack_start (GTK_BOX (hbox), swid, FALSE, FALSE, 0);
+		gtk_table_attach (GTK_TABLE (table), swid, 3, 4, row,
+							row + 1, GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
 	}
 	gtk_container_add (GTK_CONTAINER (align), hbox);
 	return wid;
 }
 
+static GtkWidget *
+settings_create_numberentry (char *text, int min, int max, int climb,
+										GtkWidget *table, int defval, int row, char *suffix)
+{
+	GtkWidget *label, *wid, *rbox, *align;
+	GtkAdjustment *adj;
+
+	label = gtk_label_new (text);
+	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (label), 0, 1, row,
+							row + 1, GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0,
+							0);
+	gtk_widget_show (label);
+
+	align = gtk_alignment_new (0.0, 1.0, 0.0, 0.0);
+	gtk_table_attach_defaults (GTK_TABLE (table), align, 2, 3, row, row + 1);
+	gtk_widget_show (align);
+
+	rbox = gtk_hbox_new (0, 0);
+	gtk_container_add (GTK_CONTAINER (align), rbox);
+	gtk_widget_show (rbox);
+
+	wid = gtk_spin_button_new (0, climb, 0);
+	adj = gtk_spin_button_get_adjustment ((GtkSpinButton*)wid);
+	adj->lower = min;
+	adj->upper = max;
+	adj->step_increment = climb;
+	gtk_adjustment_changed (adj);
+	gtk_spin_button_set_value ((GtkSpinButton*)wid, defval);
+	gtk_widget_set_usize (wid, 60, 0);
+	gtk_box_pack_start (GTK_BOX (rbox), wid, 0, 0, 0);
+	gtk_widget_show (wid);
+
+	if (suffix)
+	{
+		label = gtk_label_new (suffix);
+		gtk_box_pack_start (GTK_BOX (rbox), label, 0, 0, 0);
+		gtk_widget_show (label);
+	}
+
+	return wid;
+}
+
 static GtkWidget *
 settings_create_toggle (char *label, GtkWidget * box, int state,
 								void *callback, void *sess)
@@ -324,6 +426,9 @@ settings_openfontdialog (GtkWidget * button, GtkWidget * entry)
 {
 	GtkWidget *dialog;
 
+	if (fontdialog)
+		gtk_widget_destroy (fontdialog);
+
 	dialog = gtk_font_selection_dialog_new (_("Select Font"));
 	gtk_signal_connect
 		(GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (dialog)->ok_button), "clicked",
@@ -346,6 +451,9 @@ expand_homedir_inplace (char *file)
 	free (new_file);
 }
 
+#define GETINT(x) gtk_spin_button_get_value_as_int((GtkSpinButton*)sess->setup->x)
+#define GETSTR(x,y) strcpy(x,gtk_entry_get_text((GtkEntry*)sess->setup->y))
+
 static void
 settings_ok_clicked (GtkWidget * wid, struct session *sess)
 {
@@ -356,84 +464,54 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	GdkPixmap *old_pix;
 	GSList *list;
 
-	strcpy (sess->setup->prefs.hostname,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_hostname)));
+	/* get all the spinbutton and GtkEntry values into our structure */
 
-	strcpy (sess->setup->prefs.proxy_host,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_proxy_host)));
-	sess->setup->prefs.proxy_port = atoi (
-			gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_proxy_port)));
+	GETSTR (sess->setup->prefs.hostname, entry_hostname);
+	GETSTR (sess->setup->prefs.proxy_host, entry_proxy_host);
+	sess->setup->prefs.proxy_port = GETINT (entry_proxy_port);
 
-	sess->setup->prefs.first_dcc_send_port = atoi (
-			gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_dcc_send_port_first)));
-	sess->setup->prefs.last_dcc_send_port = atoi (
-			gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_dcc_send_port_last)));
+	sess->setup->prefs.first_dcc_send_port = GETINT (entry_dcc_send_port_first);
+	sess->setup->prefs.last_dcc_send_port = GETINT (entry_dcc_send_port_last);
 
-	strcpy (sess->setup->prefs.background,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->background)));
+	GETSTR (sess->setup->prefs.background, background);
 	expand_homedir_inplace (sess->setup->prefs.background);
-	strcpy (sess->setup->prefs.background_dialog,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->background_dialog)));
+	GETSTR (sess->setup->prefs.background_dialog, background_dialog);
 	expand_homedir_inplace (sess->setup->prefs.background_dialog);
-	strcpy (sess->setup->prefs.soundcmd,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_soundcmd)));
-	strcpy (sess->setup->prefs.sounddir,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_sounddir)));
+	GETSTR (sess->setup->prefs.soundcmd, entry_soundcmd);
+	GETSTR (sess->setup->prefs.sounddir, entry_sounddir);
 
-	strcpy (sess->setup->prefs.dnsprogram,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->entry_dnsprogram));
-	strcpy (sess->setup->prefs.bluestring,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->entry_bluestring));
-	strcpy (sess->setup->prefs.doubleclickuser,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->
-										 entry_doubleclickuser));
-	strcpy (sess->setup->prefs.awayreason,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->entry_away));
-	strcpy (sess->setup->prefs.quitreason,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->entry_quit));
-	strcpy (sess->setup->prefs.logmask,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->logmask_entry));
-	strcpy (sess->setup->prefs.nick_suffix,
-			  gtk_entry_get_text ((GtkEntry *) sess->setup->entry_nick_suffix));
-	sess->setup->prefs.max_lines =
-		atoi (gtk_entry_get_text ((GtkEntry *) sess->setup->entry_max_lines));
-	sess->setup->prefs.notify_timeout =
-		atol (gtk_entry_get_text ((GtkEntry *) sess->setup->entry_timeout));
-	sess->setup->prefs.recon_delay =
-		atoi (gtk_entry_get_text ((GtkEntry *) sess->setup->entry_recon_delay));
+	GETSTR (sess->setup->prefs.dnsprogram, entry_dnsprogram);
+	GETSTR (sess->setup->prefs.bluestring, entry_bluestring);
+	GETSTR (sess->setup->prefs.doubleclickuser, entry_doubleclickuser);
+	GETSTR (sess->setup->prefs.awayreason, entry_away);
+	GETSTR (sess->setup->prefs.quitreason, entry_quit);
+	GETSTR (sess->setup->prefs.partreason, entry_part);
+	GETSTR (sess->setup->prefs.logmask, logmask_entry);
+	GETSTR (sess->setup->prefs.nick_suffix, entry_nick_suffix);
+
+	sess->setup->prefs.max_lines = GETINT (entry_max_lines);
+	sess->setup->prefs.notify_timeout = GETINT (entry_timeout);
+	sess->setup->prefs.recon_delay = GETINT (entry_recon_delay);
 
 	sscanf (gtk_entry_get_text ((GtkEntry *) sess->setup->entry_permissions),
 			  "%o", &sess->setup->prefs.dccpermissions);
 
-	sess->setup->prefs.dcc_blocksize =
-		atoi (gtk_entry_get_text
-				((GtkEntry *) sess->setup->entry_dcc_blocksize));
-	sess->setup->prefs.dccstalltimeout =
-		atoi (gtk_entry_get_text
-				((GtkEntry *) sess->setup->entry_dccstalltimeout));
-	sess->setup->prefs.dcctimeout =
-		atoi (gtk_entry_get_text ((GtkEntry *) sess->setup->entry_dcctimeout));
-	strcpy (sess->setup->prefs.dccdir,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_dccdir)));
+	sess->setup->prefs.dcc_blocksize = GETINT (entry_dcc_blocksize);
+	sess->setup->prefs.dccstalltimeout = GETINT (entry_dccstalltimeout);
+	sess->setup->prefs.dcctimeout = GETINT (entry_dcctimeout);
+	GETSTR (sess->setup->prefs.dccdir, entry_dccdir);
 	expand_homedir_inplace (sess->setup->prefs.dccdir);
 
-	strcpy (sess->setup->prefs.font_normal,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->font_normal)));
-	strcpy (sess->setup->prefs.dialog_font_normal,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->dialog_font_normal)));
+	GETSTR (sess->setup->prefs.font_normal, font_normal);
+	GETSTR (sess->setup->prefs.dialog_font_normal, dialog_font_normal);
 
-	sess->setup->prefs.mainwindow_left =
-		atoi (gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_mainw_left)));
-	sess->setup->prefs.mainwindow_top =
-		atoi (gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_mainw_top)));
-	sess->setup->prefs.mainwindow_width =
-		atoi (gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_mainw_width)));
-	sess->setup->prefs.mainwindow_height =
-		atoi (gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_mainw_height)));
+	sess->setup->prefs.mainwindow_left = GETINT (entry_mainw_left);
+	sess->setup->prefs.mainwindow_top = GETINT (entry_mainw_top);
+	sess->setup->prefs.mainwindow_width = GETINT (entry_mainw_width);
+	sess->setup->prefs.mainwindow_height = GETINT (entry_mainw_height);
 
 #ifdef USE_TRANS
-	strcpy (sess->setup->prefs.trans_file,
-			  gtk_entry_get_text (GTK_ENTRY (sess->setup->entry_trans_file)));
+	GETSTR (sess->setup->prefs.trans_file, entry_trans_file);
 	if (sess->setup->prefs.use_trans)
 	{
 		if (load_trans_table (sess->setup->prefs.trans_file) == 0)
@@ -467,13 +545,14 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 			my_font_load (sess->setup->prefs.dialog_font_normal);
 		fontdialogchange = TRUE;
 	}
+
 	if (prefs.tabchannels != sess->setup->prefs.tabchannels)
 		noapply = TRUE;
 
-	if (prefs.nopaned != sess->setup->prefs.nopaned)
+	if (prefs.paned_userlist != sess->setup->prefs.paned_userlist)
 		noapply = TRUE;
 
-	if (prefs.nochanmodebuttons != sess->setup->prefs.nochanmodebuttons)
+	if (prefs.chanmodebuttons != sess->setup->prefs.chanmodebuttons)
 		noapply = TRUE;
 
 	if (prefs.treeview != sess->setup->prefs.treeview)
@@ -640,11 +719,11 @@ settings_ptoggle_check (GtkWidget * widget, int *pref)
 	*pref = (int) (GTK_TOGGLE_BUTTON (widget)->active);
 }
 
-static void
+/*static void
 settings_pinvtoggle_check (GtkWidget * widget, int *pref)
 {
 	*pref = ((int) (GTK_TOGGLE_BUTTON (widget)->active) ? 0 : 1);
-}
+}*/
 
 /* Misc callbacks */
 
@@ -695,13 +774,6 @@ settings_nu_color_clicked (GtkWidget * igad, struct session *sess)
 									&sess->setup->prefs.nu_color);
 }
 
-static void
-settings_bt_color_clicked (GtkWidget * igad, struct session *sess)
-{
-	settings_color_clicked (igad, sess->setup->bt_color,
-									&sess->setup->prefs.bt_color);
-}
-
 static void
 settings_slider_cb (GtkAdjustment * adj, int *value)
 {
@@ -776,15 +848,13 @@ settings_create_window_page (GtkWidget * vvbox,
 									  int *red, int *green, int *blue,
 									  int *autoindent, struct session *sess)
 {
-	GtkWidget *vbox, *table;
+	GtkWidget *vbox, *table, *wid, *left_box, *right_box;
 	gint row_index = 0;
-	GtkWidget *wid;
 
 	vbox = settings_create_group (vvbox, _("Appearance"));
 	table = settings_create_table (vbox);
-	settings_create_toggle (_("Use gdk_fontset_load instead of gdk_font_load"),
-									vbox, prefs.use_fontset, settings_ptoggle_check,
-									&(sess->setup->prefs.use_fontset));
+	settings_create_tworows (vbox, &left_box, &right_box);
+
 	settings_create_entry (_("Font:"), FONTNAMELEN, table, font_normal_str,
 								  font_normal_wid, 0, settings_openfontdialog,
 								  row_index++);
@@ -800,30 +870,33 @@ settings_create_window_page (GtkWidget * vvbox,
 	if (transparent)
 		gtk_widget_set_sensitive (*backgroundwid, FALSE);
 
-	wid = settings_create_toggle (_("Indent Nicks"), vbox, *indent_nicks,
+	wid = settings_create_toggle (_("Indent Nicks"), left_box, *indent_nicks,
 											settings_ptoggle_check, indent_nicks);
 	add_tip (wid, _("Make nicknames right justified."));
 
-	wid = settings_create_toggle (_("Auto Indent"), vbox, *autoindent,
+	wid = settings_create_toggle (_("Auto Indent"), left_box, *autoindent,
 											settings_ptoggle_check, autoindent);
 	add_tip (wid, _("Auto adjust the separator bar position as needed."));
 
-	wid = settings_create_toggle (_("Draw Separator Bar"), vbox, *separator,
+	wid = settings_create_toggle (_("Draw Separator Bar"), left_box, *separator,
 											settings_ptoggle_check, separator);
 	add_tip (wid, _("Make the separator an actual visible line."));
 
-	wid = settings_create_toggle (_("Word Wrap"), vbox, *wordwrap,
+	wid = settings_create_toggle (_("Word Wrap"), right_box, *wordwrap,
 											settings_ptoggle_check, wordwrap);
 	add_tip (wid, _("Don't split words from one line to the next"));
 
-	*transwid = settings_create_toggle (_("Transparent Background"), vbox,
+	*transwid = settings_create_toggle (_("Transparent Background"), right_box,
 													transparent, transparent_callback,
 													sess);
 	add_tip (*transwid, _("Make the text box seem see-through"));
 
-	*tintwid = settings_create_toggle (_("Tint Transparency"), vbox, tint,
+	*tintwid = settings_create_toggle (_("Tint Transparency"), right_box, tint,
 												  settings_ptoggle_check, tint_pref);
 	add_tip (*tintwid, _("Tint the see-through text box to make it darker"));
+	settings_create_toggle (_("Use gdk_fontset_load"),
+									left_box, prefs.use_fontset, settings_ptoggle_check,
+									&(sess->setup->prefs.use_fontset));
 
 #ifdef USE_GDK_PIXBUF
 	if (!transparent)
@@ -864,9 +937,16 @@ settings_page_interface (struct session *sess, GtkWidget * vbox)
 	GtkWidget *wid;
 	GtkWidget *tab;
 	GtkWidget *tog;
-	GtkWidget *menu;
-	GtkWidget *align;
 	gint row_index;
+	static char *ulmenutext[] = 
+	{
+		N_("A-Z, Ops first"),
+		N_("A-Z"),
+		N_("Z-A, Ops last"),
+		N_("Z-A"),
+		N_("Unsorted"),
+		NULL
+	};
 
 	wid = settings_create_group (vbox, _("Startup and Shutdown"));
 	tog = settings_create_toggle (_("No Server List On Startup"), wid,
@@ -882,6 +962,15 @@ settings_page_interface (struct session *sess, GtkWidget * vbox)
 	tog = settings_create_toggle (_ ("Give the User List style"), wid,
                                   prefs.style_namelistgad, settings_ptoggle_check,
                                   &(sess->setup->prefs.style_namelistgad));
+	tog = settings_create_toggle (_("Show userhost"), wid,
+											prefs.showhostname_in_userlist,
+											settings_ptoggle_check,
+											&(sess->setup->prefs.showhostname_in_userlist));
+	add_tip (tog, _("Show userhost of users in channel userlist"));
+	tog = settings_create_toggle (_("Enable Paned Userlist"), wid,
+											prefs.paned_userlist, settings_ptoggle_check,
+											&(sess->setup->prefs.paned_userlist));
+	add_tip (tog, _("Use paned user list instead of a fixed width one"));
 	tab = settings_create_table (wid);
 	row_index = 0;
 	settings_create_entry (_("Double Click Command:"),
@@ -889,45 +978,8 @@ settings_page_interface (struct session *sess, GtkWidget * vbox)
 								  prefs.doubleclickuser,
 								  &sess->setup->entry_doubleclickuser, 0, 0,
 								  row_index++);
-
-	wid = gtk_label_new (_("Userlist sorted by: "));
-	gtk_table_attach (GTK_TABLE (tab), wid, 0, 1, row_index, row_index + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-	gtk_widget_show (wid);
-
-	wid = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	menu_quick_item_with_callback (settings_sortset, _("A-Z, Ops first"), menu,
-											 (void *) 0);
-	menu_quick_item_with_callback (settings_sortset, _("A-Z"), menu,
-											 (void *) 1);
-	menu_quick_item_with_callback (settings_sortset, _("Z-A, Ops last"), menu,
-											 (void *) 2);
-	menu_quick_item_with_callback (settings_sortset, _("Z-A"), menu,
-											 (void *) 3);
-	menu_quick_item_with_callback (settings_sortset, _("Unsorted"), menu,
-											 (void *) 4);
-	gtk_object_set_user_data (GTK_OBJECT (menu), sess);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (wid), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (wid), prefs.userlist_sort);
-	gtk_widget_show (menu);
-	align = gtk_alignment_new (0.0, 1.0, 0.0, 0.0);
-	gtk_table_attach_defaults (GTK_TABLE (tab), align, 2, 3, row_index,
-										row_index + 1);
-	gtk_widget_show (align);
-	gtk_container_add (GTK_CONTAINER (align), wid);
-	gtk_widget_show (wid);
-
-	row_index++;
-	tog = settings_create_toggle (_("Show userhost"), 0,
-											prefs.showhostname_in_userlist,
-											settings_ptoggle_check,
-											&(sess->setup->
-											  prefs.showhostname_in_userlist));
-	add_tip (tog, _("Show userhost of users in channel userlist"));
-	gtk_table_attach (GTK_TABLE (tab), tog, 0, 1, row_index, row_index + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-	gtk_widget_show (tog);
+	settings_create_optmenu (tab, _("Userlist sorted by:"), ulmenutext,
+								settings_sortset, sess, row_index, prefs.userlist_sort);
 }
 
 static void
@@ -937,75 +989,75 @@ settings_page_interface_inout (struct session *sess, GtkWidget * vbox)
 	GtkWidget *tab;
 	GtkWidget *tog;
 	GtkWidget *entry;
-	gint row_index;
-	char buf[127];
+	GtkWidget *left_box, *right_box;
 
 	wid = settings_create_group (vbox, _("Input Box"));
-	tog = settings_create_toggle (_("Nick Name Completion"), wid,
+
+	settings_create_tworows (wid, &left_box, &right_box);
+
+	tog = settings_create_toggle (_("Nick Name Completion"), left_box,
 											prefs.nickcompletion, settings_ptoggle_check,
 											&(sess->setup->prefs.nickcompletion));
 	add_tip (tog, _("Complete nicknames when a partial one is entered"));
-	tog = settings_create_toggle (_("Old-style Nickname Completion"), wid,
+	tog = settings_create_toggle (_("Old-style Nickname Completion"), left_box,
 											prefs.old_nickcompletion,
 											settings_ptoggle_check,
 											&(sess->setup->prefs.old_nickcompletion));
 	add_tip (tog,
 				_("Nickname completion is old-style (instead of GNU-style)"));
 	tog =
-		settings_create_toggle (_("Give the Input Box style"), wid,
+		settings_create_toggle (_("Give the Input Box style"), left_box,
 										prefs.style_inputbox, settings_ptoggle_check,
 										&(sess->setup->prefs.style_inputbox));
 	add_tip (tog, _("Input box gets same style as main text area"));
-	tab = settings_create_table (wid);
-	entry = settings_create_entry (_("Nickname Completion Character:"), 1, tab,
-											 prefs.nick_suffix,
-											 &sess->setup->entry_nick_suffix,
-											 _("(Default is \":\" if left blank.)"), 0,
-											 0);
-	add_tip (entry, _("Character to append to completed nicknames"));
 	tog =
-		settings_create_toggle (_("InputBox always in focus"), wid,
+		settings_create_toggle (_("Input Box always in focus"), right_box,
 										prefs.inputgad_superfocus,
 										settings_ptoggle_check,
 										&(sess->setup->prefs.inputgad_superfocus));
 	tog =
-		settings_create_toggle (_("Show nickgad"), wid, prefs.nickgad,
+		settings_create_toggle (_("Show nickname"), right_box, prefs.nickgad,
 										settings_ptoggle_check,
 										&(sess->setup->prefs.nickgad));
-	add_tip (entry, _("Show your nick before input box"));
+	add_tip (tog, _("Show nickname and op/voice icon before the input box"));
+
+	tab = settings_create_table (wid);
+	entry = settings_create_entry (_("Nickname Completion Character:"), 1, tab,
+											 prefs.nick_suffix,
+											 &sess->setup->entry_nick_suffix,
+											 0, 0, 0);
+	add_tip (entry, _("Character to append to completed nicknames"));
 
 	wid = settings_create_group (vbox, _("Output Box"));
-	tog = settings_create_toggle (_("Time Stamp All Text"), wid,
+	settings_create_tworows (wid, &left_box, &right_box);
+	tog = settings_create_toggle (_("Time Stamp All Text"), right_box,
 											prefs.timestamp, settings_ptoggle_check,
 											&(sess->setup->prefs.timestamp));
 	add_tip (tog, _("Prefix all text with the current time stamp"));
-	tog = settings_create_toggle (_("Colored Nicks"), wid,
+	tog = settings_create_toggle (_("Colored Nicks"), left_box,
 											prefs.colorednicks, settings_ptoggle_check,
 											&(sess->setup->prefs.colorednicks));
 	add_tip (tog, _("Output nicknames in different colors"));
-	tog = settings_create_toggle (_("Show invites in the active window"), wid,
+	tog = settings_create_toggle (_("Show invites in active window"), left_box,
 											prefs.show_invite_in_front_session,
 											settings_ptoggle_check,
 											&(sess->setup->
 											  prefs.show_invite_in_front_session));
 
-	wid = settings_create_group (vbox, _("Output Filtering"));
-	tog = settings_create_toggle (_("Strip MIRC Color"), wid,
+	tog = settings_create_toggle (_("Strip MIRC Color"), left_box,
 											prefs.stripcolor, settings_ptoggle_check,
 											&(sess->setup->prefs.stripcolor));
 	add_tip (tog, _("Strip MIRC color codes from text before displaying"));
-	tog = settings_create_toggle (_("Filter out BEEPs"), wid,
+	tog = settings_create_toggle (_("Filter out BEEPs"), right_box,
 											prefs.filterbeep, settings_ptoggle_check,
 											&(sess->setup->prefs.filterbeep));
 	add_tip (tog, _("Remove ^G BEEP codes from text before displaying"));
 
 	wid = settings_create_group (vbox, _("Buffer Settings"));
 	tab = settings_create_table (wid);
-	row_index = 0;
-	sprintf (buf, "%d", prefs.max_lines);
-	settings_create_entry (_("Text Buffer Size:"), sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_max_lines,
-								  _("lines (0=Unlimited)."), 0, row_index++);
+	sess->setup->entry_max_lines =
+		settings_create_numberentry (_("Text Buffer Size:"), 0, 10000, 1, tab,
+								prefs.max_lines, 0, _("lines (0=Unlimited)."));
 }
 
 static void
@@ -1019,41 +1071,34 @@ static void
 settings_page_interface_layout (struct session *sess, GtkWidget * vbox)
 {
 	GtkWidget *wid;
-	GtkWidget *tog, *menu, *omenu;
+	GtkWidget *tog, *tab;
+	static char *tabmenutext[] = 
+	{
+		N_("Bottom"),
+		N_("Top"),
+		N_("Left"),
+		N_("Right"),
+		N_("Hidden"),
+		NULL
+	};
 
 	wid = settings_create_group (vbox, _("Buttons"));
 	tog = settings_create_toggle (_("Channel Mode Buttons"), wid,
-											!prefs.nochanmodebuttons,
-											settings_pinvtoggle_check,
-											&(sess->setup->prefs.nochanmodebuttons));
+											prefs.chanmodebuttons,
+											settings_ptoggle_check,
+											&(sess->setup->prefs.chanmodebuttons));
 	add_tip (tog, _("Show the TNSIPMLK buttons"));
 	tog = settings_create_toggle (_("User List Buttons"), wid,
-											!prefs.nouserlistbuttons,
-											settings_pinvtoggle_check,
-											&(sess->setup->prefs.nouserlistbuttons));
+											prefs.userlistbuttons,
+											settings_ptoggle_check,
+											&(sess->setup->prefs.userlistbuttons));
 	add_tip (tog, _("Show the buttons below the user list"));
 
 	wid = settings_create_group (vbox, _("Tabs"));
+	tab = settings_create_table (wid);
 
-	omenu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	menu_quick_item_with_callback (settings_tabsset, _("Bottom"), menu,
-											 (void *) 0);
-	menu_quick_item_with_callback (settings_tabsset, _("Top"), menu,
-											 (void *) 1);
-	menu_quick_item_with_callback (settings_tabsset, _("Left"), menu,
-											 (void *) 2);
-	menu_quick_item_with_callback (settings_tabsset, _("Right"), menu,
-											 (void *) 3);
-	menu_quick_item_with_callback (settings_tabsset, _("Hidden"), menu,
-											 (void *) 4);
-	gtk_object_set_user_data (GTK_OBJECT (menu), sess);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), prefs.tabs_position);
-	gtk_widget_show (menu);
-	gtk_widget_set_usize (menu, 120, 0);
-	gtk_box_pack_start (GTK_BOX (wid), omenu, 0, 0, 0);
-	gtk_widget_show (omenu);
+	settings_create_optmenu (tab, _("Tabs Located at:"), tabmenutext,
+								settings_tabsset, sess, 0, prefs.tabs_position);
 
 	tog = settings_create_toggle (_("New Tabs to front"), wid,
 											prefs.newtabstofront, settings_ptoggle_check,
@@ -1082,12 +1127,6 @@ wid, prefs.windows_as_tabs, settings_ptoggle_check,
 											prefs.limitedtabhighlight, settings_ptoggle_check,
 											&(sess->setup->prefs.limitedtabhighlight));
 	add_tip (tog, _("Only highlight channel tabs for channel messages and actions"));
-
-	wid = settings_create_group (vbox, _("Other Layout Settings"));
-	tog = settings_create_toggle (_("Disable Paned Userlist"), wid,
-											prefs.nopaned, settings_ptoggle_check,
-											&(sess->setup->prefs.nopaned));
-	add_tip (tog, _("Use a fixed width user list instead of the paned one"));
 }
 
 static void
@@ -1096,30 +1135,28 @@ settings_page_interface_mainwindow (struct session *sess, GtkWidget * vbox)
 	GtkWidget *wid;
 	GtkWidget *tab;
 	gint row_index;
-	char buf[127];
 
 	wid = settings_create_group (vbox, _("Window Position"));
-	gtkutil_label_new (_("If Left and Top are set to zero, X-Chat will use "
+	gtkutil_label_new (_("If Left and Top are set to zero, X-Chat will use\n"
 								"your window manager defaults."), wid);
 	tab = settings_create_table (wid);
 	row_index = 0;
-	sprintf (buf, "%d", prefs.mainwindow_left);
-	settings_create_entry (_("Left:"), sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_mainw_left, 0, 0, row_index++);
-	sprintf (buf, "%d", prefs.mainwindow_top);
-	settings_create_entry (_("Top:"), sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_mainw_top, 0, 0, row_index++);
+	sess->setup->entry_mainw_left =
+		settings_create_numberentry (_("Left:"), 0, 40000, 1, tab,
+								prefs.mainwindow_left, row_index++, 0);
+	sess->setup->entry_mainw_top =
+		settings_create_numberentry (_("Top:"), 0, 40000, 1, tab,
+								prefs.mainwindow_top, row_index++, 0);
 
 	wid = settings_create_group (vbox, _("Window Size"));
 	tab = settings_create_table (wid);
 	row_index = 0;
-	sprintf (buf, "%d", prefs.mainwindow_width);
-	settings_create_entry (_("Width:"), sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_mainw_width, 0, 0, row_index++);
-	sprintf (buf, "%d", prefs.mainwindow_height);
-	settings_create_entry (_("Height:"), sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_mainw_height, 0, 0,
-								  row_index++);
+	sess->setup->entry_mainw_width =
+		settings_create_numberentry (_("Width:"), 106, 40000, 1, tab,
+								prefs.mainwindow_width, row_index++, 0);
+	sess->setup->entry_mainw_height =
+		settings_create_numberentry (_("Height:"), 138, 40000, 1, tab,
+								prefs.mainwindow_height, row_index++, 0);
 	settings_create_toggle (_("Show Session Tree View"), wid, prefs.treeview,
 									settings_ptoggle_check,
 									&(sess->setup->prefs.treeview));
@@ -1200,32 +1237,40 @@ settings_page_irc (struct session *sess, GtkWidget * vbox)
 	GtkWidget *wid;
 	GtkWidget *tab;
 	GtkWidget *tog;
+	GtkWidget *left_box, *right_box;
 	gint row_index;
-	char buf[16];
 
 	wid = settings_create_group (vbox, _("General"));
-	tog = settings_create_toggle (_("Raw Mode Display"), wid,
+	settings_create_tworows (wid, &left_box, &right_box);
+	tog = settings_create_toggle (_("Raw Mode Display"), left_box,
 											prefs.raw_modes, settings_ptoggle_check,
 											&(sess->setup->prefs.raw_modes));
 	add_tip (tog, _("Display raw mode changes instead of interpretations"));
-	tog = settings_create_toggle (_("Beep on Private Messages"), wid,
+	tog = settings_create_toggle (_("Beep on Private Messages"), left_box,
 											prefs.beepmsg, settings_ptoggle_check,
 											&(sess->setup->prefs.beepmsg));
 	add_tip (tog, _("Beep when a private message for you is received"));
-	tog = settings_create_toggle (_("Don't send /who #chan on join."), wid,
-											prefs.nouserhost, settings_ptoggle_check,
-											&(sess->setup->prefs.nouserhost));
-	add_tip (tog, _("Don't find user information when joining a channel."));
-	tog = settings_create_toggle (_("Perform a periodic mail check."), wid,
+	tog = settings_create_toggle (_("Send /who #chan on join."), right_box,
+											prefs.userhost, settings_ptoggle_check,
+											&(sess->setup->prefs.userhost));
+	add_tip (tog, _("Find user information when joining a channel."));
+	tog = settings_create_toggle (_("Perform a periodic mail check."), right_box,
 											prefs.mail_check, settings_ptoggle_check,
 											&(sess->setup->prefs.mail_check));
 	wid = settings_create_group (vbox, _("Your irc settings"));
 	tab = settings_create_table (wid);
 	row_index = 0;
+
 	settings_create_entry (_("Quit Message:"),
 								  sizeof (prefs.quitreason) - 1, tab,
 								  prefs.quitreason,
 								  &sess->setup->entry_quit, 0, 0, row_index++);
+	settings_create_entry (_("Part Message:"),
+								  sizeof (prefs.partreason) - 1, tab,
+								  prefs.partreason,
+								  &sess->setup->entry_part, 0, 0, row_index++);
+	gtkutil_label_new (_("These can be a filename relative to ~/.xchat/ to be\n"
+								"used as a list of random quit or part reasons."), wid);
 
 	wid = settings_create_group (vbox, _("External Programs"));
 	tab = settings_create_table (wid);
@@ -1237,13 +1282,9 @@ settings_page_irc (struct session *sess, GtkWidget * vbox)
 
 	wid = settings_create_group (vbox, _("Timing"));
 	tab = settings_create_table (wid);
-	row_index = 0;
-	sprintf (buf, "%d", prefs.recon_delay);
-	settings_create_entry (_("Auto ReConnect Delay:"),
-								  sizeof (prefs.recon_delay) - 1, tab,
-								  buf,
-								  &sess->setup->entry_recon_delay, _("seconds."), 0,
-								  row_index++);
+	sess->setup->entry_recon_delay =
+		settings_create_numberentry (_("Auto ReConnect Delay:"), 0, 10000, 1, tab,
+								prefs.recon_delay, 0, _("seconds."));
 }
 
 static void
@@ -1284,9 +1325,15 @@ settings_page_proxy (struct session *sess, GtkWidget * vbox)
 {
 	GtkWidget *wid;
 	GtkWidget *tab;
-	GtkWidget *menu;
 	gint row_index;
-	char buf[16];
+	static char *proxymenutext[] =
+	{
+		N_("(Disabled)"),
+		N_("Wingate"),
+		N_("Socks4"),
+		N_("Socks5"),
+		NULL
+	};
 
 	wid = settings_create_group (vbox, _("General"));
 	tab = settings_create_table (wid);
@@ -1294,34 +1341,11 @@ settings_page_proxy (struct session *sess, GtkWidget * vbox)
 	settings_create_entry (_("Proxy Server Hostname:"),
 								  sizeof (prefs.proxy_host) - 1, tab, prefs.proxy_host,
 								  &sess->setup->entry_proxy_host, 0, 0, row_index++);
-
-	sprintf (buf, "%d", prefs.proxy_port);
-	settings_create_entry (_("Proxy Server Port:"),
-								  sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_proxy_port, 0, 0, row_index++);
-
-	wid = gtk_label_new (_("Proxy Type:"));
-	gtk_table_attach (GTK_TABLE (tab), wid, 0, 1, row_index, row_index + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-	gtk_widget_show (wid);
-
-	wid = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	menu_quick_item_with_callback (settings_proxytypeset, _("(Disabled)"), menu,
-											 (void *) 0);
-	menu_quick_item_with_callback (settings_proxytypeset, _("Wingate"), menu,
-											 (void *) 1);
-	menu_quick_item_with_callback (settings_proxytypeset, _("Socks4"), menu,
-											 (void *) 2);
-	menu_quick_item_with_callback (settings_proxytypeset, _("Socks5"), menu,
-											 (void *) 3);
-	gtk_object_set_user_data (GTK_OBJECT (menu), sess);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (wid), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (wid), prefs.proxy_type);
-	gtk_widget_show (menu);
-	gtk_table_attach (GTK_TABLE (tab), wid, 2, 3, row_index, row_index + 1,
-							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-	gtk_widget_show (wid);
+	sess->setup->entry_proxy_port =
+		settings_create_numberentry (_("Proxy Server Port:"), 0, 65535, 1, tab,
+								prefs.proxy_port, row_index++, 0);
+	settings_create_optmenu (tab, _("Proxy Type:"), proxymenutext,
+								settings_proxytypeset, sess, row_index, prefs.proxy_type);
 }
 
 static void
@@ -1363,15 +1387,11 @@ settings_page_irc_highlighting (struct session *sess, GtkWidget * vbox)
 	wid = settings_create_group (vbox, _("General"));
 	tab = settings_create_table (wid);
 	row_index = 0;
-	settings_create_entry (_("Extra Words to Highlight on:"),
+	settings_create_entry (_("Words to Highlight:"),
 								  sizeof (prefs.bluestring) - 1, tab,
 								  prefs.bluestring,
 								  &sess->setup->entry_bluestring,
 								  _("(separate with commas)"), 0, row_index++);
-	settings_create_color_box (_("Hilighted Nick Color:"), tab,
-										prefs.bt_color,
-										&sess->setup->bt_color,
-										settings_bt_color_clicked, sess, row_index++);
 }
 
 static void
@@ -1386,7 +1406,11 @@ settings_page_irc_logging (struct session *sess, GtkWidget * vbox)
 											prefs.logging, settings_ptoggle_check,
 											&(sess->setup->prefs.logging));
 	add_tip (tog, _("Enable logging conversations to disk"));
+	tog = settings_create_toggle (_("Always timestamp logs"), wid,
+											prefs.timestamp_logs, settings_ptoggle_check,
+											&(sess->setup->prefs.timestamp_logs));
 
+	add_tip (tog, _("Always apply timestamp to disk logs"));
 	settings_create_entry (_("Log name mask:"),
 								  sizeof (prefs.logmask) - 1, tab,
 								  prefs.logmask, &sess->setup->logmask_entry,
@@ -1400,7 +1424,6 @@ settings_page_irc_notification (struct session *sess, GtkWidget * vbox)
 	GtkWidget *tab;
 	GtkWidget *tog;
 	gint row_index;
-	char buf[127];
 
 	wid = settings_create_group (vbox, _("Generic"));
 	tog = settings_create_toggle (_("Show notifies in active window"), wid,
@@ -1428,13 +1451,9 @@ settings_page_irc_notification (struct session *sess, GtkWidget * vbox)
 
 	wid = settings_create_group (vbox, _("Notification Timeouts"));
 	tab = settings_create_table (wid);
-	row_index = 0;
-	sprintf (buf, "%d", prefs.notify_timeout);
-	settings_create_entry (_("Notify Check Interval:"),
-								  sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_timeout,
-								  _("seconds. (15=Normal, 0=Disable)"), 0,
-								  row_index++);
+	sess->setup->entry_timeout =
+		settings_create_numberentry (_("Notify Check Interval:"), 0, 960, 1, tab,
+								prefs.notify_timeout, 0, _("seconds (0=Disable)."));
 }
 
 #ifdef USE_TRANS
@@ -1466,22 +1485,22 @@ settings_page_dcc (struct session *sess, GtkWidget * vbox)
 
 	wid = settings_create_group (vbox, _("General"));
 	tog = settings_create_toggle (_("Auto Open DCC Send Window"), wid,
-											!prefs.noautoopendccsendwindow,
-											settings_pinvtoggle_check,
+											prefs.autoopendccsendwindow,
+											settings_ptoggle_check,
 											&(sess->setup->prefs.
-											  noautoopendccsendwindow));
+											  autoopendccsendwindow));
 	add_tip (tog, _("Automatically open DCC Send Window"));
 	tog = settings_create_toggle (_("Auto Open DCC Recv Window"), wid,
-											!prefs.noautoopendccrecvwindow,
-											settings_pinvtoggle_check,
+											prefs.autoopendccrecvwindow,
+											settings_ptoggle_check,
 											&(sess->setup->prefs.
-											  noautoopendccrecvwindow));
+											  autoopendccrecvwindow));
 	add_tip (tog, _("Automatically open DCC Recv Window"));
 	tog = settings_create_toggle (_("Auto Open DCC Chat Window"), wid,
-											!prefs.noautoopendccchatwindow,
-											settings_pinvtoggle_check,
+											prefs.autoopendccchatwindow,
+											settings_ptoggle_check,
 											&(sess->setup->prefs.
-											  noautoopendccchatwindow));
+											  autoopendccchatwindow));
 	add_tip (tog, _("Automatically open DCC Chat Window"));
 	tog = settings_create_toggle (_("Resume on Auto Accept"), wid,
 											prefs.autoresume,
@@ -1502,16 +1521,13 @@ settings_page_dcc_filetransfer (struct session *sess, GtkWidget * vbox)
 	wid = settings_create_group (vbox, _("Timeouts"));
 	tab = settings_create_table (wid);
 	row_index = 0;
-	sprintf (buf, "%d", prefs.dcctimeout);
-	settings_create_entry (_("DCC Offers Timeout:"),
-								  sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_dcctimeout, _("seconds."), 0,
-								  row_index++);
-	sprintf (buf, "%d", prefs.dccstalltimeout);
-	settings_create_entry (_("DCC Stall Timeout:"),
-								  sizeof (buf) - 1, tab, buf,
-								  &sess->setup->entry_dccstalltimeout,
-								  _("seconds."), 0, row_index++);
+
+	sess->setup->entry_dcctimeout =
+		settings_create_numberentry (_("DCC Offers Timeout:"), 0, 10000, 1, tab,
+								prefs.dcctimeout, row_index++, _("seconds."));
+	sess->setup->entry_dccstalltimeout =
+		settings_create_numberentry (_("DCC Stall Timeout:"), 0, 10000, 1, tab,
+								prefs.dccstalltimeout, row_index++, _("seconds."));
 
 	wid = settings_create_group (vbox, _("Received files"));
 	tab = settings_create_table (wid);
@@ -1536,23 +1552,16 @@ settings_page_dcc_filetransfer (struct session *sess, GtkWidget * vbox)
 											&(sess->setup->prefs.fastdccsend));
 	add_tip (tog, _("Don\'t wait for ACKs to send more data"));
 
-	sprintf (buf, "%d", prefs.first_dcc_send_port);
-	settings_create_entry (_("First DCC Send Port:"),
-			sizeof (buf) - 1, tab, buf,
-			&sess->setup->entry_dcc_send_port_first,
-			_("(0=Disabled)"), 0, row_index++);
+	sess->setup->entry_dcc_send_port_first =
+		settings_create_numberentry (_("First DCC Send Port:"), 0, 10000, 1, tab,
+								prefs.first_dcc_send_port, row_index++, _("(0=Disabled)"));
+	sess->setup->entry_dcc_send_port_last =
+		settings_create_numberentry (_("Last DCC Send Port:"), 0, 10000, 1, tab,
+								prefs.last_dcc_send_port, row_index++, 0);
 
-	sprintf (buf, "%d", prefs.last_dcc_send_port);
-	settings_create_entry (_("Last DCC Send Port:"),
-			sizeof (buf) - 1, tab, buf,
-			&sess->setup->entry_dcc_send_port_last,
-			0, 0, row_index++);
-
-	sprintf (buf, "%d", prefs.dcc_blocksize);
-	settings_create_entry (_("Send Block Size:"),
-								  5, tab, buf,
-								  &sess->setup->entry_dcc_blocksize,
-								  _("(1024=Normal)"), 0, row_index++);
+	sess->setup->entry_dcc_blocksize =
+		settings_create_numberentry (_("Send Block Size:"), 0, 10000, 1, tab,
+								prefs.dcc_blocksize, row_index++, _("(1024=Normal)"));
 }
 
 static void
