@@ -33,7 +33,6 @@
 
 extern GdkColor colors[];
 extern GdkFont *font_normal;
-extern GdkFont *dialog_font_normal;
 extern GdkPixmap *channelwin_pix;
 extern struct xchatprefs prefs;
 
@@ -41,8 +40,7 @@ extern int text_event (int i);
 extern void pevent_make_pntevts ();
 extern int pevent_load (char *filename);
 extern void pevent_dialog_save (char *fn);
-extern int timecat (char *buf);
-extern void check_special_chars (char *);
+extern void check_special_chars (char *, int perc_ascii);
 extern int buf_get_line (char *, char **, int *, int len);
 extern void show_and_unfocus (GtkWidget *);
 #ifdef USE_PERL
@@ -69,13 +67,22 @@ static GtkWidget *pevent_dialog = NULL, *pevent_dialog_twid,
 
 int pevt_build_string (char *input, char **output, int *max_arg);
 
+static void
+timecpy (char *buf)
+{
+	time_t timval = time (0);
+	char *tim = ctime (&timval) + 10;
+	memcpy (buf, tim, 9);
+	buf[0] = '[';
+	buf[9] = ']';
+	buf[10] = ' ';
+}
 
 static void
-PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len)
+PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 {
 	char *tab, *new_text;
-	int leftlen, indent = prefs.indent_nicks;
-	session *sess;
+	int leftlen;
 
 	if (len == -1)
 		len = strlen (text);
@@ -83,20 +90,12 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len)
 	if (len == 0)
 		len = 1;
 
-	sess = gtk_object_get_user_data (GTK_OBJECT (textwidget));
-	if (sess)
-	{
-		if (sess->is_dialog)
-			indent = prefs.dialog_indent_nicks;
-	}
-
 	if (!indent)
 	{
 		if (prefs.timestamp)
 		{
 			new_text = malloc (len + 12);
-			*new_text = 0;
-			timecat (new_text);
+			timecpy (new_text);
 			memcpy (new_text + 11, text, len);
 			gtk_xtext_append (GTK_XTEXT (textwidget), new_text, len + 11);
 			free (new_text);
@@ -116,7 +115,7 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len)
 }
 
 void
-PrintTextRaw (GtkWidget * textwidget, unsigned char *text)
+PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
 {
 	char *cr;
 
@@ -129,20 +128,20 @@ PrintTextRaw (GtkWidget * textwidget, unsigned char *text)
 		while (1)
 		{
 			PrintTextLine (textwidget, text,
-								(unsigned long) cr - (unsigned long) text);
+								(unsigned long) cr - (unsigned long) text, indent);
 			text = cr + 1;
 			if (*text == 0)
 				break;
 			cr = strchr (text, '\n');
 			if (!cr)
 			{
-				PrintTextLine (textwidget, text, -1);
+				PrintTextLine (textwidget, text, -1, indent);
 				break;
 			}
 		}
 	} else
 	{
-		PrintTextLine (textwidget, text, -1);
+		PrintTextLine (textwidget, text, -1, indent);
 	}
 }
 
@@ -220,9 +219,9 @@ pevent_dialog_update (GtkWidget * wid, GtkWidget * twid)
 	memcpy (out, text, len + 1);
 	out[len] = '\n';
 	out[len + 1] = 0;
-	check_special_chars (out);
+	check_special_chars (out, TRUE);
 
-	PrintTextRaw (twid, out);
+	PrintTextRaw (twid, out, 0);
 	free (out);
 }
 
@@ -310,8 +309,11 @@ pevent_dialog_fill (GtkWidget * list)
 static void
 pevent_save_req_cb (void *arg1, void *arg2, char *file)
 {
-	pevent_dialog_save (file);
-	free (file);
+	if (file)
+	{
+		pevent_dialog_save (file);
+		free (file);
+	}
 }
 
 static void
@@ -329,11 +331,14 @@ pevent_save_cb (GtkWidget * wid, void *data)
 static void
 pevent_load_req_cb (void *arg1, void *arg2, char *file)
 {
-	pevent_load (file);
-	pevent_make_pntevts ();
-	pevent_dialog_fill (pevent_dialog_list);
-	pevent_dialog_select (pevent_dialog_list, -1, -1, NULL, NULL);
-	free (file);
+	if (file)
+	{
+		pevent_load (file);
+		pevent_make_pntevts ();
+		pevent_dialog_fill (pevent_dialog_list);
+		pevent_dialog_select (pevent_dialog_list, -1, -1, NULL, NULL);
+		free (file);
+	}
 }
 
 static void
@@ -366,33 +371,25 @@ pevent_test_cb (GtkWidget * wid, GtkWidget * twid)
 		memcpy (out, text, len + 1);
 		out[len] = '\n';
 		out[len + 1] = 0;
-		check_special_chars (out);
+		check_special_chars (out, TRUE);
 
-		PrintTextRaw (twid, out);
+		PrintTextRaw (twid, out, 0);
 		free (out);
 	}
 }
 
 /* from settings.c - but its not there anymore */
-void
+static void
 gui_entry (char *label, int max, GtkWidget * box, GtkWidget ** entry)
 {
 	GtkWidget *wid, *hbox;
 
-	hbox = gtk_hbox_new (0, 5);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+	hbox = gtk_hbox_new (0, 0);
 	gtk_widget_show (hbox);
 
-	wid = gtk_label_new (label);
-	gtk_widget_set_usize (wid, 132, 0);
-	gtk_misc_set_alignment (GTK_MISC (wid), 1, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
-	gtk_widget_show (wid);
+	gtkutil_label_new (label, hbox);
 
-	wid = gtk_entry_new_with_max_length (max);
-	if (entry)
-		*entry = wid;
-	gtk_entry_set_text (GTK_ENTRY (wid), "");
+	*entry = wid = gtk_entry_new_with_max_length (max);
 	gtk_container_add (GTK_CONTAINER (hbox), wid);
 	gtk_widget_show (wid);
 
