@@ -19,6 +19,10 @@
 #include <stdio.h>
 #include <signal.h>
 #include <string.h>
+#include "../../config.h"
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include <stdlib.h>
 #include "../common/xchat.h"
 #include "fe-gtk.h"
@@ -36,7 +40,7 @@ extern void menu_urlmenu (struct session *sess, GdkEventButton * event,
 
 static GtkWidget *urlgrabberwindow = 0;
 static GtkWidget *urlgrabberlist;
-void url_addurlgui (char *urltext);
+static void url_addurlgui (char *urltext);
 
 
 static int
@@ -84,17 +88,14 @@ url_autosave (void)
 }
 
 static void
-url_save_callback (GtkWidget * b, GtkWidget * d)
+url_save_callback (void *arg1, void *arg2, char *file)
 {
 	FILE *fd;
 	GSList *list;
-	char *f;
 
-	f = gtk_file_selection_get_filename (GTK_FILE_SELECTION (d));
-
-	if (f)
+	if (file)
 	{
-		fd = fopen (f, "w");
+		fd = fopen (file, "w");
 		if (fd == NULL)
 			return;
 		list = url_list;
@@ -104,30 +105,15 @@ url_save_callback (GtkWidget * b, GtkWidget * d)
 			list = list->next;
 		}
 		fclose (fd);
+		free (file);
 	}
-	gtk_widget_destroy (d);
-}
-
-static void
-url_save_cancel (GtkWidget * b, GtkWidget * d)
-{
-	gtk_widget_destroy (d);
 }
 
 static void
 url_button_save (void)
 {
-	GtkWidget *d;
-
-	d = gtk_file_selection_new ("Select a file to save to");
-
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (d)->cancel_button),
-							  "clicked", (GtkSignalFunc) url_save_cancel,
-							  (gpointer) d);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (d)->ok_button),
-							  "clicked", (GtkSignalFunc) url_save_callback,
-							  (gpointer) d);
-	gtk_widget_show (d);
+	gtkutil_file_req ("Select a file to save to",
+							url_save_callback, 0, 0, TRUE);
 }
 
 static void
@@ -199,7 +185,7 @@ url_opengui ()
 	}
 }
 
-void
+static void
 url_addurlgui (char *urltext)
 {
 	if (urlgrabberwindow)
