@@ -5,17 +5,6 @@
 #include <gtk/gtkadjustment.h>
 #include <gtk/gtkwidget.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
-#ifdef USE_MITSHM
-
-#include <sys/ipc.h>
-#include <sys/shm.h>
-#include <X11/extensions/XShm.h>
-
-#endif
-
 /*#define GTK_XTEXT(obj)          GTK_CHECK_CAST (obj, gtk_xtext_get_type (), GtkXText)*/
 #define GTK_XTEXT(obj) ((GtkXText*)obj)
 #define GTK_XTEXT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_xtext_get_type (), GtkXTextClass)
@@ -56,10 +45,13 @@ struct _GtkXText
 	GtkWidget widget;
 
 	GtkAdjustment *adj;
-	gfloat old_value;				  /* last known adj->value */
-	GdkPixmap *pixmap;			  /* 0 = use palette[19] */
+	gfloat old_value;					/* last known adj->value */
+	GdkPixmap *pixmap;				/* 0 = use palette[19] */
+	GdkDrawable *draw_buf;			/* points to ->window */
 	GdkCursor *hand_cursor;
 
+	int pixel_offset;					/* amount of pixels the top line is chopped by */
+
 	int last_win_x;
 	int last_win_y;
 	int last_win_h;
@@ -71,6 +63,8 @@ struct _GtkXText
 
 	GdkGC *bgc;						  /* backing pixmap */
 	GdkGC *fgc;						  /* text foreground color */
+	GdkGC *light_gc;				  /* sep bar */
+	GdkGC *dark_gc;
 	gulong palette[20];
 
 	textentry *text_first;
@@ -80,20 +74,6 @@ struct _GtkXText
 	gint add_io_tag;				  /* "" when adding new text */
 	gint scroll_tag;				  /* marking-scroll timeout */
 
-	GC xfgc;							  /* this stuff is repeated, but its the X11 pointers */
-	GC xbgc;
-	Drawable drawable;
-	Drawable draw_buf;
-	Display *display;
-	XFontStruct *xfont;
-
-	Pixmap tmp_pix;				  /* for double buffering */
-
-#ifdef USE_MITSHM
-	XShmSegmentInfo shminfo;
-	XImage *img;
-#endif
-
 	GdkFont *font;
 	int fontsize;
 	int fonttype;
@@ -115,15 +95,11 @@ struct _GtkXText
 	int last_offset_start;
 	int last_offset_end;
 
-	textentry *old_ent_start;
-	textentry *old_ent_end;
-
 	int num_lines;
 	int max_lines;
 
-	int last_subline;
-	int last_line;
-	textentry *last_ent;
+	int pagetop_subline;
+	textentry *pagetop_ent;			/* what's at xtext->adj->value */
 
 	int col_fore;
 	int col_back;
@@ -146,7 +122,7 @@ struct _GtkXText
 
 	unsigned char scratch_buffer[4096];
 
-	unsigned int double_buffer:1;
+	unsigned int fixed_width_font:1;
 	unsigned int auto_indent:1;
 	unsigned int moving_separator:1;
 	unsigned int time_stamp:1;
@@ -167,6 +143,10 @@ struct _GtkXText
 	unsigned int wordwrap:1;
 	unsigned int dont_render:1;
 	unsigned int cursor_hand:1;
+	unsigned int skip_fills:1;
+	unsigned int skip_border_fills:1;
+	unsigned int skip_stamp:1;
+	unsigned int do_underline_fills_only:1;
 };
 
 struct _GtkXTextClass
@@ -185,12 +165,12 @@ void gtk_xtext_set_font (GtkXText * xtext, GdkFont * font, char *name);
 void gtk_xtext_set_background (GtkXText * xtext, GdkPixmap * pixmap,
 										 int trans, int shaded);
 void gtk_xtext_set_palette (GtkXText * xtext, GdkColor palette[]);
-void gtk_xtext_remove_lines (GtkXText * xtext, int lines, int refresh);
+void gtk_xtext_clear (GtkXText * xtext);
 gchar *gtk_xtext_get_chars (GtkXText * xtext);
 void gtk_xtext_refresh (GtkXText * xtext, int do_trans);
 void gtk_xtext_thaw (GtkXText * xtext);
 void gtk_xtext_freeze (GtkXText * xtext);
 void *gtk_xtext_search (GtkXText * xtext, char *text, void *start);
-char *gtk_xtext_strip_color (unsigned char *text, int len, char *outbuf, int *newlen);
+char *gtk_xtext_strip_color (unsigned char *text, int len, char *outbuf, int *newlen, int fonttype);
 
 #endif
