@@ -34,7 +34,6 @@ extern char *get_cpu_str (int color);
 extern int handle_command (char *cmd, struct session *sess, int history,
 									int nocommand);
 extern int get_mhz (void);
-extern int timecat (char *buf);
 extern void kill_session_callback (struct session *sess);
 
 extern struct xchatprefs prefs;
@@ -92,7 +91,6 @@ void
 fe_new_window (struct session *sess)
 {
 	char buf[512];
-	time_t tim = time (0);
 
 	sess->gui = malloc (4);
 
@@ -105,24 +103,22 @@ fe_new_window (struct session *sess)
 		return;
 	done_intro = 1;
 
-	snprintf (buf, sizeof buf,
-				"\n\n"
-				" \037\0031                                                  \017\n"
-				" \037\0035,2                xchat "VERSION" dEVEL!              \017\n"
-				"\n"
-				"\0033  GLIB Version\0032...\0035:\017 %d.%d.%d\n"
-				"\0033  System Type\0032....\0035:\017 %s\n"
-				"\0033  Launch Time\0032....\0035:\017 %.19s\n"
-				"\0033  Binary Date\0032....\0035:\017 " __DATE__ "\n"
-				"\0033  Current Pid\0032....\0035:\017 %d\n"
-				"\0033  Unix Username\0032..\0035:\017 %s\n"
-				"\0033  Your UserId\0032....\0035:\017 %d\n",
-				 glib_major_version, glib_minor_version, glib_micro_version,
-				 get_cpu_str (1), ctime (&tim), getpid (), g_get_user_name (),
-				 getuid ());
+	snprintf (buf, sizeof (buf),
+   	"\n"
+		"\00314-\00315-\00301-\00310-\00311 xchat "VERSION""
+		" \00310-\00301-\00315-\00314-\017\n"
+		"\n"
+		"\00315[ \0036 system  \00315] \017 %s\017\n"
+		"\00315[ \0036  lib    \00315] \017 GLIB %d.%d.%d\n"
+		"\00315[ \0036nickname \00315] \017 %s\n"
+		"\00315[ \0036compiled \00315] \017 "__DATE__"\n",
+				get_cpu_str (1),
+				glib_major_version, glib_minor_version, glib_micro_version,
+				prefs.nick1
+				);
 	fe_print_text (sess, buf);
 
-	strcpy (buf, "\0033  Features\0032.......\0035: \017");
+	strcpy (buf, "\00315[ \0036features \00315] \017 ");
 #ifdef USE_PERL
 	strcat (buf, "Perl ");
 #endif
@@ -149,10 +145,24 @@ fe_new_window (struct session *sess)
 #endif
 	strcat (buf, "\n\n");
 	fe_print_text (sess, buf);
+	fe_print_text (sess, "\n\0033*\00315 Type \017/help\00315 for a list of commands\017\n\n");
 	fflush (stdout);
 	fflush (stdin);
 }
 
+static int
+timecat (char *buf)
+{
+	time_t timval = time (0);
+	char *tim = ctime (&timval) + 10;
+	tim[0] = '[';
+	tim[9] = ']';
+	tim[10] = ' ';
+	tim[11] = 0;
+	strcat (buf, tim);
+	return 11;
+}
+
 /*                       0  1  2  3  4  5  6  7   8   9   10 11  12  13  14 15 */
 static int colconv[] = { 0, 7, 4, 2, 1, 3, 5, 11, 13, 12, 6, 16, 14, 15, 10, 7 };
 
@@ -475,11 +485,21 @@ fe_main (void)
 		if (FD_ISSET (STDIN_FILENO, &rd))
 			read_stdin ();
 
+		/* set all checked flags to false */
 		list = se_list;
 		while (list)
 		{
 			se = (socketevent *) list->data;
+			se->checked = 0;
 			list = list->next;
+		}
+
+		/* check all the socket callbacks */
+		list = se_list;
+		while (list)
+		{
+			se = (socketevent *) list->data;
+			se->checked = 1;
 			if (se->rread && FD_ISSET (se->sok, &rd))
 			{
 				se->callback (se->userdata, se->sok);
@@ -490,6 +510,18 @@ fe_main (void)
 			{
 				se->callback (se->userdata, se->sok);
 			}
+			list = se_list;
+			if (list)
+			{
+				se = (socketevent *) list->data;
+				while (se->checked)
+				{
+					list = list->next;
+					if (!list)
+						break;
+					se = (socketevent *) list->data;
+				}
+			}
 		}
 
 		/* now check our list of timeout events, some might need to be called! */
@@ -718,13 +750,15 @@ void
 fe_userlist_hide (session * sess)
 {
 }
-char *
-fe_buffer_get (session * sess)
+void
+fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
 {
-	return NULL;
 }
-
 void
 fe_set_lag (server * serv, int lag)
 {
 }
+void
+fe_set_away (server *serv)
+{
+}
